h1.logo a {
  font-family: "Raleway";
  font-weight: 500;
  text-decoration: none;
  border-bottom: none;
}

a.headerlink {
  color: rgba(0, 0, 0, 0.1);
}

div.sphinxsidebarwrapper p.blurb {
  font-family: "Raleway", sans-serif;
}

h1, h2, h3, h4, h5, h6 {
  font-family: "Raleway", sans-serif;
  font-weight: 500;
  color: #555;
}

div.sphinxsidebar li.toctree-l1 {
  font-family: "Raleway", sans-serif;
  font-weight: 500;
}

body {
  font-family: "Charis SIL", "Noto Serif", serif;
  font-size: 17px;
}

ul ul {
  padding-left: 1.5rem;
}

code {
  vertical-align: baseline;
}

pre, .pre {
  font-family: "Code new roman", "Ubuntu Mono", monospace;
  font-size: 1rem;
}

.search-btn {
  padding: 0 1rem;
  line-height: normal;
  align-self: stretch;
}
