/*
(The MIT License)

Copyright (c) 2013 Gary Hepting <ghepting@gmail.com>

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the 'Software'), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
the Software, and to permit persons to whom the Software is furnished to do so,
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/
*,:after,:before{-moz-box-sizing:border-box;box-sizing:border-box}html,body{-webkit-text-size-adjust:none}html,body,div,span,header,footer,aside,nav,menu,main,section,article,details,h1,h2,h3,h4,h5,h6,p,ol,ul,li,dl,dt,dd,em,small,big,strong,b,u,i,sub,sup,del,strike,q,blockquote,cite,time,address,code,pre,a,img,center,hr,form,fieldset,legend,label,input,textarea,button,table,caption,tbody,thead,tfoot,tr,th,td,iframe,object,canvas,audio,video,embed{margin:0;padding:0;border:0;font:inherit;vertical-align:baseline}header,footer,aside,menu,nav,section,article,details{display:block}table{border-collapse:collapse;border-spacing:0}script{display:none!important}html,body{width:100%!important;min-height:100%;background:#fff}html{width:100%;height:100%}hr{border:0;border-bottom:1px solid #ddd;margin:10px auto;clear:both}img,a img{max-width:100%;border:0}.clear,.clearfix,ul.button-group:after,ol.button-group:after,.nav ul,.tabs>div,[role~=tabpanel]{clear:both}.clear:after,.clearfix:after,ul.button-group:after,ol.button-group:after,.nav ul:after,.tabs>div:after,[role~=tabpanel]:after{content:" ";display:block;height:0;font-size:0;clear:both;visibility:hidden}.padded,.box,.tabs>div,[role~=tabpanel]{padding:10px}.half-padded{padding:5px}.double-padded{padding:20px}.triple-padded{padding:30px}.pad-top{padding-top:10px}.half-pad-top{padding-top:5px}.double-pad-top{padding-top:20px}.triple-pad-top{padding-top:30px}.pad-right{padding-right:10px}.half-pad-right{padding-right:5px}.double-pad-right{padding-right:20px}.triple-pad-right{padding-right:30px}.pad-bottom{padding-bottom:10px}.half-pad-bottom{padding-bottom:5px}.double-pad-bottom{padding-bottom:20px}.triple-pad-bottom{padding-bottom:30px}.pad-left{padding-left:10px}.half-pad-left{padding-left:5px}.double-pad-left{padding-left:20px}.triple-pad-left{padding-left:30px}.gapped{margin:10px}.half-gapped{margin:5px}.double-gapped{margin:20px}.triple-gapped{margin:30px}.gap-top{margin-top:10px}.half-gap-top{margin-top:5px}.double-gap-top{margin-top:20px}.triple-gap-top{margin-top:30px}.gap-right{margin-right:10px}.half-gap-right{margin-right:5px}.double-gap-right{margin-right:20px}.triple-gap-right{margin-right:30px}.gap-bottom{margin-bottom:10px}.half-gap-bottom{margin-bottom:5px}.double-gap-bottom{margin-bottom:20px}.triple-gap-bottom{margin-bottom:30px}.gap-left,.dropdown>a:after,.dropdown>button:after,.dropdown .button:after{margin-left:10px}.half-gap-left{margin-left:5px}.double-gap-left{margin-left:20px}.triple-gap-left{margin-left:30px}.offset{margin:-10px}.half-offset{margin:-5px}.double-offset{margin:-20px}.triple-offset{margin:-30px}.offset-top{margin-top:-10px}.half-offset-top{margin-top:-5px}.double-offset-top{margin-top:-20px}.triple-offset-top{margin-top:-30px}.offset-right{margin-right:-10px}.half-offset-right{margin-right:-5px}.double-offset-right{margin-right:-20px}.triple-offset-right{margin-right:-30px}.offset-bottom{margin-bottom:-10px}.half-offset-bottom{margin-bottom:-5px}.double-offset-bottom{margin-bottom:-20px}.triple-offset-bottom{margin-bottom:-30px}.offset-left{margin-left:-10px}.half-offset-left{margin-left:-5px}.double-offset-left{margin-left:-20px}.triple-offset-left{margin-left:-30px}.stick,.fixed{position:fixed}.stick.top,.fixed.top{position:fixed;top:0}.stick.right,.fixed.right{position:fixed;right:0}.stick.bottom,.fixed.bottom{position:fixed;bottom:0}.stick.left,.fixed.left{position:fixed;left:0}.stick.middle,.fixed.middle{position:fixed;top:50%}.stick.center,.fixed.center{position:fixed;left:50%}.absolute,.tabs:not(.accordion).vertical>ul{position:absolute}.absolute.top,.tabs:not(.accordion).vertical>ul{position:absolute;top:0}.absolute.right{position:absolute;right:0}.absolute.bottom,.tabs:not(.accordion).vertical>ul{position:absolute;bottom:0}.absolute.left,.tabs:not(.accordion).vertical>ul{position:absolute;left:0}.absolute.middle{position:absolute;top:50%}.absolute.center{position:absolute;left:50%}.relative,sup,sub,blockquote,.box,.message,[role~=alert],[role~=status],.nav,[role~=navigation],.nav li,.tabs:not(.accordion){position:relative}.static{position:static}.inline{display:inline}.inline-block,a:before,a:after,code,button,input[type=submit],input[type=button],input[type=reset],.button,a.button,a.button:visited,[role~=button],.dropdown,.dropdown>a:after,.dropdown>button:after,.dropdown .button:after,input[type=checkbox],input[type=radio],.nav a,.nav a:visited,.nav button,.tabs>ul li,[role~=tab]{display:inline-block;width:auto}.block,button.block,input[type=submit].block,input[type=button].block,input[type=reset].block,.button.block,a.button.block,a.button.block:visited,ul.button-group li a.block,ul.button-group li button.block,ol.button-group li a.block,ol.button-group li button.block,[role~=button].block,.dropdown ul li,.dropdown ul li a,.dropdown ul li button,ul.button-group:after,ol.button-group:after,ul.button-group li,ol.button-group li,ul.button-group li a,ol.button-group li a,ul.button-group li button,ol.button-group li button,.dropdown.on ul,.dropdown.active ul,.dropdown[aria-selected=true] ul,.dropdown[aria-pressed=true] ul,.nav li,.tabs.vertical>ul li,.tabs.accordion>ul li{display:block}.hidden{display:none!important}.small,sup,sub,small{font-size:.8em}.medium{font-size:1em}big,.large{font-size:1.5em}em,q{font-style:italic}b,q,blockquote,strong,.nav a,.nav a:visited,.nav button{font-weight:900}html,body,.unstyled,ul.checklist,ul.list,ol.list,ul.unstyled,ol.unstyled,ul.button-group,ol.button-group,.nav ul,.tabs>ul,[role~=tablist],.zero,input[type=checkbox],input[type=radio]{margin:0;padding:0}.pull-right,.nav button.hamburger,.nav li[role=menu]>a:after,.nav li[role=menu]>button:after{float:right}.pull-left,.nav li{float:left}.pull-none,.nav.vertical li{float:none}.bordered{border:1px solid #ddd}.border-top{border-top:1px solid #ddd}.border-right{border-right:1px solid #ddd}.border-bottom{border-bottom:1px solid #ddd}.border-left{border-left:1px solid #ddd}.align-top{display:table-cell;vertical-align:top}.align-right{text-align:right}.align-bottom{display:table-cell;vertical-align:bottom}.align-left,.nav li ul a,.nav li ul button{text-align:left}.align-center,.nav a,.nav a:visited,.nav button{text-align:center}.justify{text-align:justify}.unstyled,ul.checklist,ul.list,ol.list,ul.unstyled,ol.unstyled,ul.button-group,ol.button-group,.nav ul,.tabs>ul,[role~=tablist]{list-style:none}.truncate,.js .responsive:not(table){overflow:hidden;white-space:nowrap;text-overflow:ellipsis}.center{display:block;margin:0 auto}.square{-o-border-radius:0;border-radius:0}.round{-o-border-radius:9999em;border-radius:9999em}.rotate-90{display:inline-block;width:auto;-webkit-transform:rotate(90deg);transform:rotate(90deg)}.rotate-90-ctr{display:inline-block;width:auto;-webkit-transform:rotate(-90deg);transform:rotate(-90deg)}.rotate-180{display:inline-block;width:auto;-webkit-transform:rotate(180deg);transform:rotate(180deg)}.rotate-180-ctr{display:inline-block;width:auto;-webkit-transform:rotate(-180deg);transform:rotate(-180deg)}.rotate-270{display:inline-block;width:auto;-webkit-transform:rotate(270deg);transform:rotate(270deg)}.rotate-270-ctr{display:inline-block;width:auto;-webkit-transform:rotate(-270deg);transform:rotate(-270deg)}.rotate-360{display:inline-block;width:auto;-webkit-transform:rotate(360deg);transform:rotate(360deg)}.rotate-360-ctr{display:inline-block;width:auto;-webkit-transform:rotate(-360deg);transform:rotate(-360deg)}.transition{-webkit-transition:all 1s;transition:all 1s}.no-transition{-webkit-transition:none!important;transition:none!important}.container{max-width:1200px;margin:0 auto}.row{clear:both;width:100%}.row:after{content:" ";display:block;height:0;font-size:0;clear:both;visibility:hidden}.column,.whole,.wholes,.whole.one,.wholes.one,.half,.halves,.half.one,.halves.one,.half.two,.halves.two,.third,.thirds,.third.one,.thirds.one,.third.two,.thirds.two,.third.three,.thirds.three,.fourth,.fourths,.fourth.one,.fourths.one,.fourth.two,.fourths.two,.fourth.three,.fourths.three,.fourth.four,.fourths.four,.fifth,.fifths,.fifth.one,.fifths.one,.fifth.two,.fifths.two,.fifth.three,.fifths.three,.fifth.four,.fifths.four,.fifth.five,.fifths.five,.sixth,.sixths,.sixth.one,.sixths.one,.sixth.two,.sixths.two,.sixth.three,.sixths.three,.sixth.four,.sixths.four,.sixth.five,.sixths.five,.sixth.six,.sixths.six,.seventh,.sevenths,.seventh.one,.sevenths.one,.seventh.two,.sevenths.two,.seventh.three,.sevenths.three,.seventh.four,.sevenths.four,.seventh.five,.sevenths.five,.seventh.six,.sevenths.six,.seventh.seven,.sevenths.seven,.eighth,.eighths,.eighth.one,.eighths.one,.eighth.two,.eighths.two,.eighth.three,.eighths.three,.eighth.four,.eighths.four,.eighth.five,.eighths.five,.eighth.six,.eighths.six,.eighth.seven,.eighths.seven,.eighth.eight,.eighths.eight,.ninth,.ninths,.ninth.one,.ninths.one,.ninth.two,.ninths.two,.ninth.three,.ninths.three,.ninth.four,.ninths.four,.ninth.five,.ninths.five,.ninth.six,.ninths.six,.ninth.seven,.ninths.seven,.ninth.eight,.ninths.eight,.ninth.nine,.ninths.nine,.tenth,.tenths,.tenth.one,.tenths.one,.tenth.two,.tenths.two,.tenth.three,.tenths.three,.tenth.four,.tenths.four,.tenth.five,.tenths.five,.tenth.six,.tenths.six,.tenth.seven,.tenths.seven,.tenth.eight,.tenths.eight,.tenth.nine,.tenths.nine,.tenth.ten,.tenths.ten,.eleventh,.elevenths,.eleventh.one,.elevenths.one,.eleventh.two,.elevenths.two,.eleventh.three,.elevenths.three,.eleventh.four,.elevenths.four,.eleventh.five,.elevenths.five,.eleventh.six,.elevenths.six,.eleventh.seven,.elevenths.seven,.eleventh.eight,.elevenths.eight,.eleventh.nine,.elevenths.nine,.eleventh.ten,.elevenths.ten,.eleventh.eleven,.elevenths.eleven,.twelfth,.twelfths,.twelfth.one,.twelfths.one,.twelfth.two,.twelfths.two,.twelfth.three,.twelfths.three,.twelfth.four,.twelfths.four,.twelfth.five,.twelfths.five,.twelfth.six,.twelfths.six,.twelfth.seven,.twelfths.seven,.twelfth.eight,.twelfths.eight,.twelfth.nine,.twelfths.nine,.twelfth.ten,.twelfths.ten,.twelfth.eleven,.twelfths.eleven,.twelfth.twelve,.twelfths.twelve{float:left;position:relative;min-height:1px}.centered{float:none!important;margin-left:auto!important;margin-right:auto!important}.whole,.wholes{width:100%}.whole.one,.wholes.one{width:100%}.half,.halves{width:50%}.half.one,.halves.one{width:50%}.half.two,.halves.two{width:100%}.half.right-one,.halves.right-one{left:50%}.half.left-one,.halves.left-one{right:50%}.half.skip-one,.halves.skip-one{margin-left:50%}.third,.thirds{width:33.33333%}.third.one,.thirds.one{width:33.33333%}.third.two,.thirds.two{width:66.66667%}.third.three,.thirds.three{width:100%}.third.right-one,.thirds.right-one{left:33.33333%}.third.left-one,.thirds.left-one{right:33.33333%}.third.skip-one,.thirds.skip-one{margin-left:33.33333%}.third.right-two,.thirds.right-two{left:66.66667%}.third.left-two,.thirds.left-two{right:66.66667%}.third.skip-two,.thirds.skip-two{margin-left:66.66667%}.fourth,.fourths{width:25%}.fourth.one,.fourths.one{width:25%}.fourth.two,.fourths.two{width:50%}.fourth.three,.fourths.three{width:75%}.fourth.four,.fourths.four{width:100%}.fourth.right-one,.fourths.right-one{left:25%}.fourth.left-one,.fourths.left-one{right:25%}.fourth.skip-one,.fourths.skip-one{margin-left:25%}.fourth.right-two,.fourths.right-two{left:50%}.fourth.left-two,.fourths.left-two{right:50%}.fourth.skip-two,.fourths.skip-two{margin-left:50%}.fourth.right-three,.fourths.right-three{left:75%}.fourth.left-three,.fourths.left-three{right:75%}.fourth.skip-three,.fourths.skip-three{margin-left:75%}.fifth,.fifths{width:20%}.fifth.one,.fifths.one{width:20%}.fifth.two,.fifths.two{width:40%}.fifth.three,.fifths.three{width:60%}.fifth.four,.fifths.four{width:80%}.fifth.five,.fifths.five{width:100%}.fifth.right-one,.fifths.right-one{left:20%}.fifth.left-one,.fifths.left-one{right:20%}.fifth.skip-one,.fifths.skip-one{margin-left:20%}.fifth.right-two,.fifths.right-two{left:40%}.fifth.left-two,.fifths.left-two{right:40%}.fifth.skip-two,.fifths.skip-two{margin-left:40%}.fifth.right-three,.fifths.right-three{left:60%}.fifth.left-three,.fifths.left-three{right:60%}.fifth.skip-three,.fifths.skip-three{margin-left:60%}.fifth.right-four,.fifths.right-four{left:80%}.fifth.left-four,.fifths.left-four{right:80%}.fifth.skip-four,.fifths.skip-four{margin-left:80%}.sixth,.sixths{width:16.66667%}.sixth.one,.sixths.one{width:16.66667%}.sixth.two,.sixths.two{width:33.33333%}.sixth.three,.sixths.three{width:50%}.sixth.four,.sixths.four{width:66.66667%}.sixth.five,.sixths.five{width:83.33333%}.sixth.six,.sixths.six{width:100%}.sixth.right-one,.sixths.right-one{left:16.66667%}.sixth.left-one,.sixths.left-one{right:16.66667%}.sixth.skip-one,.sixths.skip-one{margin-left:16.66667%}.sixth.right-two,.sixths.right-two{left:33.33333%}.sixth.left-two,.sixths.left-two{right:33.33333%}.sixth.skip-two,.sixths.skip-two{margin-left:33.33333%}.sixth.right-three,.sixths.right-three{left:50%}.sixth.left-three,.sixths.left-three{right:50%}.sixth.skip-three,.sixths.skip-three{margin-left:50%}.sixth.right-four,.sixths.right-four{left:66.66667%}.sixth.left-four,.sixths.left-four{right:66.66667%}.sixth.skip-four,.sixths.skip-four{margin-left:66.66667%}.sixth.right-five,.sixths.right-five{left:83.33333%}.sixth.left-five,.sixths.left-five{right:83.33333%}.sixth.skip-five,.sixths.skip-five{margin-left:83.33333%}.seventh,.sevenths{width:14.28571%}.seventh.one,.sevenths.one{width:14.28571%}.seventh.two,.sevenths.two{width:28.57143%}.seventh.three,.sevenths.three{width:42.85714%}.seventh.four,.sevenths.four{width:57.14286%}.seventh.five,.sevenths.five{width:71.42857%}.seventh.six,.sevenths.six{width:85.71429%}.seventh.seven,.sevenths.seven{width:100%}.seventh.right-one,.sevenths.right-one{left:14.28571%}.seventh.left-one,.sevenths.left-one{right:14.28571%}.seventh.skip-one,.sevenths.skip-one{margin-left:14.28571%}.seventh.right-two,.sevenths.right-two{left:28.57143%}.seventh.left-two,.sevenths.left-two{right:28.57143%}.seventh.skip-two,.sevenths.skip-two{margin-left:28.57143%}.seventh.right-three,.sevenths.right-three{left:42.85714%}.seventh.left-three,.sevenths.left-three{right:42.85714%}.seventh.skip-three,.sevenths.skip-three{margin-left:42.85714%}.seventh.right-four,.sevenths.right-four{left:57.14286%}.seventh.left-four,.sevenths.left-four{right:57.14286%}.seventh.skip-four,.sevenths.skip-four{margin-left:57.14286%}.seventh.right-five,.sevenths.right-five{left:71.42857%}.seventh.left-five,.sevenths.left-five{right:71.42857%}.seventh.skip-five,.sevenths.skip-five{margin-left:71.42857%}.seventh.right-six,.sevenths.right-six{left:85.71429%}.seventh.left-six,.sevenths.left-six{right:85.71429%}.seventh.skip-six,.sevenths.skip-six{margin-left:85.71429%}.eighth,.eighths{width:12.5%}.eighth.one,.eighths.one{width:12.5%}.eighth.two,.eighths.two{width:25%}.eighth.three,.eighths.three{width:37.5%}.eighth.four,.eighths.four{width:50%}.eighth.five,.eighths.five{width:62.5%}.eighth.six,.eighths.six{width:75%}.eighth.seven,.eighths.seven{width:87.5%}.eighth.eight,.eighths.eight{width:100%}.eighth.right-one,.eighths.right-one{left:12.5%}.eighth.left-one,.eighths.left-one{right:12.5%}.eighth.skip-one,.eighths.skip-one{margin-left:12.5%}.eighth.right-two,.eighths.right-two{left:25%}.eighth.left-two,.eighths.left-two{right:25%}.eighth.skip-two,.eighths.skip-two{margin-left:25%}.eighth.right-three,.eighths.right-three{left:37.5%}.eighth.left-three,.eighths.left-three{right:37.5%}.eighth.skip-three,.eighths.skip-three{margin-left:37.5%}.eighth.right-four,.eighths.right-four{left:50%}.eighth.left-four,.eighths.left-four{right:50%}.eighth.skip-four,.eighths.skip-four{margin-left:50%}.eighth.right-five,.eighths.right-five{left:62.5%}.eighth.left-five,.eighths.left-five{right:62.5%}.eighth.skip-five,.eighths.skip-five{margin-left:62.5%}.eighth.right-six,.eighths.right-six{left:75%}.eighth.left-six,.eighths.left-six{right:75%}.eighth.skip-six,.eighths.skip-six{margin-left:75%}.eighth.right-seven,.eighths.right-seven{left:87.5%}.eighth.left-seven,.eighths.left-seven{right:87.5%}.eighth.skip-seven,.eighths.skip-seven{margin-left:87.5%}.ninth,.ninths{width:11.11111%}.ninth.one,.ninths.one{width:11.11111%}.ninth.two,.ninths.two{width:22.22222%}.ninth.three,.ninths.three{width:33.33333%}.ninth.four,.ninths.four{width:44.44444%}.ninth.five,.ninths.five{width:55.55556%}.ninth.six,.ninths.six{width:66.66667%}.ninth.seven,.ninths.seven{width:77.77778%}.ninth.eight,.ninths.eight{width:88.88889%}.ninth.nine,.ninths.nine{width:100%}.ninth.right-one,.ninths.right-one{left:11.11111%}.ninth.left-one,.ninths.left-one{right:11.11111%}.ninth.skip-one,.ninths.skip-one{margin-left:11.11111%}.ninth.right-two,.ninths.right-two{left:22.22222%}.ninth.left-two,.ninths.left-two{right:22.22222%}.ninth.skip-two,.ninths.skip-two{margin-left:22.22222%}.ninth.right-three,.ninths.right-three{left:33.33333%}.ninth.left-three,.ninths.left-three{right:33.33333%}.ninth.skip-three,.ninths.skip-three{margin-left:33.33333%}.ninth.right-four,.ninths.right-four{left:44.44444%}.ninth.left-four,.ninths.left-four{right:44.44444%}.ninth.skip-four,.ninths.skip-four{margin-left:44.44444%}.ninth.right-five,.ninths.right-five{left:55.55556%}.ninth.left-five,.ninths.left-five{right:55.55556%}.ninth.skip-five,.ninths.skip-five{margin-left:55.55556%}.ninth.right-six,.ninths.right-six{left:66.66667%}.ninth.left-six,.ninths.left-six{right:66.66667%}.ninth.skip-six,.ninths.skip-six{margin-left:66.66667%}.ninth.right-seven,.ninths.right-seven{left:77.77778%}.ninth.left-seven,.ninths.left-seven{right:77.77778%}.ninth.skip-seven,.ninths.skip-seven{margin-left:77.77778%}.ninth.right-eight,.ninths.right-eight{left:88.88889%}.ninth.left-eight,.ninths.left-eight{right:88.88889%}.ninth.skip-eight,.ninths.skip-eight{margin-left:88.88889%}.tenth,.tenths{width:10%}.tenth.one,.tenths.one{width:10%}.tenth.two,.tenths.two{width:20%}.tenth.three,.tenths.three{width:30%}.tenth.four,.tenths.four{width:40%}.tenth.five,.tenths.five{width:50%}.tenth.six,.tenths.six{width:60%}.tenth.seven,.tenths.seven{width:70%}.tenth.eight,.tenths.eight{width:80%}.tenth.nine,.tenths.nine{width:90%}.tenth.ten,.tenths.ten{width:100%}.tenth.right-one,.tenths.right-one{left:10%}.tenth.left-one,.tenths.left-one{right:10%}.tenth.skip-one,.tenths.skip-one{margin-left:10%}.tenth.right-two,.tenths.right-two{left:20%}.tenth.left-two,.tenths.left-two{right:20%}.tenth.skip-two,.tenths.skip-two{margin-left:20%}.tenth.right-three,.tenths.right-three{left:30%}.tenth.left-three,.tenths.left-three{right:30%}.tenth.skip-three,.tenths.skip-three{margin-left:30%}.tenth.right-four,.tenths.right-four{left:40%}.tenth.left-four,.tenths.left-four{right:40%}.tenth.skip-four,.tenths.skip-four{margin-left:40%}.tenth.right-five,.tenths.right-five{left:50%}.tenth.left-five,.tenths.left-five{right:50%}.tenth.skip-five,.tenths.skip-five{margin-left:50%}.tenth.right-six,.tenths.right-six{left:60%}.tenth.left-six,.tenths.left-six{right:60%}.tenth.skip-six,.tenths.skip-six{margin-left:60%}.tenth.right-seven,.tenths.right-seven{left:70%}.tenth.left-seven,.tenths.left-seven{right:70%}.tenth.skip-seven,.tenths.skip-seven{margin-left:70%}.tenth.right-eight,.tenths.right-eight{left:80%}.tenth.left-eight,.tenths.left-eight{right:80%}.tenth.skip-eight,.tenths.skip-eight{margin-left:80%}.tenth.right-nine,.tenths.right-nine{left:90%}.tenth.left-nine,.tenths.left-nine{right:90%}.tenth.skip-nine,.tenths.skip-nine{margin-left:90%}.eleventh,.elevenths{width:9.09091%}.eleventh.one,.elevenths.one{width:9.09091%}.eleventh.two,.elevenths.two{width:18.18182%}.eleventh.three,.elevenths.three{width:27.27273%}.eleventh.four,.elevenths.four{width:36.36364%}.eleventh.five,.elevenths.five{width:45.45455%}.eleventh.six,.elevenths.six{width:54.54545%}.eleventh.seven,.elevenths.seven{width:63.63636%}.eleventh.eight,.elevenths.eight{width:72.72727%}.eleventh.nine,.elevenths.nine{width:81.81818%}.eleventh.ten,.elevenths.ten{width:90.90909%}.eleventh.eleven,.elevenths.eleven{width:100%}.eleventh.right-one,.elevenths.right-one{left:9.09091%}.eleventh.left-one,.elevenths.left-one{right:9.09091%}.eleventh.skip-one,.elevenths.skip-one{margin-left:9.09091%}.eleventh.right-two,.elevenths.right-two{left:18.18182%}.eleventh.left-two,.elevenths.left-two{right:18.18182%}.eleventh.skip-two,.elevenths.skip-two{margin-left:18.18182%}.eleventh.right-three,.elevenths.right-three{left:27.27273%}.eleventh.left-three,.elevenths.left-three{right:27.27273%}.eleventh.skip-three,.elevenths.skip-three{margin-left:27.27273%}.eleventh.right-four,.elevenths.right-four{left:36.36364%}.eleventh.left-four,.elevenths.left-four{right:36.36364%}.eleventh.skip-four,.elevenths.skip-four{margin-left:36.36364%}.eleventh.right-five,.elevenths.right-five{left:45.45455%}.eleventh.left-five,.elevenths.left-five{right:45.45455%}.eleventh.skip-five,.elevenths.skip-five{margin-left:45.45455%}.eleventh.right-six,.elevenths.right-six{left:54.54545%}.eleventh.left-six,.elevenths.left-six{right:54.54545%}.eleventh.skip-six,.elevenths.skip-six{margin-left:54.54545%}.eleventh.right-seven,.elevenths.right-seven{left:63.63636%}.eleventh.left-seven,.elevenths.left-seven{right:63.63636%}.eleventh.skip-seven,.elevenths.skip-seven{margin-left:63.63636%}.eleventh.right-eight,.elevenths.right-eight{left:72.72727%}.eleventh.left-eight,.elevenths.left-eight{right:72.72727%}.eleventh.skip-eight,.elevenths.skip-eight{margin-left:72.72727%}.eleventh.right-nine,.elevenths.right-nine{left:81.81818%}.eleventh.left-nine,.elevenths.left-nine{right:81.81818%}.eleventh.skip-nine,.elevenths.skip-nine{margin-left:81.81818%}.eleventh.right-ten,.elevenths.right-ten{left:90.90909%}.eleventh.left-ten,.elevenths.left-ten{right:90.90909%}.eleventh.skip-ten,.elevenths.skip-ten{margin-left:90.90909%}.twelfth,.twelfths{width:8.33333%}.twelfth.one,.twelfths.one{width:8.33333%}.twelfth.two,.twelfths.two{width:16.66667%}.twelfth.three,.twelfths.three{width:25%}.twelfth.four,.twelfths.four{width:33.33333%}.twelfth.five,.twelfths.five{width:41.66667%}.twelfth.six,.twelfths.six{width:50%}.twelfth.seven,.twelfths.seven{width:58.33333%}.twelfth.eight,.twelfths.eight{width:66.66667%}.twelfth.nine,.twelfths.nine{width:75%}.twelfth.ten,.twelfths.ten{width:83.33333%}.twelfth.eleven,.twelfths.eleven{width:91.66667%}.twelfth.twelve,.twelfths.twelve{width:100%}.twelfth.right-one,.twelfths.right-one{left:8.33333%}.twelfth.left-one,.twelfths.left-one{right:8.33333%}.twelfth.skip-one,.twelfths.skip-one{margin-left:8.33333%}.twelfth.right-two,.twelfths.right-two{left:16.66667%}.twelfth.left-two,.twelfths.left-two{right:16.66667%}.twelfth.skip-two,.twelfths.skip-two{margin-left:16.66667%}.twelfth.right-three,.twelfths.right-three{left:25%}.twelfth.left-three,.twelfths.left-three{right:25%}.twelfth.skip-three,.twelfths.skip-three{margin-left:25%}.twelfth.right-four,.twelfths.right-four{left:33.33333%}.twelfth.left-four,.twelfths.left-four{right:33.33333%}.twelfth.skip-four,.twelfths.skip-four{margin-left:33.33333%}.twelfth.right-five,.twelfths.right-five{left:41.66667%}.twelfth.left-five,.twelfths.left-five{right:41.66667%}.twelfth.skip-five,.twelfths.skip-five{margin-left:41.66667%}.twelfth.right-six,.twelfths.right-six{left:50%}.twelfth.left-six,.twelfths.left-six{right:50%}.twelfth.skip-six,.twelfths.skip-six{margin-left:50%}.twelfth.right-seven,.twelfths.right-seven{left:58.33333%}.twelfth.left-seven,.twelfths.left-seven{right:58.33333%}.twelfth.skip-seven,.twelfths.skip-seven{margin-left:58.33333%}.twelfth.right-eight,.twelfths.right-eight{left:66.66667%}.twelfth.left-eight,.twelfths.left-eight{right:66.66667%}.twelfth.skip-eight,.twelfths.skip-eight{margin-left:66.66667%}.twelfth.right-nine,.twelfths.right-nine{left:75%}.twelfth.left-nine,.twelfths.left-nine{right:75%}.twelfth.skip-nine,.twelfths.skip-nine{margin-left:75%}.twelfth.right-ten,.twelfths.right-ten{left:83.33333%}.twelfth.left-ten,.twelfths.left-ten{right:83.33333%}.twelfth.skip-ten,.twelfths.skip-ten{margin-left:83.33333%}.twelfth.right-eleven,.twelfths.right-eleven{left:91.66667%}.twelfth.left-eleven,.twelfths.left-eleven{right:91.66667%}.twelfth.skip-eleven,.twelfths.skip-eleven{margin-left:91.66667%}.one-up,.one-up-desktop{width:100%!important;clear:none!important;float:left!important;left:0!important;right:0!important}.two-up,.two-up-desktop{width:50%!important;clear:none!important;float:left!important;left:0!important;right:0!important}.three-up,.three-up-desktop{width:33.33333%!important;clear:none!important;float:left!important;left:0!important;right:0!important}.four-up,.four-up-desktop{width:25%!important;clear:none!important;float:left!important;left:0!important;right:0!important}.five-up,.five-up-desktop{width:20%!important;clear:none!important;float:left!important;left:0!important;right:0!important}.one-up:nth-child(1n+1),.one-up-desktop:nth-child(1n+1){clear:both!important}.two-up:nth-child(2n+1),.two-up-desktop:nth-child(2n+1){clear:both!important}.three-up:nth-child(3n+1),.three-up-desktop:nth-child(3n+1){clear:both!important}.four-up:nth-child(4n+1),.four-up-desktop:nth-child(4n+1){clear:both!important}.five-up:nth-child(5n+1),.five-up-desktop:nth-child(5n+1){clear:both!important}@media (min-width:768px) and (max-width:1024px){.one-up-ipad{width:100%!important;clear:none!important;float:left!important;left:0!important;right:0!important}.two-up-ipad{width:50%!important;clear:none!important;float:left!important;left:0!important;right:0!important}.three-up-ipad{width:33.33333%!important;clear:none!important;float:left!important;left:0!important;right:0!important}.four-up-ipad{width:25%!important;clear:none!important;float:left!important;left:0!important;right:0!important}.five-up-ipad{width:20%!important;clear:none!important;float:left!important;left:0!important;right:0!important}}@media (min-width:768px) and (max-width:1024px) and (min-width:768px) and (min-width:481px){.one-up-ipad:nth-child(1n+1){clear:both!important}.two-up-ipad:nth-child(2n+1){clear:both!important}.three-up-ipad:nth-child(3n+1){clear:both!important}.four-up-ipad:nth-child(4n+1){clear:both!important}.five-up-ipad:nth-child(5n+1){clear:both!important}}@media (max-width:767px) and (min-width:481px){.one-up-small-tablet{width:100%!important;clear:none!important;float:left!important;left:0!important;right:0!important}.two-up-small-tablet{width:50%!important;clear:none!important;float:left!important;left:0!important;right:0!important}.three-up-small-tablet{width:33.33333%!important;clear:none!important;float:left!important;left:0!important;right:0!important}.four-up-small-tablet{width:25%!important;clear:none!important;float:left!important;left:0!important;right:0!important}.five-up-small-tablet{width:20%!important;clear:none!important;float:left!important;left:0!important;right:0!important}}@media (max-width:767px) and (min-width:481px){.one-up-small-tablet:nth-child(1n+1){clear:both!important}.two-up-small-tablet:nth-child(2n+1){clear:both!important}.three-up-small-tablet:nth-child(3n+1){clear:both!important}.four-up-small-tablet:nth-child(4n+1){clear:both!important}.five-up-small-tablet:nth-child(5n+1){clear:both!important}}@media (max-width:480px){.one-up-mobile{width:100%!important;clear:none!important;float:left!important;left:0!important;right:0!important}.one-up-mobile:nth-child(2){clear:both!important}.two-up-mobile{width:50%!important;clear:none!important;float:left!important;left:0!important;right:0!important}.two-up-mobile:nth-child(3){clear:both!important}.three-up-mobile{width:33.33333%!important;clear:none!important;float:left!important;left:0!important;right:0!important}.three-up-mobile:nth-child(4){clear:both!important}.four-up-mobile{width:25%!important;clear:none!important;float:left!important;left:0!important;right:0!important}.four-up-mobile:nth-child(5){clear:both!important}.five-up-mobile{width:20%!important;clear:none!important;float:left!important;left:0!important;right:0!important}.five-up-mobile:nth-child(6){clear:both!important}}@media (max-width:480px){.one-up-mobile:nth-child(1n+1){clear:both!important}.two-up-mobile:nth-child(2n+1){clear:both!important}.three-up-mobile:nth-child(3n+1){clear:both!important}.four-up-mobile:nth-child(4n+1){clear:both!important}.five-up-mobile:nth-child(5n+1){clear:both!important}}.flex-wrapper{display:-webkit-box;display:-webkit-flex;display:-ms-flexbox;display:flex}.flex-box{width:percent(1);-webkit-box-flex:1;-webkit-flex:1;-ms-flex:1;flex:1}.vertical-center{display:-webkit-box;display:-webkit-flex;display:-ms-flexbox;display:box;-webkit-box-orient:vertical;-webkit-box-direction:normal;-webkit-flex-direction:vertical;-ms-flex-direction:vertical;box-orient:vertical;-webkit-box-pack:center;-webkit-justify-content:center;-ms-flex-pack:center;box-pack:center}@font-face{font-family:FontAwesome;src:url(../fonts/fontawesome-webfont.eot?v=3.2.1);src:url(../fonts/fontawesome-webfont.eot?#iefix&v=3.2.1) format("embedded-opentype"),url(../fonts/fontawesome-webfont.woff?v=3.2.1) format("woff"),url(../fonts/fontawesome-webfont.ttf?v=3.2.1) format("truetype"),url(../fonts/fontawesome-webfont.svg#fontawesomeregular?v=3.2.1) format("svg");font-weight:400;font-style:normal}[class^=icon-],[class*=" icon-"]{font-family:FontAwesome;font-weight:400;font-style:normal;text-decoration:inherit;-webkit-font-smoothing:antialiased;*margin-right:.3em}[class^=icon-]:before,[class*=" icon-"]:before{text-decoration:inherit;display:inline-block;speak:none}.icon-large:before{vertical-align:-10%;font-size:1.33333em}a [class^=icon-],a [class*=" icon-"]{display:inline}[class^=icon-].icon-fixed-width,[class*=" icon-"].icon-fixed-width{display:inline-block;width:1.14286em;text-align:right;padding-right:.28571em}[class^=icon-].icon-fixed-width.icon-large,[class*=" icon-"].icon-fixed-width.icon-large{width:1.42857em}.icons-ul{margin-left:2.14286em;list-style-type:none}.icons-ul>li{position:relative}.icons-ul .icon-li{position:absolute;left:-2.14286em;width:2.14286em;text-align:center;line-height:inherit}[class^=icon-].hide,[class*=" icon-"].hide{display:none}.icon-muted{color:#eee}.icon-light{color:#fff}.icon-dark{color:#333}.icon-border{border:solid 1px #eee;padding:.2em .25em .15em;-o-border-radius:3px;border-radius:3px}.icon-2x{font-size:2em}.icon-2x.icon-border{border-width:2px;-o-border-radius:4px;border-radius:4px}.icon-3x{font-size:3em}.icon-3x.icon-border{border-width:3px;-o-border-radius:5px;border-radius:5px}.icon-4x{font-size:4em}.icon-4x.icon-border{border-width:4px;-o-border-radius:6px;border-radius:6px}.icon-5x{font-size:5em}.icon-5x.icon-border{border-width:5px;-o-border-radius:7px;border-radius:7px}.pull-right{float:right}.pull-left{float:left}[class^=icon-].pull-left,[class*=" icon-"].pull-left{margin-right:.3em}[class^=icon-].pull-right,[class*=" icon-"].pull-right{margin-left:.3em}.icon-stack{position:relative;display:inline-block;width:2em;height:2em;line-height:2em;vertical-align:-35%}.icon-stack [class^=icon-],.icon-stack [class*=" icon-"]{display:block;text-align:center;position:absolute;width:100%;height:100%;font-size:1em;line-height:inherit;*line-height:2em}.icon-stack .icon-stack-base{font-size:2em;*line-height:1em}.icon-spin{display:inline-block;-webkit-animation:spin 2s infinite linear;animation:spin 2s infinite linear}a .icon-stack,a .icon-spin{display:inline-block;text-decoration:none}@-webkit-keyframes spin{0%{-webkit-transform:rotate(0deg);transform:rotate(0deg)}100%{-webkit-transform:rotate(359deg);transform:rotate(359deg)}}@keyframes spin{0%{-webkit-transform:rotate(0deg);transform:rotate(0deg)}100%{-webkit-transform:rotate(359deg);transform:rotate(359deg)}}.icon-rotate-90:before{-webkit-transform:rotate(90deg);transform:rotate(90deg);filter:progid:DXImageTransform.Microsoft.BasicImage(rotation=1)}.icon-rotate-180:before{-webkit-transform:rotate(180deg);transform:rotate(180deg);filter:progid:DXImageTransform.Microsoft.BasicImage(rotation=2)}.icon-rotate-270:before{-webkit-transform:rotate(270deg);transform:rotate(270deg);filter:progid:DXImageTransform.Microsoft.BasicImage(rotation=3)}.icon-flip-horizontal:before{-webkit-transform:scale(-1,1);transform:scale(-1,1)}.icon-flip-vertical:before{-webkit-transform:scale(1,-1);transform:scale(1,-1)}a .icon-rotate-90:before,a .icon-rotate-180:before,a .icon-rotate-270:before,a .icon-flip-horizontal:before,a .icon-flip-vertical:before{display:inline-block}.icon-glass:before{content:"\f000"}.icon-music:before{content:"\f001"}.icon-search:before{content:"\f002"}.icon-envelope-alt:before{content:"\f003"}.icon-heart:before{content:"\f004"}.icon-star:before{content:"\f005"}.icon-star-empty:before{content:"\f006"}.icon-user:before{content:"\f007"}.icon-film:before{content:"\f008"}.icon-th-large:before{content:"\f009"}.icon-th:before{content:"\f00a"}.icon-th-list:before{content:"\f00b"}.icon-ok:before{content:"\f00c"}.icon-remove:before{content:"\f00d"}.icon-zoom-in:before{content:"\f00e"}.icon-zoom-out:before{content:"\f010"}.icon-power-off:before,.icon-off:before{content:"\f011"}.icon-signal:before{content:"\f012"}.icon-gear:before,.icon-cog:before{content:"\f013"}.icon-trash:before{content:"\f014"}.icon-home:before{content:"\f015"}.icon-file-alt:before{content:"\f016"}.icon-time:before{content:"\f017"}.icon-road:before{content:"\f018"}.icon-download-alt:before{content:"\f019"}.icon-download:before{content:"\f01a"}.icon-upload:before{content:"\f01b"}.icon-inbox:before{content:"\f01c"}.icon-play-circle:before{content:"\f01d"}.icon-rotate-right:before,.icon-repeat:before{content:"\f01e"}.icon-refresh:before{content:"\f021"}.icon-list-alt:before{content:"\f022"}.icon-lock:before{content:"\f023"}.icon-flag:before{content:"\f024"}.icon-headphones:before{content:"\f025"}.icon-volume-off:before{content:"\f026"}.icon-volume-down:before{content:"\f027"}.icon-volume-up:before{content:"\f028"}.icon-qrcode:before{content:"\f029"}.icon-barcode:before{content:"\f02a"}.icon-tag:before{content:"\f02b"}.icon-tags:before{content:"\f02c"}.icon-book:before{content:"\f02d"}.icon-bookmark:before{content:"\f02e"}.icon-print:before{content:"\f02f"}.icon-camera:before{content:"\f030"}.icon-font:before{content:"\f031"}.icon-bold:before{content:"\f032"}.icon-italic:before{content:"\f033"}.icon-text-height:before{content:"\f034"}.icon-text-width:before{content:"\f035"}.icon-align-left:before{content:"\f036"}.icon-align-center:before{content:"\f037"}.icon-align-right:before{content:"\f038"}.icon-align-justify:before{content:"\f039"}.icon-list:before{content:"\f03a"}.icon-indent-left:before{content:"\f03b"}.icon-indent-right:before{content:"\f03c"}.icon-facetime-video:before{content:"\f03d"}.icon-picture:before{content:"\f03e"}.icon-pencil:before{content:"\f040"}.icon-map-marker:before{content:"\f041"}.icon-adjust:before{content:"\f042"}.icon-tint:before{content:"\f043"}.icon-edit:before{content:"\f044"}.icon-share:before{content:"\f045"}.icon-check:before{content:"\f046"}.icon-move:before{content:"\f047"}.icon-step-backward:before{content:"\f048"}.icon-fast-backward:before{content:"\f049"}.icon-backward:before{content:"\f04a"}.icon-play:before{content:"\f04b"}.icon-pause:before{content:"\f04c"}.icon-stop:before{content:"\f04d"}.icon-forward:before{content:"\f04e"}.icon-fast-forward:before{content:"\f050"}.icon-step-forward:before{content:"\f051"}.icon-eject:before{content:"\f052"}.icon-chevron-left:before{content:"\f053"}.icon-chevron-right:before{content:"\f054"}.icon-plus-sign:before{content:"\f055"}.icon-minus-sign:before{content:"\f056"}.icon-remove-sign:before{content:"\f057"}.icon-ok-sign:before{content:"\f058"}.icon-question-sign:before{content:"\f059"}.icon-info-sign:before{content:"\f05a"}.icon-screenshot:before{content:"\f05b"}.icon-remove-circle:before{content:"\f05c"}.icon-ok-circle:before{content:"\f05d"}.icon-ban-circle:before{content:"\f05e"}.icon-arrow-left:before{content:"\f060"}.icon-arrow-right:before{content:"\f061"}.icon-arrow-up:before{content:"\f062"}.icon-arrow-down:before{content:"\f063"}.icon-mail-forward:before,.icon-share-alt:before{content:"\f064"}.icon-resize-full:before{content:"\f065"}.icon-resize-small:before{content:"\f066"}.icon-plus:before{content:"\f067"}.icon-minus:before{content:"\f068"}.icon-asterisk:before{content:"\f069"}.icon-exclamation-sign:before{content:"\f06a"}.icon-gift:before{content:"\f06b"}.icon-leaf:before{content:"\f06c"}.icon-fire:before{content:"\f06d"}.icon-eye-open:before{content:"\f06e"}.icon-eye-close:before{content:"\f070"}.icon-warning-sign:before{content:"\f071"}.icon-plane:before{content:"\f072"}.icon-calendar:before{content:"\f073"}.icon-random:before{content:"\f074"}.icon-comment:before{content:"\f075"}.icon-magnet:before{content:"\f076"}.icon-chevron-up:before{content:"\f077"}.icon-chevron-down:before{content:"\f078"}.icon-retweet:before{content:"\f079"}.icon-shopping-cart:before{content:"\f07a"}.icon-folder-close:before{content:"\f07b"}.icon-folder-open:before{content:"\f07c"}.icon-resize-vertical:before{content:"\f07d"}.icon-resize-horizontal:before{content:"\f07e"}.icon-bar-chart:before{content:"\f080"}.icon-twitter-sign:before{content:"\f081"}.icon-facebook-sign:before{content:"\f082"}.icon-camera-retro:before{content:"\f083"}.icon-key:before{content:"\f084"}.icon-gears:before,.icon-cogs:before{content:"\f085"}.icon-comments:before{content:"\f086"}.icon-thumbs-up-alt:before{content:"\f087"}.icon-thumbs-down-alt:before{content:"\f088"}.icon-star-half:before{content:"\f089"}.icon-heart-empty:before{content:"\f08a"}.icon-signout:before{content:"\f08b"}.icon-linkedin-sign:before{content:"\f08c"}.icon-pushpin:before{content:"\f08d"}.icon-external-link:before{content:"\f08e"}.icon-signin:before{content:"\f090"}.icon-trophy:before{content:"\f091"}.icon-github-sign:before{content:"\f092"}.icon-upload-alt:before{content:"\f093"}.icon-lemon:before{content:"\f094"}.icon-phone:before{content:"\f095"}.icon-unchecked:before,.icon-check-empty:before{content:"\f096"}.icon-bookmark-empty:before{content:"\f097"}.icon-phone-sign:before{content:"\f098"}.icon-twitter:before{content:"\f099"}.icon-facebook:before{content:"\f09a"}.icon-github:before{content:"\f09b"}.icon-unlock:before{content:"\f09c"}.icon-credit-card:before{content:"\f09d"}.icon-rss:before{content:"\f09e"}.icon-hdd:before{content:"\f0a0"}.icon-bullhorn:before{content:"\f0a1"}.icon-bell:before{content:"\f0a2"}.icon-certificate:before{content:"\f0a3"}.icon-hand-right:before{content:"\f0a4"}.icon-hand-left:before{content:"\f0a5"}.icon-hand-up:before{content:"\f0a6"}.icon-hand-down:before{content:"\f0a7"}.icon-circle-arrow-left:before{content:"\f0a8"}.icon-circle-arrow-right:before{content:"\f0a9"}.icon-circle-arrow-up:before{content:"\f0aa"}.icon-circle-arrow-down:before{content:"\f0ab"}.icon-globe:before{content:"\f0ac"}.icon-wrench:before{content:"\f0ad"}.icon-tasks:before{content:"\f0ae"}.icon-filter:before{content:"\f0b0"}.icon-briefcase:before{content:"\f0b1"}.icon-fullscreen:before{content:"\f0b2"}.icon-group:before{content:"\f0c0"}.icon-link:before{content:"\f0c1"}.icon-cloud:before{content:"\f0c2"}.icon-beaker:before{content:"\f0c3"}.icon-cut:before{content:"\f0c4"}.icon-copy:before{content:"\f0c5"}.icon-paperclip:before,.icon-paper-clip:before{content:"\f0c6"}.icon-save:before{content:"\f0c7"}.icon-sign-blank:before{content:"\f0c8"}.icon-reorder:before{content:"\f0c9"}.icon-list-ul:before{content:"\f0ca"}.icon-list-ol:before{content:"\f0cb"}.icon-strikethrough:before{content:"\f0cc"}.icon-underline:before{content:"\f0cd"}.icon-table:before{content:"\f0ce"}.icon-magic:before{content:"\f0d0"}.icon-truck:before{content:"\f0d1"}.icon-pinterest:before{content:"\f0d2"}.icon-pinterest-sign:before{content:"\f0d3"}.icon-google-plus-sign:before{content:"\f0d4"}.icon-google-plus:before{content:"\f0d5"}.icon-money:before{content:"\f0d6"}.icon-caret-down:before{content:"\f0d7"}.icon-caret-up:before{content:"\f0d8"}.icon-caret-left:before{content:"\f0d9"}.icon-caret-right:before{content:"\f0da"}.icon-columns:before{content:"\f0db"}.icon-sort:before{content:"\f0dc"}.icon-sort-down:before{content:"\f0dd"}.icon-sort-up:before{content:"\f0de"}.icon-envelope:before{content:"\f0e0"}.icon-linkedin:before{content:"\f0e1"}.icon-rotate-left:before,.icon-undo:before{content:"\f0e2"}.icon-legal:before{content:"\f0e3"}.icon-dashboard:before{content:"\f0e4"}.icon-comment-alt:before{content:"\f0e5"}.icon-comments-alt:before{content:"\f0e6"}.icon-bolt:before{content:"\f0e7"}.icon-sitemap:before{content:"\f0e8"}.icon-umbrella:before{content:"\f0e9"}.icon-paste:before{content:"\f0ea"}.icon-lightbulb:before{content:"\f0eb"}.icon-exchange:before{content:"\f0ec"}.icon-cloud-download:before{content:"\f0ed"}.icon-cloud-upload:before{content:"\f0ee"}.icon-user-md:before{content:"\f0f0"}.icon-stethoscope:before{content:"\f0f1"}.icon-suitcase:before{content:"\f0f2"}.icon-bell-alt:before{content:"\f0f3"}.icon-coffee:before{content:"\f0f4"}.icon-food:before{content:"\f0f5"}.icon-file-text-alt:before{content:"\f0f6"}.icon-building:before{content:"\f0f7"}.icon-hospital:before{content:"\f0f8"}.icon-ambulance:before{content:"\f0f9"}.icon-medkit:before{content:"\f0fa"}.icon-fighter-jet:before{content:"\f0fb"}.icon-beer:before{content:"\f0fc"}.icon-h-sign:before{content:"\f0fd"}.icon-plus-sign-alt:before{content:"\f0fe"}.icon-double-angle-left:before{content:"\f100"}.icon-double-angle-right:before{content:"\f101"}.icon-double-angle-up:before{content:"\f102"}.icon-double-angle-down:before{content:"\f103"}.icon-angle-left:before{content:"\f104"}.icon-angle-right:before{content:"\f105"}.icon-angle-up:before{content:"\f106"}.icon-angle-down:before{content:"\f107"}.icon-desktop:before{content:"\f108"}.icon-laptop:before{content:"\f109"}.icon-tablet:before{content:"\f10a"}.icon-mobile-phone:before{content:"\f10b"}.icon-circle-blank:before{content:"\f10c"}.icon-quote-left:before{content:"\f10d"}.icon-quote-right:before{content:"\f10e"}.icon-spinner:before{content:"\f110"}.icon-circle:before{content:"\f111"}.icon-mail-reply:before,.icon-reply:before{content:"\f112"}.icon-github-alt:before{content:"\f113"}.icon-folder-close-alt:before{content:"\f114"}.icon-folder-open-alt:before{content:"\f115"}.icon-expand-alt:before{content:"\f116"}.icon-collapse-alt:before{content:"\f117"}.icon-smile:before{content:"\f118"}.icon-frown:before{content:"\f119"}.icon-meh:before{content:"\f11a"}.icon-gamepad:before{content:"\f11b"}.icon-keyboard:before{content:"\f11c"}.icon-flag-alt:before{content:"\f11d"}.icon-flag-checkered:before{content:"\f11e"}.icon-terminal:before{content:"\f120"}.icon-code:before{content:"\f121"}.icon-reply-all:before{content:"\f122"}.icon-mail-reply-all:before{content:"\f122"}.icon-star-half-full:before,.icon-star-half-empty:before{content:"\f123"}.icon-location-arrow:before{content:"\f124"}.icon-crop:before{content:"\f125"}.icon-code-fork:before{content:"\f126"}.icon-unlink:before{content:"\f127"}.icon-question:before{content:"\f128"}.icon-info:before{content:"\f129"}.icon-exclamation:before{content:"\f12a"}.icon-superscript:before{content:"\f12b"}.icon-subscript:before{content:"\f12c"}.icon-eraser:before{content:"\f12d"}.icon-puzzle-piece:before{content:"\f12e"}.icon-microphone:before{content:"\f130"}.icon-microphone-off:before{content:"\f131"}.icon-shield:before{content:"\f132"}.icon-calendar-empty:before{content:"\f133"}.icon-fire-extinguisher:before{content:"\f134"}.icon-rocket:before{content:"\f135"}.icon-maxcdn:before{content:"\f136"}.icon-chevron-sign-left:before{content:"\f137"}.icon-chevron-sign-right:before{content:"\f138"}.icon-chevron-sign-up:before{content:"\f139"}.icon-chevron-sign-down:before{content:"\f13a"}.icon-html5:before{content:"\f13b"}.icon-css3:before{content:"\f13c"}.icon-anchor:before{content:"\f13d"}.icon-unlock-alt:before{content:"\f13e"}.icon-bullseye:before{content:"\f140"}.icon-ellipsis-horizontal:before{content:"\f141"}.icon-ellipsis-vertical:before{content:"\f142"}.icon-rss-sign:before{content:"\f143"}.icon-play-sign:before{content:"\f144"}.icon-ticket:before{content:"\f145"}.icon-minus-sign-alt:before{content:"\f146"}.icon-check-minus:before{content:"\f147"}.icon-level-up:before{content:"\f148"}.icon-level-down:before{content:"\f149"}.icon-check-sign:before{content:"\f14a"}.icon-edit-sign:before{content:"\f14b"}.icon-external-link-sign:before{content:"\f14c"}.icon-share-sign:before{content:"\f14d"}.icon-compass:before{content:"\f14e"}.icon-collapse:before{content:"\f150"}.icon-collapse-top:before{content:"\f151"}.icon-expand:before{content:"\f152"}.icon-euro:before,.icon-eur:before{content:"\f153"}.icon-gbp:before{content:"\f154"}.icon-dollar:before,.icon-usd:before{content:"\f155"}.icon-rupee:before,.icon-inr:before{content:"\f156"}.icon-yen:before,.icon-jpy:before{content:"\f157"}.icon-renminbi:before,.icon-cny:before{content:"\f158"}.icon-won:before,.icon-krw:before{content:"\f159"}.icon-bitcoin:before,.icon-btc:before{content:"\f15a"}.icon-file:before{content:"\f15b"}.icon-file-text:before{content:"\f15c"}.icon-sort-by-alphabet:before{content:"\f15d"}.icon-sort-by-alphabet-alt:before{content:"\f15e"}.icon-sort-by-attributes:before{content:"\f160"}.icon-sort-by-attributes-alt:before{content:"\f161"}.icon-sort-by-order:before{content:"\f162"}.icon-sort-by-order-alt:before{content:"\f163"}.icon-thumbs-up:before{content:"\f164"}.icon-thumbs-down:before{content:"\f165"}.icon-youtube-sign:before{content:"\f166"}.icon-youtube:before{content:"\f167"}.icon-xing:before{content:"\f168"}.icon-xing-sign:before{content:"\f169"}.icon-youtube-play:before{content:"\f16a"}.icon-dropbox:before{content:"\f16b"}.icon-stackexchange:before{content:"\f16c"}.icon-instagram:before{content:"\f16d"}.icon-flickr:before{content:"\f16e"}.icon-adn:before{content:"\f170"}.icon-bitbucket:before{content:"\f171"}.icon-bitbucket-sign:before{content:"\f172"}.icon-tumblr:before{content:"\f173"}.icon-tumblr-sign:before{content:"\f174"}.icon-long-arrow-down:before{content:"\f175"}.icon-long-arrow-up:before{content:"\f176"}.icon-long-arrow-left:before{content:"\f177"}.icon-long-arrow-right:before{content:"\f178"}.icon-apple:before{content:"\f179"}.icon-windows:before{content:"\f17a"}.icon-android:before{content:"\f17b"}.icon-linux:before{content:"\f17c"}.icon-dribbble:before{content:"\f17d"}.icon-skype:before{content:"\f17e"}.icon-foursquare:before{content:"\f180"}.icon-trello:before{content:"\f181"}.icon-female:before{content:"\f182"}.icon-male:before{content:"\f183"}.icon-gittip:before{content:"\f184"}.icon-sun:before{content:"\f185"}.icon-moon:before{content:"\f186"}.icon-archive:before{content:"\f187"}.icon-bug:before{content:"\f188"}.icon-vk:before{content:"\f189"}.icon-weibo:before{content:"\f18a"}.icon-renren:before{content:"\f18b"}a:before,a:after{font-family:FontAwesome;font-weight:400;font-style:normal;text-decoration:inherit;-webkit-font-smoothing:antialiased;*margin-right:.3em;text-decoration:none}a:before{padding-right:.25em}a:after{padding-left:.25em}a:not(.noicon)[rel=home]:before,button:not(.noicon)[rel=home]:before{content:"\f015"}a:not(.noicon)[rel=search]:before,button:not(.noicon)[rel=search]:before{content:"\f002"}a:not(.noicon)[rel=author]:before,button:not(.noicon)[rel=author]:before{content:"\f007"}a:not(.noicon)[rel=help]:before,button:not(.noicon)[rel=help]:before{content:"\f059"}a:not(.noicon)[rel=tag]:before,button:not(.noicon)[rel=tag]:before{content:"\f02b"}a:not(.noicon)[rel=bookmark]:before,button:not(.noicon)[rel=bookmark]:before{content:"\f02e"}a:not(.noicon)[rel=prev]:before,a:not(.noicon)[rel=previous]:before,button:not(.noicon)[rel=prev]:before,button:not(.noicon)[rel=previous]:before{content:"\f0a8"}a:not(.noicon)[rel=next]:after,button:not(.noicon)[rel=next]:after{content:"\f0a9"}a:not(.noicon)[rel=directory]:after,button:not(.noicon)[rel=directory]:after{content:"\f115"}a:not(.noicon)[rel=payment]:after,button:not(.noicon)[rel=payment]:after{content:"\f0d6"}a:not(.noicon)[href^="#"]:not([href="#"]):after{content:"\f0c1"}a:not(.noicon)[href^="https:"]:after{content:"\f023"}a:not(.noicon)[target="_blank"]:after,a:not(.noicon)[rel=external]:after{content:"\f08e"}a:not(.noicon)[href^="view-source:"]:after{content:"\f121"}a:not(.noicon)[href^="mailto:"]:before{content:"\f0e0"}a:not(.noicon)[href^="tel:"]:before{content:"\f098"}a:not(.noicon)[href^="skype:"]:before{content:"\f17e"}a:not(.noicon)[href$=".pdf"]:after{content:"\f016"}a:not(.noicon)[href$=".doc"]:after{content:"\f15c"}a:not(.noicon)[href$=".txt"]:after,a:not(.noicon)[href$=".md"]:after{content:"\f0f6"}a:not(.noicon)[href$=".jpg"]:after,a:not(.noicon)[href$=".jpeg"]:after,a:not(.noicon)[href$=".png"]:after,a:not(.noicon)[href$=".gif"]:after,a:not(.noicon)[href$=".svg"]:after{content:"\f03e"}a:not(.noicon)[href$=".exe"]:after{content:"\f0c7"}a:not(.noicon)[href*=".zip"]:after,a:not(.noicon)[href*=".tar"]:after,a:not(.noicon)[href*=".gz"]:after{content:"\f019"}a:not(.noicon)[href$=".rss"]:before,a:not(.noicon)[href$="rss.xml"]:before,a:not(.noicon)[href*="/feed/"]:before,a:not(.noicon)[href*="/rss/"]:before{content:"\f143"}a:not(.noicon)[href$=".rss"]:after,a:not(.noicon)[href$="rss.xml"]:after,a:not(.noicon)[href*="/feed/"]:after,a:not(.noicon)[href*="/rss/"]:after{content:"";padding-left:0}a:not(.noicon)[href*="twitter.com"]:before{content:"\f081"}a:not(.noicon)[href*="twitter.com"]:after{content:"";padding-left:0}a:not(.noicon)[href*="pinterest.com"]:before{content:"\f0d3"}a:not(.noicon)[href*="pinterest.com"]:after{content:"";padding-left:0}a:not(.noicon)[href*="facebook.com"]:before{content:"\f082"}a:not(.noicon)[href*="facebook.com"]:after{content:"";padding-left:0}a:not(.noicon)[href*="linkedin.com"]:before{content:"\f08c"}a:not(.noicon)[href*="linkedin.com"]:after{content:"";padding-left:0}a:not(.noicon)[href*="tumblr.com"]:before{content:"\f174"}a:not(.noicon)[href*="tumblr.com"]:after{content:"";padding-left:0}a:not(.noicon)[href*="plus.google.com"]:before{content:"\f0d4"}a:not(.noicon)[href*="plus.google.com"]:after{content:"";padding-left:0}a:not(.noicon)[href*="youtube.com"]:before,a:not(.noicon)[href*="youtu.be"]:before{content:"\f166"}a:not(.noicon)[href*="youtube.com"]:after,a:not(.noicon)[href*="youtu.be"]:after{content:"";padding-left:0}a:not(.noicon)[href*="instagram.com"]:before{content:"\f16d"}a:not(.noicon)[href*="instagram.com"]:after{content:"";padding-left:0}a:not(.noicon)[href*="flickr.com"]:before{content:"\f16e"}a:not(.noicon)[href*="flickr.com"]:after{content:"";padding-left:0}a:not(.noicon)[href*="foursquare.com"]:before{content:"\f180"}a:not(.noicon)[href*="foursquare.com"]:after{content:"";padding-left:0}a:not(.noicon)[href*="dribbble.com"]:before{content:"\f17d"}a:not(.noicon)[href*="dribbble.com"]:after{content:"";padding-left:0}a:not(.noicon)[href*="dropbox.com"]:before{content:"\f16b"}a:not(.noicon)[href*="dropbox.com"]:after{content:"";padding-left:0}a:not(.noicon)[href*="github.com"]:before{content:"\f092"}a:not(.noicon)[href*="github.com"]:after{content:"";padding-left:0}a:not(.noicon)[href*="stackoverflow.com"]:before{content:"\f16c"}a:not(.noicon)[href*="stackoverflow.com"]:after{content:"";padding-left:0}a:not(.noicon)[href*="trello.com"]:before{content:"\f181"}a:not(.noicon)[href*="trello.com"]:after{content:"";padding-left:0}@font-face{font-family:source-sans-pro;font-style:normal;font-weight:400;src:url(../fonts/sourcesanspro-regular-webfont.eot);src:url(../fonts/sourcesanspro-regular-webfont.eot#iefix) format("embedded-opentype"),url(../fonts/sourcesanspro-regular-webfont.woff) format("woff"),url(../fonts/sourcesanspro-regular-webfont.ttf) format("truetype"),url(../fonts/sourcesanspro-regular-webfont.svg#source-sans-pro) format("svg")}.source-sans-pro,h1,h2,h3,h4,h5,h6{font-family:source-sans-pro,Helvetica,Arial,Geneva,sans-serif}@font-face{font-family:museo-slab;font-style:normal;font-weight:400;src:url(../fonts/museo-slab-500-webfont.eot);src:url(../fonts/museo-slab-500-webfont.eot#iefix) format("embedded-opentype"),url(../fonts/museo-slab-500-webfont.woff) format("woff"),url(../fonts/museo-slab-500-webfont.ttf) format("truetype"),url(../fonts/museo-slab-500-webfont.svg#museo-slab) format("svg")}.museo-slab{font-family:museo-slab,Helvetica,Arial,Geneva,sans-serif}@font-face{font-family:quicksand;font-style:normal;font-weight:400;src:url(../fonts/quicksand-regular-webfont.eot);src:url(../fonts/quicksand-regular-webfont.eot#iefix) format("embedded-opentype"),url(../fonts/quicksand-regular-webfont.woff) format("woff"),url(../fonts/quicksand-regular-webfont.ttf) format("truetype"),url(../fonts/quicksand-regular-webfont.svg#quicksand) format("svg")}.quicksand{font-family:quicksand,Helvetica,Arial,Geneva,sans-serif}@font-face{font-family:redacted;font-style:normal;font-weight:400;src:url(../fonts/redacted-regular.eot);src:url(../fonts/redacted-regular.eot#iefix) format("embedded-opentype"),url(../fonts/redacted-regular.woff) format("woff"),url(../fonts/redacted-regular.ttf) format("truetype"),url(../fonts/redacted-regular.svg#redacted) format("svg")}.redacted{font-family:redacted,Helvetica,Arial,Geneva,sans-serif}@font-face{font-family:redacted-script;font-style:normal;font-weight:400;src:url(../fonts/redacted-script-regular.eot);src:url(../fonts/redacted-script-regular.eot#iefix) format("embedded-opentype"),url(../fonts/redacted-script-regular.woff) format("woff"),url(../fonts/redacted-script-regular.ttf) format("truetype"),url(../fonts/redacted-script-regular.svg#redacted-script) format("svg")}.redacted-script{font-family:redacted-script,Helvetica,Arial,Geneva,sans-serif}@font-face{font-family:redacted-script-light;font-style:normal;font-weight:400;src:url(../fonts/redacted-script-light.eot);src:url(../fonts/redacted-script-light.eot#iefix) format("embedded-opentype"),url(../fonts/redacted-script-light.woff) format("woff"),url(../fonts/redacted-script-light.ttf) format("truetype"),url(../fonts/redacted-script-light.svg#redacted-script-light) format("svg")}.redacted-script-light{font-family:redacted-script-light,Helvetica,Arial,Geneva,sans-serif}@font-face{font-family:redacted-script-bold;font-style:normal;font-weight:400;src:url(../fonts/redacted-script-bold.eot);src:url(../fonts/redacted-script-bold.eot#iefix) format("embedded-opentype"),url(../fonts/redacted-script-bold.woff) format("woff"),url(../fonts/redacted-script-bold.ttf) format("truetype"),url(../fonts/redacted-script-bold.svg#redacted-script-bold) format("svg")}.redacted-script-bold{font-family:redacted-script-bold,Helvetica,Arial,Geneva,sans-serif}html,body{font-family:Helvetica,Arial,Geneva,sans-serif;font-size:14px;line-height:1.5;color:#2b2b2d}a,a:visited{color:#489ac1;text-decoration:none;-webkit-transition:color .25s;transition:color .25s}a:hover,a:focus{color:#198d98;text-decoration:underline}h1,h2,h3,h4,h5,h6{margin-bottom:15px;line-height:1.3}h1{font-size:2.33333em}h2{font-size:2em}h3{font-size:1.5em}h4{font-size:1.33333em}h5{font-size:1.16667em}h6{font-size:1.16667em;text-transform:uppercase}p,ul,ol,dl{margin-bottom:10px;font-weight:400;line-height:1.5}ul,ol{margin-left:20px}ul li,ol li,ul.list>li,ol.list>li{display:list-item}dl dt{font-weight:700}dl dd{padding-left:1em;margin-bottom:10px}ul.checklist{margin-left:0}ul.checklist>li:before{content:"\f096";display:inline-block;width:1.1em;margin-right:.25em;font-family:FontAwesome;font-weight:400;font-style:normal;text-decoration:inherit;-webkit-font-smoothing:antialiased;*margin-right:.3em;font-size:1.2em;vertical-align:baseline;background-image:none;background-position:0 0;background-repeat:repeat;margin-top:0}ul.checklist>li[aria-checked~=true]:before,ul.checklist>li[data-checked~=true]:before,ul.checklist>li[checked=checked]:before,ul.checklist>li.checked:before,ul.checklist>li.completed:before{content:"\f046"}ul.checklist:not([readonly]) ul.checklist>li:not([readonly]){cursor:pointer}ul.list{margin-left:0}ul.list li:before{content:"\f111";display:inline-block;width:1.1em;margin-right:.25em;font-family:FontAwesome;font-weight:400;font-style:normal;text-decoration:inherit;-webkit-font-smoothing:antialiased;*margin-right:.3em;font-size:.6em;vertical-align:middle;background-image:none;background-position:0 0;background-repeat:repeat;margin-top:0}ol.list{counter-reset:li;margin-left:1.5em;line-height:1.6}ol.list li{position:relative;padding:2px 6px 0}ol.list li:before{content:counter(li);counter-increment:li;position:absolute;top:.1em;left:-2em;width:2em;text-align:center;padding:.3em .1em .1em;font-size:.8em;line-height:1.5;color:#fff;background:#2b2b2d;font-weight:700;-o-border-radius:4px;border-radius:4px}sup{top:-.4em}sub{top:.2em}pre,code{font-family:monospace}del,strike{text-decoration:line-through}.one-column,.one-columns{-webkit-column-count:1;-moz-column-count:1;column-count:1}.two-column,.two-columns{-webkit-column-count:2;-moz-column-count:2;column-count:2}.three-column,.three-columns{-webkit-column-count:3;-moz-column-count:3;column-count:3}.four-column,.four-columns{-webkit-column-count:4;-moz-column-count:4;column-count:4}.five-column,.five-columns{-webkit-column-count:5;-moz-column-count:5;column-count:5}q:before{content:"\201C"}q:after{content:"\201D"}blockquote{padding:1.3em 4em 1em;color:#2b2b2d;font-family:source-sans-pro,Helvetica,Arial,Geneva,sans-serif;font-size:1.15em;background:#ddd;-o-border-radius:4px;border-radius:4px}blockquote:before{content:"\201C";color:#aaa;font-size:4em;display:block;position:absolute;top:-.2em;left:.1em}blockquote:after{content:"\201D";color:#aaa;font-size:4em;display:block;position:absolute;top:-.2em;right:.1em}cite{display:block;position:relative;left:-1em;margin-top:.25em;font-size:.95em;font-weight:400;font-style:normal;color:#505054}cite:before{content:"\2014"}abbr[title],dfn[title],abbr[data-title],dfn[data-title]{border-bottom:1px dotted;cursor:help}pre{white-space:pre;max-width:100%;overflow:auto}code{white-space:nowrap;background:#fff;color:#555;border:1px solid #ddd;-o-border-radius:4px;border-radius:4px;padding:0 5px}code.block{padding:5px 10px}.info{color:#3a87ad}.alert{color:#c09853}.warning{color:#ff971e}.error{color:#b94a48}.success{color:#468847}.question{color:#9855b4}.js .responsive:not(table){-webkit-transition:font .25s;transition:font .25s}::-moz-selection{background:#489ac1;color:#fff;text-shadow:none}::selection{background:#489ac1;color:#fff;text-shadow:none}ul.list>li.info:before{content:\f05a;font-size:1.2em}ul.list>li.alert:before{content:\f06a;font-size:1.2em}ul.list>li.warning:before{content:\f071;font-size:1.2em}ul.list>li.error:before{content:\f057;font-size:1.2em}ul.list>li.success:before{content:\f058;font-size:1.2em}ul.list>li.question:before{content:\f059;font-size:1.2em}ol.list.green li:before{background:#2ecc71}ol.list.blue li:before{background:#3498db}ol.list.purple li:before{background:#9b59b6}ol.list.yellow li:before{background:#f1c40f}ol.list.orange li:before{background:#e67e22}ol.list.red li:before{background:#e74c3c}ol.list.pink li:before{background:#f02475}ol.list.turquoise li:before{background:#1abc9c}ol.list.asphalt li:before{background:#34495e}ol.list.charcoal li:before{background:#383636}ol.list.white li:before{background:#fff}ol.list.black li:before{background:#111}ol.list li.green:before{background:#2ecc71}ol.list li.blue:before{background:#3498db}ol.list li.purple:before{background:#9b59b6}ol.list li.yellow:before{background:#f1c40f}ol.list li.orange:before{background:#e67e22}ol.list li.red:before{background:#e74c3c}ol.list li.pink:before{background:#f02475}ol.list li.turquoise:before{background:#1abc9c}ol.list li.asphalt:before{background:#34495e}ol.list li.charcoal:before{background:#383636}ol.list li.white:before{background:#fff}ol.list li.black:before{background:#111}ol.list li.info:before{background-color:#3a87ad}ol.list li.alert:before{background-color:#c09853}ol.list li.warning:before{background-color:#ff971e}ol.list li.error:before{background-color:#b94a48}ol.list li.success:before{background-color:#468847}ol.list li.question:before{background-color:#9855b4}.green{color:#2ecc71}.green-bg{background:#2ecc71;color:#fff}.blue{color:#3498db}.blue-bg{background:#3498db;color:#fff}.purple{color:#9b59b6}.purple-bg{background:#9b59b6;color:#fff}.yellow{color:#f1c40f}.yellow-bg{background:#f1c40f;color:#fff}.orange{color:#e67e22}.orange-bg{background:#e67e22;color:#fff}.red{color:#e74c3c}.red-bg{background:#e74c3c;color:#fff}.pink{color:#f02475}.pink-bg{background:#f02475;color:#fff}.turquoise{color:#1abc9c}.turquoise-bg{background:#1abc9c;color:#fff}.asphalt{color:#34495e}.asphalt-bg{background:#34495e;color:#fff}.charcoal{color:#383636}.charcoal-bg{background:#383636;color:#fff}.white{color:#fff}.white-bg{background:#fff;color:#111}.black{color:#111}.black-bg{background:#111;color:#fff}.info-bg{background:#d9edf7;color:#3a87ad}.alert-bg{background:#fcf8e3;color:#c09853}.warning-bg{background:#ffe5c7;color:#ff971e}.error-bg{background:#f2dede;color:#b94a48}.success-bg{background:#dff0d8;color:#468847}.question-bg{background:#f0e6f4;color:#9855b4}@font-face{font-family:source-sans-pro;font-style:normal;font-weight:400;src:url(../fonts/sourcesanspro-regular-webfont.eot);src:url(../fonts/sourcesanspro-regular-webfont.eot#iefix) format("embedded-opentype"),url(../fonts/sourcesanspro-regular-webfont.woff) format("woff"),url(../fonts/sourcesanspro-regular-webfont.ttf) format("truetype"),url(../fonts/sourcesanspro-regular-webfont.svg#source-sans-pro) format("svg")}.source-sans-pro,h1,h2,h3,h4,h5,h6{font-family:source-sans-pro,Helvetica,Arial,Geneva,sans-serif}@font-face{font-family:museo-slab;font-style:normal;font-weight:400;src:url(../fonts/museo-slab-500-webfont.eot);src:url(../fonts/museo-slab-500-webfont.eot#iefix) format("embedded-opentype"),url(../fonts/museo-slab-500-webfont.woff) format("woff"),url(../fonts/museo-slab-500-webfont.ttf) format("truetype"),url(../fonts/museo-slab-500-webfont.svg#museo-slab) format("svg")}.museo-slab{font-family:museo-slab,Helvetica,Arial,Geneva,sans-serif}@font-face{font-family:quicksand;font-style:normal;font-weight:400;src:url(../fonts/quicksand-regular-webfont.eot);src:url(../fonts/quicksand-regular-webfont.eot#iefix) format("embedded-opentype"),url(../fonts/quicksand-regular-webfont.woff) format("woff"),url(../fonts/quicksand-regular-webfont.ttf) format("truetype"),url(../fonts/quicksand-regular-webfont.svg#quicksand) format("svg")}.quicksand{font-family:quicksand,Helvetica,Arial,Geneva,sans-serif}@font-face{font-family:redacted;font-style:normal;font-weight:400;src:url(../fonts/redacted-regular.eot);src:url(../fonts/redacted-regular.eot#iefix) format("embedded-opentype"),url(../fonts/redacted-regular.woff) format("woff"),url(../fonts/redacted-regular.ttf) format("truetype"),url(../fonts/redacted-regular.svg#redacted) format("svg")}.redacted{font-family:redacted,Helvetica,Arial,Geneva,sans-serif}@font-face{font-family:redacted-script;font-style:normal;font-weight:400;src:url(../fonts/redacted-script-regular.eot);src:url(../fonts/redacted-script-regular.eot#iefix) format("embedded-opentype"),url(../fonts/redacted-script-regular.woff) format("woff"),url(../fonts/redacted-script-regular.ttf) format("truetype"),url(../fonts/redacted-script-regular.svg#redacted-script) format("svg")}.redacted-script{font-family:redacted-script,Helvetica,Arial,Geneva,sans-serif}@font-face{font-family:redacted-script-light;font-style:normal;font-weight:400;src:url(../fonts/redacted-script-light.eot);src:url(../fonts/redacted-script-light.eot#iefix) format("embedded-opentype"),url(../fonts/redacted-script-light.woff) format("woff"),url(../fonts/redacted-script-light.ttf) format("truetype"),url(../fonts/redacted-script-light.svg#redacted-script-light) format("svg")}.redacted-script-light{font-family:redacted-script-light,Helvetica,Arial,Geneva,sans-serif}@font-face{font-family:redacted-script-bold;font-style:normal;font-weight:400;src:url(../fonts/redacted-script-bold.eot);src:url(../fonts/redacted-script-bold.eot#iefix) format("embedded-opentype"),url(../fonts/redacted-script-bold.woff) format("woff"),url(../fonts/redacted-script-bold.ttf) format("truetype"),url(../fonts/redacted-script-bold.svg#redacted-script-bold) format("svg")}.redacted-script-bold{font-family:redacted-script-bold,Helvetica,Arial,Geneva,sans-serif}.box{background:lightgrey;color:#5a5a5a;border:1px solid #ddd;-o-border-radius:4px;border-radius:4px}.box a{color:inherit}.box.small{font-size:.8em}.box.medium{font-size:1em}.box.large{font-size:1.5em}.box.square{-o-border-radius:0;border-radius:0}.box.round{-o-border-radius:9999em;border-radius:9999em}.box.info{background-color:#d9edf7;color:#3a87ad;border-color:#bce8f1}.box.question{background-color:#f0e6f4;color:#9855b4;border-color:#f1dbfa}.box.alert{background-color:#fcf8e3;color:#c09853;border-color:#fbeee0}.box.warning{background:#ffe5c7;color:#ff971e;border-color:#ffdbb3}.box.error{background:#f2dede;color:#b94a48;border-color:#eed3d7}.box.success{background:#dff0d8;color:#468847;border-color:#d6e9c6}.box.green{border-color:#2ecc71;background-color:#54d98c;color:#fff}.box.blue{border-color:#3498db;background-color:#5faee3;color:#fff}.box.purple{border-color:#9b59b6;background-color:#b07cc6;color:#fff}.box.yellow{border-color:#f1c40f;background-color:#f4d03f;color:#fff}.box.orange{border-color:#e67e22;background-color:#eb9950;color:#fff}.box.red{border-color:#e74c3c;background-color:#ed7669;color:#fff}.box.pink{border-color:#f02475;background-color:#f35493;color:#fff}.box.turquoise{border-color:#1abc9c;background-color:#28e1bd;color:#fff}.box.asphalt{border-color:#34495e;background-color:#46627f;color:#fff}.box.charcoal{border-color:#383636;background-color:#524f4f;color:#fff}.box.white{border-color:#fff;background-color:#fff;color:#111}.box.black{border-color:#111;background-color:#2b2b2b;color:#fff}button,input[type=submit],input[type=button],input[type=reset],.button,a.button,a.button:visited,[role~=button]{word-wrap:break-word;padding:.25em 1em;background:lightgrey;border:1px solid #c6c6c6;color:#5a5a5a;font-family:Helvetica,Arial,Geneva,sans-serif;font-weight:700;text-decoration:none;cursor:pointer;line-height:1.5;width:auto;height:auto;text-align:center;-webkit-transition:background-color .25s ease-out;transition:background-color .25s ease-out;-o-border-radius:4px;border-radius:4px}button:visited,input[type=submit]:visited,input[type=button]:visited,input[type=reset]:visited,.button:visited,[role~=button]:visited{color:#5a5a5a}button.disabled,input.disabled[type=submit],input.disabled[type=button],input.disabled[type=reset],.disabled.button,a.disabled.button:visited,.disabled[role~=button],button[disabled=disabled],input[disabled=disabled][type=submit],input[disabled=disabled][type=button],input[disabled=disabled][type=reset],[disabled=disabled].button,a[disabled=disabled].button:visited,[disabled=disabled][role~=button]{color:#f3f3f3;background:#d9d9d9;border-color:#ccc;cursor:not-allowed;text-decoration:none;outline:0}button.disabled:hover,input.disabled[type=submit]:hover,input.disabled[type=button]:hover,input.disabled[type=reset]:hover,.disabled.button:hover,.disabled[role~=button]:hover,button.disabled:focus,input.disabled[type=submit]:focus,input.disabled[type=button]:focus,input.disabled[type=reset]:focus,.disabled.button:focus,.disabled[role~=button]:focus,button[disabled=disabled]:hover,input[disabled=disabled][type=submit]:hover,input[disabled=disabled][type=button]:hover,input[disabled=disabled][type=reset]:hover,[disabled=disabled].button:hover,[disabled=disabled][role~=button]:hover,button[disabled=disabled]:focus,input[disabled=disabled][type=submit]:focus,input[disabled=disabled][type=button]:focus,input[disabled=disabled][type=reset]:focus,[disabled=disabled].button:focus,[disabled=disabled][role~=button]:focus{color:#f3f3f3;background:#d9d9d9;border-color:#ccc;cursor:not-allowed;text-decoration:none;outline:0}button.on,input[type=submit].on,input[type=button].on,input[type=reset].on,.button.on,a.button.on,a.button.on:visited,ul.button-group li a.on,ul.button-group li button.on,ol.button-group li a.on,ol.button-group li button.on,[role~=button].on,button.active,input.active[type=submit],input.active[type=button],input.active[type=reset],.active.button,.active[role~=button],button[aria-selected=true],input[aria-selected=true][type=submit],input[aria-selected=true][type=button],input[aria-selected=true][type=reset],[aria-selected=true].button,a[aria-selected=true].button:visited,[aria-selected=true][role~=button],button[aria-pressed=true],input[aria-pressed=true][type=submit],input[aria-pressed=true][type=button],input[aria-pressed=true][type=reset],[aria-pressed=true].button,a[aria-pressed=true].button:visited,[aria-pressed=true][role~=button],button:hover,input[type=submit]:hover,input[type=button]:hover,input[type=reset]:hover,.button:hover,a.button:hover:visited,[role~=button]:hover,button:focus,input[type=submit]:focus,input[type=button]:focus,input[type=reset]:focus,.button:focus,a.button:focus:visited,[role~=button]:focus,a.button.active,ul.button-group li a.active,ul.button-group li button.active,ol.button-group li a.active,ol.button-group li button.active,button.selected,input[type=submit].selected,input[type=button].selected,input[type=reset].selected,.button.selected,a.button.selected,a.button.selected:visited,ul.button-group li a.selected,ul.button-group li button.selected,ol.button-group li a.selected,ol.button-group li button.selected,[role~=button][aria-selected],[role~=button].selected,button.pressed,input[type=submit].pressed,input[type=button].pressed,input[type=reset].pressed,.button.pressed,a.button.pressed,a.button.pressed:visited,ul.button-group li a.pressed,ul.button-group li button.pressed,ol.button-group li a.pressed,ol.button-group li button.pressed,[role~=button][aria-pressed],[role~=button].pressed{background:#c6c6c6;text-decoration:none;color:#616161}button.small,input[type=submit].small,input[type=button].small,input[type=reset].small,.button.small,a.button.small,a.button.small:visited,ul.button-group li a.small,ul.button-group li button.small,ol.button-group li a.small,ol.button-group li button.small,[role~=button].small{font-size:.8em}button.large,input[type=submit].large,input[type=button].large,input[type=reset].large,.button.large,a.button.large,a.button.large:visited,ul.button-group li a.large,ul.button-group li button.large,ol.button-group li a.large,ol.button-group li button.large,[role~=button].large{font-size:1.5em}button.square,input[type=submit].square,input[type=button].square,input[type=reset].square,.button.square,a.button.square,a.button.square:visited,ul.button-group li a.square,ul.button-group li button.square,ol.button-group li a.square,ol.button-group li button.square,[role~=button].square{-o-border-radius:0;border-radius:0}button.round,input[type=submit].round,input[type=button].round,input[type=reset].round,.button.round,a.button.round,a.button.round:visited,ul.button-group li a.round,ul.button-group li button.round,ol.button-group li a.round,ol.button-group li button.round,[role~=button].round{-o-border-radius:9999em;border-radius:9999em}button.green,input[type=submit].green,input[type=button].green,input[type=reset].green,.button.green,a.button.green,a.button.green:visited,ul.button-group li a.green,ul.button-group li button.green,ol.button-group li a.green,ol.button-group li button.green,[role~=button].green{background-color:#2ecc71;color:#fff;border-color:#29b765}button.active.green,input.active[type=submit].green,input.active[type=button].green,input.active[type=reset].green,.active.button.green,a.active.button.green:visited,ul.button-group li a.active.green,ul.button-group li button.active.green,ol.button-group li a.active.green,ol.button-group li button.active.green,.active[role~=button].green,button[aria-selected=true].green,input[aria-selected=true][type=submit].green,input[aria-selected=true][type=button].green,input[aria-selected=true][type=reset].green,[aria-selected=true].button.green,a[aria-selected=true].button.green:visited,ul.button-group li a[aria-selected=true].green,ul.button-group li button[aria-selected=true].green,ol.button-group li a[aria-selected=true].green,ol.button-group li button[aria-selected=true].green,[aria-selected=true][role~=button].green,button[aria-pressed=true].green,input[aria-pressed=true][type=submit].green,input[aria-pressed=true][type=button].green,input[aria-pressed=true][type=reset].green,[aria-pressed=true].button.green,a[aria-pressed=true].button.green:visited,ul.button-group li a[aria-pressed=true].green,ul.button-group li button[aria-pressed=true].green,ol.button-group li a[aria-pressed=true].green,ol.button-group li button[aria-pressed=true].green,[aria-pressed=true][role~=button].green,button.green:hover,input[type=submit].green:hover,input[type=button].green:hover,input[type=reset].green:hover,.button.green:hover,a.button.green:hover:visited,ul.button-group li a.green:hover,ul.button-group li button.green:hover,ol.button-group li a.green:hover,ol.button-group li button.green:hover,[role~=button].green:hover,button.green:focus,input[type=submit].green:focus,input[type=button].green:focus,input[type=reset].green:focus,.button.green:focus,a.button.green:focus:visited,ul.button-group li a.green:focus,ul.button-group li button.green:focus,ol.button-group li a.green:focus,ol.button-group li button.green:focus,[role~=button].green:focus{background:#29b765;border-color:#25a25a}button.disabled.green,input.disabled[type=submit].green,input.disabled[type=button].green,input.disabled[type=reset].green,.disabled.button.green,a.disabled.button.green:visited,ul.button-group li a.disabled.green,ul.button-group li button.disabled.green,ol.button-group li a.disabled.green,ol.button-group li button.disabled.green,.disabled[role~=button].green,button[disabled=disabled].green,input[disabled=disabled][type=submit].green,input[disabled=disabled][type=button].green,input[disabled=disabled][type=reset].green,[disabled=disabled].button.green,a[disabled=disabled].button.green:visited,ul.button-group li a[disabled=disabled].green,ul.button-group li button[disabled=disabled].green,ol.button-group li a[disabled=disabled].green,ol.button-group li button[disabled=disabled].green,[disabled=disabled][role~=button].green{color:#a0d9b8;background:#7ccb9d;border-color:#69c490;cursor:not-allowed;text-decoration:none;outline:0}button.blue,input[type=submit].blue,input[type=button].blue,input[type=reset].blue,.button.blue,a.button.blue,a.button.blue:visited,ul.button-group li a.blue,ul.button-group li button.blue,ol.button-group li a.blue,ol.button-group li button.blue,[role~=button].blue{background-color:#3498db;color:#fff;border-color:#258cd1}button.active.blue,input.active[type=submit].blue,input.active[type=button].blue,input.active[type=reset].blue,.active.button.blue,a.active.button.blue:visited,ul.button-group li a.active.blue,ul.button-group li button.active.blue,ol.button-group li a.active.blue,ol.button-group li button.active.blue,.active[role~=button].blue,button[aria-selected=true].blue,input[aria-selected=true][type=submit].blue,input[aria-selected=true][type=button].blue,input[aria-selected=true][type=reset].blue,[aria-selected=true].button.blue,a[aria-selected=true].button.blue:visited,ul.button-group li a[aria-selected=true].blue,ul.button-group li button[aria-selected=true].blue,ol.button-group li a[aria-selected=true].blue,ol.button-group li button[aria-selected=true].blue,[aria-selected=true][role~=button].blue,button[aria-pressed=true].blue,input[aria-pressed=true][type=submit].blue,input[aria-pressed=true][type=button].blue,input[aria-pressed=true][type=reset].blue,[aria-pressed=true].button.blue,a[aria-pressed=true].button.blue:visited,ul.button-group li a[aria-pressed=true].blue,ul.button-group li button[aria-pressed=true].blue,ol.button-group li a[aria-pressed=true].blue,ol.button-group li button[aria-pressed=true].blue,[aria-pressed=true][role~=button].blue,button.blue:hover,input[type=submit].blue:hover,input[type=button].blue:hover,input[type=reset].blue:hover,.button.blue:hover,a.button.blue:hover:visited,ul.button-group li a.blue:hover,ul.button-group li button.blue:hover,ol.button-group li a.blue:hover,ol.button-group li button.blue:hover,[role~=button].blue:hover,button.blue:focus,input[type=submit].blue:focus,input[type=button].blue:focus,input[type=reset].blue:focus,.button.blue:focus,a.button.blue:focus:visited,ul.button-group li a.blue:focus,ul.button-group li button.blue:focus,ol.button-group li a.blue:focus,ol.button-group li button.blue:focus,[role~=button].blue:focus{background:#258cd1;border-color:#217dbb}button.disabled.blue,input.disabled[type=submit].blue,input.disabled[type=button].blue,input.disabled[type=reset].blue,.disabled.button.blue,a.disabled.button.blue:visited,ul.button-group li a.disabled.blue,ul.button-group li button.disabled.blue,ol.button-group li a.disabled.blue,ol.button-group li button.disabled.blue,.disabled[role~=button].blue,button[disabled=disabled].blue,input[disabled=disabled][type=submit].blue,input[disabled=disabled][type=button].blue,input[disabled=disabled][type=reset].blue,[disabled=disabled].button.blue,a[disabled=disabled].button.blue:visited,ul.button-group li a[disabled=disabled].blue,ul.button-group li button[disabled=disabled].blue,ol.button-group li a[disabled=disabled].blue,ol.button-group li button[disabled=disabled].blue,[disabled=disabled][role~=button].blue{color:#abcde3;background:#85b6d6;border-color:#72aad0;cursor:not-allowed;text-decoration:none;outline:0}button.purple,input[type=submit].purple,input[type=button].purple,input[type=reset].purple,.button.purple,a.button.purple,a.button.purple:visited,ul.button-group li a.purple,ul.button-group li button.purple,ol.button-group li a.purple,ol.button-group li button.purple,[role~=button].purple{background-color:#9b59b6;color:#fff;border-color:#8f4bab}button.active.purple,input.active[type=submit].purple,input.active[type=button].purple,input.active[type=reset].purple,.active.button.purple,a.active.button.purple:visited,ul.button-group li a.active.purple,ul.button-group li button.active.purple,ol.button-group li a.active.purple,ol.button-group li button.active.purple,.active[role~=button].purple,button[aria-selected=true].purple,input[aria-selected=true][type=submit].purple,input[aria-selected=true][type=button].purple,input[aria-selected=true][type=reset].purple,[aria-selected=true].button.purple,a[aria-selected=true].button.purple:visited,ul.button-group li a[aria-selected=true].purple,ul.button-group li button[aria-selected=true].purple,ol.button-group li a[aria-selected=true].purple,ol.button-group li button[aria-selected=true].purple,[aria-selected=true][role~=button].purple,button[aria-pressed=true].purple,input[aria-pressed=true][type=submit].purple,input[aria-pressed=true][type=button].purple,input[aria-pressed=true][type=reset].purple,[aria-pressed=true].button.purple,a[aria-pressed=true].button.purple:visited,ul.button-group li a[aria-pressed=true].purple,ul.button-group li button[aria-pressed=true].purple,ol.button-group li a[aria-pressed=true].purple,ol.button-group li button[aria-pressed=true].purple,[aria-pressed=true][role~=button].purple,button.purple:hover,input[type=submit].purple:hover,input[type=button].purple:hover,input[type=reset].purple:hover,.button.purple:hover,a.button.purple:hover:visited,ul.button-group li a.purple:hover,ul.button-group li button.purple:hover,ol.button-group li a.purple:hover,ol.button-group li button.purple:hover,[role~=button].purple:hover,button.purple:focus,input[type=submit].purple:focus,input[type=button].purple:focus,input[type=reset].purple:focus,.button.purple:focus,a.button.purple:focus:visited,ul.button-group li a.purple:focus,ul.button-group li button.purple:focus,ol.button-group li a.purple:focus,ol.button-group li button.purple:focus,[role~=button].purple:focus{background:#8f4bab;border-color:#804399}button.disabled.purple,input.disabled[type=submit].purple,input.disabled[type=button].purple,input.disabled[type=reset].purple,.disabled.button.purple,a.disabled.button.purple:visited,ul.button-group li a.disabled.purple,ul.button-group li button.disabled.purple,ol.button-group li a.disabled.purple,ol.button-group li button.disabled.purple,.disabled[role~=button].purple,button[disabled=disabled].purple,input[disabled=disabled][type=submit].purple,input[disabled=disabled][type=button].purple,input[disabled=disabled][type=reset].purple,[disabled=disabled].button.purple,a[disabled=disabled].button.purple:visited,ul.button-group li a[disabled=disabled].purple,ul.button-group li button[disabled=disabled].purple,ol.button-group li a[disabled=disabled].purple,ol.button-group li button[disabled=disabled].purple,[disabled=disabled][role~=button].purple{color:#ccbdd2;background:#b49ebd;border-color:#a88fb3;cursor:not-allowed;text-decoration:none;outline:0}button.yellow,input[type=submit].yellow,input[type=button].yellow,input[type=reset].yellow,.button.yellow,a.button.yellow,a.button.yellow:visited,ul.button-group li a.yellow,ul.button-group li button.yellow,ol.button-group li a.yellow,ol.button-group li button.yellow,[role~=button].yellow{background-color:#f1c40f;color:#fff;border-color:#dab10d}button.active.yellow,input.active[type=submit].yellow,input.active[type=button].yellow,input.active[type=reset].yellow,.active.button.yellow,a.active.button.yellow:visited,ul.button-group li a.active.yellow,ul.button-group li button.active.yellow,ol.button-group li a.active.yellow,ol.button-group li button.active.yellow,.active[role~=button].yellow,button[aria-selected=true].yellow,input[aria-selected=true][type=submit].yellow,input[aria-selected=true][type=button].yellow,input[aria-selected=true][type=reset].yellow,[aria-selected=true].button.yellow,a[aria-selected=true].button.yellow:visited,ul.button-group li a[aria-selected=true].yellow,ul.button-group li button[aria-selected=true].yellow,ol.button-group li a[aria-selected=true].yellow,ol.button-group li button[aria-selected=true].yellow,[aria-selected=true][role~=button].yellow,button[aria-pressed=true].yellow,input[aria-pressed=true][type=submit].yellow,input[aria-pressed=true][type=button].yellow,input[aria-pressed=true][type=reset].yellow,[aria-pressed=true].button.yellow,a[aria-pressed=true].button.yellow:visited,ul.button-group li a[aria-pressed=true].yellow,ul.button-group li button[aria-pressed=true].yellow,ol.button-group li a[aria-pressed=true].yellow,ol.button-group li button[aria-pressed=true].yellow,[aria-pressed=true][role~=button].yellow,button.yellow:hover,input[type=submit].yellow:hover,input[type=button].yellow:hover,input[type=reset].yellow:hover,.button.yellow:hover,a.button.yellow:hover:visited,ul.button-group li a.yellow:hover,ul.button-group li button.yellow:hover,ol.button-group li a.yellow:hover,ol.button-group li button.yellow:hover,[role~=button].yellow:hover,button.yellow:focus,input[type=submit].yellow:focus,input[type=button].yellow:focus,input[type=reset].yellow:focus,.button.yellow:focus,a.button.yellow:focus:visited,ul.button-group li a.yellow:focus,ul.button-group li button.yellow:focus,ol.button-group li a.yellow:focus,ol.button-group li button.yellow:focus,[role~=button].yellow:focus{background:#dab10d;border-color:#c29d0b}button.disabled.yellow,input.disabled[type=submit].yellow,input.disabled[type=button].yellow,input.disabled[type=reset].yellow,.disabled.button.yellow,a.disabled.button.yellow:visited,ul.button-group li a.disabled.yellow,ul.button-group li button.disabled.yellow,ol.button-group li a.disabled.yellow,ol.button-group li button.disabled.yellow,.disabled[role~=button].yellow,button[disabled=disabled].yellow,input[disabled=disabled][type=submit].yellow,input[disabled=disabled][type=button].yellow,input[disabled=disabled][type=reset].yellow,[disabled=disabled].button.yellow,a[disabled=disabled].button.yellow:visited,ul.button-group li a[disabled=disabled].yellow,ul.button-group li button[disabled=disabled].yellow,ol.button-group li a[disabled=disabled].yellow,ol.button-group li button[disabled=disabled].yellow,[disabled=disabled][role~=button].yellow{color:#ebda94;background:#e3cb69;border-color:#e0c453;cursor:not-allowed;text-decoration:none;outline:0}button.orange,input[type=submit].orange,input[type=button].orange,input[type=reset].orange,.button.orange,a.button.orange,a.button.orange:visited,ul.button-group li a.orange,ul.button-group li button.orange,ol.button-group li a.orange,ol.button-group li button.orange,[role~=button].orange{background-color:#e67e22;color:#fff;border-color:#d67118}button.active.orange,input.active[type=submit].orange,input.active[type=button].orange,input.active[type=reset].orange,.active.button.orange,a.active.button.orange:visited,ul.button-group li a.active.orange,ul.button-group li button.active.orange,ol.button-group li a.active.orange,ol.button-group li button.active.orange,.active[role~=button].orange,button[aria-selected=true].orange,input[aria-selected=true][type=submit].orange,input[aria-selected=true][type=button].orange,input[aria-selected=true][type=reset].orange,[aria-selected=true].button.orange,a[aria-selected=true].button.orange:visited,ul.button-group li a[aria-selected=true].orange,ul.button-group li button[aria-selected=true].orange,ol.button-group li a[aria-selected=true].orange,ol.button-group li button[aria-selected=true].orange,[aria-selected=true][role~=button].orange,button[aria-pressed=true].orange,input[aria-pressed=true][type=submit].orange,input[aria-pressed=true][type=button].orange,input[aria-pressed=true][type=reset].orange,[aria-pressed=true].button.orange,a[aria-pressed=true].button.orange:visited,ul.button-group li a[aria-pressed=true].orange,ul.button-group li button[aria-pressed=true].orange,ol.button-group li a[aria-pressed=true].orange,ol.button-group li button[aria-pressed=true].orange,[aria-pressed=true][role~=button].orange,button.orange:hover,input[type=submit].orange:hover,input[type=button].orange:hover,input[type=reset].orange:hover,.button.orange:hover,a.button.orange:hover:visited,ul.button-group li a.orange:hover,ul.button-group li button.orange:hover,ol.button-group li a.orange:hover,ol.button-group li button.orange:hover,[role~=button].orange:hover,button.orange:focus,input[type=submit].orange:focus,input[type=button].orange:focus,input[type=reset].orange:focus,.button.orange:focus,a.button.orange:focus:visited,ul.button-group li a.orange:focus,ul.button-group li button.orange:focus,ol.button-group li a.orange:focus,ol.button-group li button.orange:focus,[role~=button].orange:focus{background:#d67118;border-color:#bf6516}button.disabled.orange,input.disabled[type=submit].orange,input.disabled[type=button].orange,input.disabled[type=reset].orange,.disabled.button.orange,a.disabled.button.orange:visited,ul.button-group li a.disabled.orange,ul.button-group li button.disabled.orange,ol.button-group li a.disabled.orange,ol.button-group li button.disabled.orange,.disabled[role~=button].orange,button[disabled=disabled].orange,input[disabled=disabled][type=submit].orange,input[disabled=disabled][type=button].orange,input[disabled=disabled][type=reset].orange,[disabled=disabled].button.orange,a[disabled=disabled].button.orange:visited,ul.button-group li a[disabled=disabled].orange,ul.button-group li button[disabled=disabled].orange,ol.button-group li a[disabled=disabled].orange,ol.button-group li button[disabled=disabled].orange,[disabled=disabled][role~=button].orange{color:#e7c2a0;background:#dda778;border-color:#d89a63;cursor:not-allowed;text-decoration:none;outline:0}button.red,input[type=submit].red,input[type=button].red,input[type=reset].red,.button.red,a.button.red,a.button.red:visited,ul.button-group li a.red,ul.button-group li button.red,ol.button-group li a.red,ol.button-group li button.red,[role~=button].red{background-color:#e74c3c;color:#fff;border-color:#e43725}button.active.red,input.active[type=submit].red,input.active[type=button].red,input.active[type=reset].red,.active.button.red,a.active.button.red:visited,ul.button-group li a.active.red,ul.button-group li button.active.red,ol.button-group li a.active.red,ol.button-group li button.active.red,.active[role~=button].red,button[aria-selected=true].red,input[aria-selected=true][type=submit].red,input[aria-selected=true][type=button].red,input[aria-selected=true][type=reset].red,[aria-selected=true].button.red,a[aria-selected=true].button.red:visited,ul.button-group li a[aria-selected=true].red,ul.button-group li button[aria-selected=true].red,ol.button-group li a[aria-selected=true].red,ol.button-group li button[aria-selected=true].red,[aria-selected=true][role~=button].red,button[aria-pressed=true].red,input[aria-pressed=true][type=submit].red,input[aria-pressed=true][type=button].red,input[aria-pressed=true][type=reset].red,[aria-pressed=true].button.red,a[aria-pressed=true].button.red:visited,ul.button-group li a[aria-pressed=true].red,ul.button-group li button[aria-pressed=true].red,ol.button-group li a[aria-pressed=true].red,ol.button-group li button[aria-pressed=true].red,[aria-pressed=true][role~=button].red,button.red:hover,input[type=submit].red:hover,input[type=button].red:hover,input[type=reset].red:hover,.button.red:hover,a.button.red:hover:visited,ul.button-group li a.red:hover,ul.button-group li button.red:hover,ol.button-group li a.red:hover,ol.button-group li button.red:hover,[role~=button].red:hover,button.red:focus,input[type=submit].red:focus,input[type=button].red:focus,input[type=reset].red:focus,.button.red:focus,a.button.red:focus:visited,ul.button-group li a.red:focus,ul.button-group li button.red:focus,ol.button-group li a.red:focus,ol.button-group li button.red:focus,[role~=button].red:focus{background:#e43725;border-color:#d62c1a}button.disabled.red,input.disabled[type=submit].red,input.disabled[type=button].red,input.disabled[type=reset].red,.disabled.button.red,a.disabled.button.red:visited,ul.button-group li a.disabled.red,ul.button-group li button.disabled.red,ol.button-group li a.disabled.red,ol.button-group li button.disabled.red,.disabled[role~=button].red,button[disabled=disabled].red,input[disabled=disabled][type=submit].red,input[disabled=disabled][type=button].red,input[disabled=disabled][type=reset].red,[disabled=disabled].button.red,a[disabled=disabled].button.red:visited,ul.button-group li a[disabled=disabled].red,ul.button-group li button[disabled=disabled].red,ol.button-group li a[disabled=disabled].red,ol.button-group li button[disabled=disabled].red,[disabled=disabled][role~=button].red{color:#ecbcb7;background:#e1968e;border-color:#dc837a;cursor:not-allowed;text-decoration:none;outline:0}button.pink,input[type=submit].pink,input[type=button].pink,input[type=reset].pink,.button.pink,a.button.pink,a.button.pink:visited,ul.button-group li a.pink,ul.button-group li button.pink,ol.button-group li a.pink,ol.button-group li button.pink,[role~=button].pink{background-color:#f02475;color:#fff;border-color:#ea1067}button.active.pink,input.active[type=submit].pink,input.active[type=button].pink,input.active[type=reset].pink,.active.button.pink,a.active.button.pink:visited,ul.button-group li a.active.pink,ul.button-group li button.active.pink,ol.button-group li a.active.pink,ol.button-group li button.active.pink,.active[role~=button].pink,button[aria-selected=true].pink,input[aria-selected=true][type=submit].pink,input[aria-selected=true][type=button].pink,input[aria-selected=true][type=reset].pink,[aria-selected=true].button.pink,a[aria-selected=true].button.pink:visited,ul.button-group li a[aria-selected=true].pink,ul.button-group li button[aria-selected=true].pink,ol.button-group li a[aria-selected=true].pink,ol.button-group li button[aria-selected=true].pink,[aria-selected=true][role~=button].pink,button[aria-pressed=true].pink,input[aria-pressed=true][type=submit].pink,input[aria-pressed=true][type=button].pink,input[aria-pressed=true][type=reset].pink,[aria-pressed=true].button.pink,a[aria-pressed=true].button.pink:visited,ul.button-group li a[aria-pressed=true].pink,ul.button-group li button[aria-pressed=true].pink,ol.button-group li a[aria-pressed=true].pink,ol.button-group li button[aria-pressed=true].pink,[aria-pressed=true][role~=button].pink,button.pink:hover,input[type=submit].pink:hover,input[type=button].pink:hover,input[type=reset].pink:hover,.button.pink:hover,a.button.pink:hover:visited,ul.button-group li a.pink:hover,ul.button-group li button.pink:hover,ol.button-group li a.pink:hover,ol.button-group li button.pink:hover,[role~=button].pink:hover,button.pink:focus,input[type=submit].pink:focus,input[type=button].pink:focus,input[type=reset].pink:focus,.button.pink:focus,a.button.pink:focus:visited,ul.button-group li a.pink:focus,ul.button-group li button.pink:focus,ol.button-group li a.pink:focus,ol.button-group li button.pink:focus,[role~=button].pink:focus{background:#ea1067;border-color:#d30e5c}button.disabled.pink,input.disabled[type=submit].pink,input.disabled[type=button].pink,input.disabled[type=reset].pink,.disabled.button.pink,a.disabled.button.pink:visited,ul.button-group li a.disabled.pink,ul.button-group li button.disabled.pink,ol.button-group li a.disabled.pink,ol.button-group li button.disabled.pink,.disabled[role~=button].pink,button[disabled=disabled].pink,input[disabled=disabled][type=submit].pink,input[disabled=disabled][type=button].pink,input[disabled=disabled][type=reset].pink,[disabled=disabled].button.pink,a[disabled=disabled].button.pink:visited,ul.button-group li a[disabled=disabled].pink,ul.button-group li button[disabled=disabled].pink,ol.button-group li a[disabled=disabled].pink,ol.button-group li button[disabled=disabled].pink,[disabled=disabled][role~=button].pink{color:#eea6c2;background:#e57ba5;border-color:#e16697;cursor:not-allowed;text-decoration:none;outline:0}button.turquoise,input[type=submit].turquoise,input[type=button].turquoise,input[type=reset].turquoise,.button.turquoise,a.button.turquoise,a.button.turquoise:visited,ul.button-group li a.turquoise,ul.button-group li button.turquoise,ol.button-group li a.turquoise,ol.button-group li button.turquoise,[role~=button].turquoise{background-color:#1abc9c;color:#fff;border-color:#17a689}button.active.turquoise,input.active[type=submit].turquoise,input.active[type=button].turquoise,input.active[type=reset].turquoise,.active.button.turquoise,a.active.button.turquoise:visited,ul.button-group li a.active.turquoise,ul.button-group li button.active.turquoise,ol.button-group li a.active.turquoise,ol.button-group li button.active.turquoise,.active[role~=button].turquoise,button[aria-selected=true].turquoise,input[aria-selected=true][type=submit].turquoise,input[aria-selected=true][type=button].turquoise,input[aria-selected=true][type=reset].turquoise,[aria-selected=true].button.turquoise,a[aria-selected=true].button.turquoise:visited,ul.button-group li a[aria-selected=true].turquoise,ul.button-group li button[aria-selected=true].turquoise,ol.button-group li a[aria-selected=true].turquoise,ol.button-group li button[aria-selected=true].turquoise,[aria-selected=true][role~=button].turquoise,button[aria-pressed=true].turquoise,input[aria-pressed=true][type=submit].turquoise,input[aria-pressed=true][type=button].turquoise,input[aria-pressed=true][type=reset].turquoise,[aria-pressed=true].button.turquoise,a[aria-pressed=true].button.turquoise:visited,ul.button-group li a[aria-pressed=true].turquoise,ul.button-group li button[aria-pressed=true].turquoise,ol.button-group li a[aria-pressed=true].turquoise,ol.button-group li button[aria-pressed=true].turquoise,[aria-pressed=true][role~=button].turquoise,button.turquoise:hover,input[type=submit].turquoise:hover,input[type=button].turquoise:hover,input[type=reset].turquoise:hover,.button.turquoise:hover,a.button.turquoise:hover:visited,ul.button-group li a.turquoise:hover,ul.button-group li button.turquoise:hover,ol.button-group li a.turquoise:hover,ol.button-group li button.turquoise:hover,[role~=button].turquoise:hover,button.turquoise:focus,input[type=submit].turquoise:focus,input[type=button].turquoise:focus,input[type=reset].turquoise:focus,.button.turquoise:focus,a.button.turquoise:focus:visited,ul.button-group li a.turquoise:focus,ul.button-group li button.turquoise:focus,ol.button-group li a.turquoise:focus,ol.button-group li button.turquoise:focus,[role~=button].turquoise:focus{background:#17a689;border-color:#148f77}button.disabled.turquoise,input.disabled[type=submit].turquoise,input.disabled[type=button].turquoise,input.disabled[type=reset].turquoise,.disabled.button.turquoise,a.disabled.button.turquoise:visited,ul.button-group li a.disabled.turquoise,ul.button-group li button.disabled.turquoise,ol.button-group li a.disabled.turquoise,ol.button-group li button.disabled.turquoise,.disabled[role~=button].turquoise,button[disabled=disabled].turquoise,input[disabled=disabled][type=submit].turquoise,input[disabled=disabled][type=button].turquoise,input[disabled=disabled][type=reset].turquoise,[disabled=disabled].button.turquoise,a[disabled=disabled].button.turquoise:visited,ul.button-group li a[disabled=disabled].turquoise,ul.button-group li button[disabled=disabled].turquoise,ol.button-group li a[disabled=disabled].turquoise,ol.button-group li button[disabled=disabled].turquoise,[disabled=disabled][role~=button].turquoise{color:#7cdac7;background:#54ceb6;border-color:#40c9ae;cursor:not-allowed;text-decoration:none;outline:0}button.asphalt,input[type=submit].asphalt,input[type=button].asphalt,input[type=reset].asphalt,.button.asphalt,a.button.asphalt,a.button.asphalt:visited,ul.button-group li a.asphalt,ul.button-group li button.asphalt,ol.button-group li a.asphalt,ol.button-group li button.asphalt,[role~=button].asphalt{background-color:#34495e;color:#fff;border-color:#2b3c4e}button.active.asphalt,input.active[type=submit].asphalt,input.active[type=button].asphalt,input.active[type=reset].asphalt,.active.button.asphalt,a.active.button.asphalt:visited,ul.button-group li a.active.asphalt,ul.button-group li button.active.asphalt,ol.button-group li a.active.asphalt,ol.button-group li button.active.asphalt,.active[role~=button].asphalt,button[aria-selected=true].asphalt,input[aria-selected=true][type=submit].asphalt,input[aria-selected=true][type=button].asphalt,input[aria-selected=true][type=reset].asphalt,[aria-selected=true].button.asphalt,a[aria-selected=true].button.asphalt:visited,ul.button-group li a[aria-selected=true].asphalt,ul.button-group li button[aria-selected=true].asphalt,ol.button-group li a[aria-selected=true].asphalt,ol.button-group li button[aria-selected=true].asphalt,[aria-selected=true][role~=button].asphalt,button[aria-pressed=true].asphalt,input[aria-pressed=true][type=submit].asphalt,input[aria-pressed=true][type=button].asphalt,input[aria-pressed=true][type=reset].asphalt,[aria-pressed=true].button.asphalt,a[aria-pressed=true].button.asphalt:visited,ul.button-group li a[aria-pressed=true].asphalt,ul.button-group li button[aria-pressed=true].asphalt,ol.button-group li a[aria-pressed=true].asphalt,ol.button-group li button[aria-pressed=true].asphalt,[aria-pressed=true][role~=button].asphalt,button.asphalt:hover,input[type=submit].asphalt:hover,input[type=button].asphalt:hover,input[type=reset].asphalt:hover,.button.asphalt:hover,a.button.asphalt:hover:visited,ul.button-group li a.asphalt:hover,ul.button-group li button.asphalt:hover,ol.button-group li a.asphalt:hover,ol.button-group li button.asphalt:hover,[role~=button].asphalt:hover,button.asphalt:focus,input[type=submit].asphalt:focus,input[type=button].asphalt:focus,input[type=reset].asphalt:focus,.button.asphalt:focus,a.button.asphalt:focus:visited,ul.button-group li a.asphalt:focus,ul.button-group li button.asphalt:focus,ol.button-group li a.asphalt:focus,ol.button-group li button.asphalt:focus,[role~=button].asphalt:focus{background:#2b3c4e;border-color:#222f3d}button.disabled.asphalt,input.disabled[type=submit].asphalt,input.disabled[type=button].asphalt,input.disabled[type=reset].asphalt,.disabled.button.asphalt,a.disabled.button.asphalt:visited,ul.button-group li a.disabled.asphalt,ul.button-group li button.disabled.asphalt,ol.button-group li a.disabled.asphalt,ol.button-group li button.disabled.asphalt,.disabled[role~=button].asphalt,button[disabled=disabled].asphalt,input[disabled=disabled][type=submit].asphalt,input[disabled=disabled][type=button].asphalt,input[disabled=disabled][type=reset].asphalt,[disabled=disabled].button.asphalt,a[disabled=disabled].button.asphalt:visited,ul.button-group li a[disabled=disabled].asphalt,ul.button-group li button[disabled=disabled].asphalt,ol.button-group li a[disabled=disabled].asphalt,ol.button-group li button[disabled=disabled].asphalt,[disabled=disabled][role~=button].asphalt{color:#7e8993;background:#656f79;border-color:#5a626b;cursor:not-allowed;text-decoration:none;outline:0}button.charcoal,input[type=submit].charcoal,input[type=button].charcoal,input[type=reset].charcoal,.button.charcoal,a.button.charcoal,a.button.charcoal:visited,ul.button-group li a.charcoal,ul.button-group li button.charcoal,ol.button-group li a.charcoal,ol.button-group li button.charcoal,[role~=button].charcoal{background-color:#383636;color:#fff;border-color:#2b2929}button.active.charcoal,input.active[type=submit].charcoal,input.active[type=button].charcoal,input.active[type=reset].charcoal,.active.button.charcoal,a.active.button.charcoal:visited,ul.button-group li a.active.charcoal,ul.button-group li button.active.charcoal,ol.button-group li a.active.charcoal,ol.button-group li button.active.charcoal,.active[role~=button].charcoal,button[aria-selected=true].charcoal,input[aria-selected=true][type=submit].charcoal,input[aria-selected=true][type=button].charcoal,input[aria-selected=true][type=reset].charcoal,[aria-selected=true].button.charcoal,a[aria-selected=true].button.charcoal:visited,ul.button-group li a[aria-selected=true].charcoal,ul.button-group li button[aria-selected=true].charcoal,ol.button-group li a[aria-selected=true].charcoal,ol.button-group li button[aria-selected=true].charcoal,[aria-selected=true][role~=button].charcoal,button[aria-pressed=true].charcoal,input[aria-pressed=true][type=submit].charcoal,input[aria-pressed=true][type=button].charcoal,input[aria-pressed=true][type=reset].charcoal,[aria-pressed=true].button.charcoal,a[aria-pressed=true].button.charcoal:visited,ul.button-group li a[aria-pressed=true].charcoal,ul.button-group li button[aria-pressed=true].charcoal,ol.button-group li a[aria-pressed=true].charcoal,ol.button-group li button[aria-pressed=true].charcoal,[aria-pressed=true][role~=button].charcoal,button.charcoal:hover,input[type=submit].charcoal:hover,input[type=button].charcoal:hover,input[type=reset].charcoal:hover,.button.charcoal:hover,a.button.charcoal:hover:visited,ul.button-group li a.charcoal:hover,ul.button-group li button.charcoal:hover,ol.button-group li a.charcoal:hover,ol.button-group li button.charcoal:hover,[role~=button].charcoal:hover,button.charcoal:focus,input[type=submit].charcoal:focus,input[type=button].charcoal:focus,input[type=reset].charcoal:focus,.button.charcoal:focus,a.button.charcoal:focus:visited,ul.button-group li a.charcoal:focus,ul.button-group li button.charcoal:focus,ol.button-group li a.charcoal:focus,ol.button-group li button.charcoal:focus,[role~=button].charcoal:focus{background:#2b2929;border-color:#1e1d1d}button.disabled.charcoal,input.disabled[type=submit].charcoal,input.disabled[type=button].charcoal,input.disabled[type=reset].charcoal,.disabled.button.charcoal,a.disabled.button.charcoal:visited,ul.button-group li a.disabled.charcoal,ul.button-group li button.disabled.charcoal,ol.button-group li a.disabled.charcoal,ol.button-group li button.disabled.charcoal,.disabled[role~=button].charcoal,button[disabled=disabled].charcoal,input[disabled=disabled][type=submit].charcoal,input[disabled=disabled][type=button].charcoal,input[disabled=disabled][type=reset].charcoal,[disabled=disabled].button.charcoal,a[disabled=disabled].button.charcoal:visited,ul.button-group li a[disabled=disabled].charcoal,ul.button-group li button[disabled=disabled].charcoal,ol.button-group li a[disabled=disabled].charcoal,ol.button-group li button[disabled=disabled].charcoal,[disabled=disabled][role~=button].charcoal{color:#777;background:#5d5d5d;border-color:#515151;cursor:not-allowed;text-decoration:none;outline:0}button.white,input[type=submit].white,input[type=button].white,input[type=reset].white,.button.white,a.button.white,a.button.white:visited,ul.button-group li a.white,ul.button-group li button.white,ol.button-group li a.white,ol.button-group li button.white,[role~=button].white{background-color:#fff;color:#111;border-color:#f2f2f2}button.active.white,input.active[type=submit].white,input.active[type=button].white,input.active[type=reset].white,.active.button.white,a.active.button.white:visited,ul.button-group li a.active.white,ul.button-group li button.active.white,ol.button-group li a.active.white,ol.button-group li button.active.white,.active[role~=button].white,button[aria-selected=true].white,input[aria-selected=true][type=submit].white,input[aria-selected=true][type=button].white,input[aria-selected=true][type=reset].white,[aria-selected=true].button.white,a[aria-selected=true].button.white:visited,ul.button-group li a[aria-selected=true].white,ul.button-group li button[aria-selected=true].white,ol.button-group li a[aria-selected=true].white,ol.button-group li button[aria-selected=true].white,[aria-selected=true][role~=button].white,button[aria-pressed=true].white,input[aria-pressed=true][type=submit].white,input[aria-pressed=true][type=button].white,input[aria-pressed=true][type=reset].white,[aria-pressed=true].button.white,a[aria-pressed=true].button.white:visited,ul.button-group li a[aria-pressed=true].white,ul.button-group li button[aria-pressed=true].white,ol.button-group li a[aria-pressed=true].white,ol.button-group li button[aria-pressed=true].white,[aria-pressed=true][role~=button].white,button.white:hover,input[type=submit].white:hover,input[type=button].white:hover,input[type=reset].white:hover,.button.white:hover,a.button.white:hover:visited,ul.button-group li a.white:hover,ul.button-group li button.white:hover,ol.button-group li a.white:hover,ol.button-group li button.white:hover,[role~=button].white:hover,button.white:focus,input[type=submit].white:focus,input[type=button].white:focus,input[type=reset].white:focus,.button.white:focus,a.button.white:focus:visited,ul.button-group li a.white:focus,ul.button-group li button.white:focus,ol.button-group li a.white:focus,ol.button-group li button.white:focus,[role~=button].white:focus{background:#f2f2f2;border-color:#e6e6e6}button.disabled.white,input.disabled[type=submit].white,input.disabled[type=button].white,input.disabled[type=reset].white,.disabled.button.white,a.disabled.button.white:visited,ul.button-group li a.disabled.white,ul.button-group li button.disabled.white,ol.button-group li a.disabled.white,ol.button-group li button.disabled.white,.disabled[role~=button].white,button[disabled=disabled].white,input[disabled=disabled][type=submit].white,input[disabled=disabled][type=button].white,input[disabled=disabled][type=reset].white,[disabled=disabled].button.white,a[disabled=disabled].button.white:visited,ul.button-group li a[disabled=disabled].white,ul.button-group li button[disabled=disabled].white,ol.button-group li a[disabled=disabled].white,ol.button-group li button[disabled=disabled].white,[disabled=disabled][role~=button].white{color:#fff;background:#fff;border-color:#fff;cursor:not-allowed;text-decoration:none;outline:0}button.black,input[type=submit].black,input[type=button].black,input[type=reset].black,.button.black,a.button.black,a.button.black:visited,ul.button-group li a.black,ul.button-group li button.black,ol.button-group li a.black,ol.button-group li button.black,[role~=button].black{background-color:#111;color:#fff;border-color:#040404}button.active.black,input.active[type=submit].black,input.active[type=button].black,input.active[type=reset].black,.active.button.black,a.active.button.black:visited,ul.button-group li a.active.black,ul.button-group li button.active.black,ol.button-group li a.active.black,ol.button-group li button.active.black,.active[role~=button].black,button[aria-selected=true].black,input[aria-selected=true][type=submit].black,input[aria-selected=true][type=button].black,input[aria-selected=true][type=reset].black,[aria-selected=true].button.black,a[aria-selected=true].button.black:visited,ul.button-group li a[aria-selected=true].black,ul.button-group li button[aria-selected=true].black,ol.button-group li a[aria-selected=true].black,ol.button-group li button[aria-selected=true].black,[aria-selected=true][role~=button].black,button[aria-pressed=true].black,input[aria-pressed=true][type=submit].black,input[aria-pressed=true][type=button].black,input[aria-pressed=true][type=reset].black,[aria-pressed=true].button.black,a[aria-pressed=true].button.black:visited,ul.button-group li a[aria-pressed=true].black,ul.button-group li button[aria-pressed=true].black,ol.button-group li a[aria-pressed=true].black,ol.button-group li button[aria-pressed=true].black,[aria-pressed=true][role~=button].black,button.black:hover,input[type=submit].black:hover,input[type=button].black:hover,input[type=reset].black:hover,.button.black:hover,a.button.black:hover:visited,ul.button-group li a.black:hover,ul.button-group li button.black:hover,ol.button-group li a.black:hover,ol.button-group li button.black:hover,[role~=button].black:hover,button.black:focus,input[type=submit].black:focus,input[type=button].black:focus,input[type=reset].black:focus,.button.black:focus,a.button.black:focus:visited,ul.button-group li a.black:focus,ul.button-group li button.black:focus,ol.button-group li a.black:focus,ol.button-group li button.black:focus,[role~=button].black:focus{background:#040404;border-color:#000}button.disabled.black,input.disabled[type=submit].black,input.disabled[type=button].black,input.disabled[type=reset].black,.disabled.button.black,a.disabled.button.black:visited,ul.button-group li a.disabled.black,ul.button-group li button.disabled.black,ol.button-group li a.disabled.black,ol.button-group li button.disabled.black,.disabled[role~=button].black,button[disabled=disabled].black,input[disabled=disabled][type=submit].black,input[disabled=disabled][type=button].black,input[disabled=disabled][type=reset].black,[disabled=disabled].button.black,a[disabled=disabled].button.black:visited,ul.button-group li a[disabled=disabled].black,ul.button-group li button[disabled=disabled].black,ol.button-group li a[disabled=disabled].black,ol.button-group li button[disabled=disabled].black,[disabled=disabled][role~=button].black{color:#515151;background:#373737;border-color:#2b2b2b;cursor:not-allowed;text-decoration:none;outline:0}button.info,input[type=submit].info,input[type=button].info,input[type=reset].info,.button.info,a.button.info,a.button.info:visited,ul.button-group li a.info,ul.button-group li button.info,ol.button-group li a.info,ol.button-group li button.info,[role~=button].info{background-color:#309dd4;color:#fff;border-color:#27a1ba}button.active.info,input.active[type=submit].info,input.active[type=button].info,input.active[type=reset].info,.active.button.info,a.active.button.info:visited,ul.button-group li a.active.info,ul.button-group li button.active.info,ol.button-group li a.active.info,ol.button-group li button.active.info,.active[role~=button].info,button[aria-selected=true].info,input[aria-selected=true][type=submit].info,input[aria-selected=true][type=button].info,input[aria-selected=true][type=reset].info,[aria-selected=true].button.info,a[aria-selected=true].button.info:visited,ul.button-group li a[aria-selected=true].info,ul.button-group li button[aria-selected=true].info,ol.button-group li a[aria-selected=true].info,ol.button-group li button[aria-selected=true].info,[aria-selected=true][role~=button].info,button[aria-pressed=true].info,input[aria-pressed=true][type=submit].info,input[aria-pressed=true][type=button].info,input[aria-pressed=true][type=reset].info,[aria-pressed=true].button.info,a[aria-pressed=true].button.info:visited,ul.button-group li a[aria-pressed=true].info,ul.button-group li button[aria-pressed=true].info,ol.button-group li a[aria-pressed=true].info,ol.button-group li button[aria-pressed=true].info,[aria-pressed=true][role~=button].info,button.info:hover,input[type=submit].info:hover,input[type=button].info:hover,input[type=reset].info:hover,.button.info:hover,a.button.info:hover:visited,ul.button-group li a.info:hover,ul.button-group li button.info:hover,ol.button-group li a.info:hover,ol.button-group li button.info:hover,[role~=button].info:hover,button.info:focus,input[type=submit].info:focus,input[type=button].info:focus,input[type=reset].info:focus,.button.info:focus,a.button.info:focus:visited,ul.button-group li a.info:focus,ul.button-group li button.info:focus,ol.button-group li a.info:focus,ol.button-group li button.info:focus,[role~=button].info:focus{background:#2582b1}button.disabled.info,input.disabled[type=submit].info,input.disabled[type=button].info,input.disabled[type=reset].info,.disabled.button.info,a.disabled.button.info:visited,ul.button-group li a.disabled.info,ul.button-group li button.disabled.info,ol.button-group li a.disabled.info,ol.button-group li button.disabled.info,.disabled[role~=button].info,button[disabled=disabled].info,input[disabled=disabled][type=submit].info,input[disabled=disabled][type=button].info,input[disabled=disabled][type=reset].info,[disabled=disabled].button.info,a[disabled=disabled].button.info:visited,ul.button-group li a[disabled=disabled].info,ul.button-group li button[disabled=disabled].info,ol.button-group li a[disabled=disabled].info,ol.button-group li button[disabled=disabled].info,[disabled=disabled][role~=button].info{color:#9dbbca;background:#7ca4b8;border-color:#6b99af;cursor:not-allowed;text-decoration:none;outline:0}button.disabled,input[type=submit].disabled,input[type=button].disabled,input[type=reset].disabled,.button.disabled,a.button.disabled,a.button.disabled:visited,ul.button-group li a.disabled,ul.button-group li button.disabled,ol.button-group li a.disabled,ol.button-group li button.disabled,[role~=button][aria-disabled],[role~=button].disabled,[role~=button][aria-readonly]{background-color:#4d4d4d;color:#ddd;border-color:#000}button.active.disabled,input.active[type=submit].disabled,input.active[type=button].disabled,input.active[type=reset].disabled,.active.button.disabled,a.active.button.disabled:visited,ul.button-group li a.active.disabled,ul.button-group li button.active.disabled,ol.button-group li a.active.disabled,ol.button-group li button.active.disabled,.active[role~=button][aria-disabled],.active[role~=button].disabled,button[aria-selected=true].disabled,input[aria-selected=true][type=submit].disabled,input[aria-selected=true][type=button].disabled,input[aria-selected=true][type=reset].disabled,[aria-selected=true].button.disabled,a[aria-selected=true].button.disabled:visited,ul.button-group li a[aria-selected=true].disabled,ul.button-group li button[aria-selected=true].disabled,ol.button-group li a[aria-selected=true].disabled,ol.button-group li button[aria-selected=true].disabled,[aria-selected=true][role~=button][aria-disabled],[aria-selected=true][role~=button].disabled,button[aria-pressed=true].disabled,input[aria-pressed=true][type=submit].disabled,input[aria-pressed=true][type=button].disabled,input[aria-pressed=true][type=reset].disabled,[aria-pressed=true].button.disabled,a[aria-pressed=true].button.disabled:visited,ul.button-group li a[aria-pressed=true].disabled,ul.button-group li button[aria-pressed=true].disabled,ol.button-group li a[aria-pressed=true].disabled,ol.button-group li button[aria-pressed=true].disabled,[aria-pressed=true][role~=button][aria-disabled],[aria-pressed=true][role~=button].disabled,button.disabled:hover,input[type=submit].disabled:hover,input[type=button].disabled:hover,input[type=reset].disabled:hover,.button.disabled:hover,a.button.disabled:hover:visited,ul.button-group li a.disabled:hover,ul.button-group li button.disabled:hover,ol.button-group li a.disabled:hover,ol.button-group li button.disabled:hover,[role~=button][aria-disabled]:hover,[role~=button].disabled:hover,button.disabled:focus,input[type=submit].disabled:focus,input[type=button].disabled:focus,input[type=reset].disabled:focus,.button.disabled:focus,a.button.disabled:focus:visited,ul.button-group li a.disabled:focus,ul.button-group li button.disabled:focus,ol.button-group li a.disabled:focus,ol.button-group li button.disabled:focus,[role~=button][aria-disabled]:focus,[role~=button].disabled:focus,.active[role~=button][aria-readonly],[aria-selected=true][role~=button][aria-readonly],[aria-pressed=true][role~=button][aria-readonly],[role~=button][aria-readonly]:hover,[role~=button][aria-readonly]:focus{background:#363636}button.disabled,input.disabled[type=submit],input.disabled[type=button],input.disabled[type=reset],.disabled.button,a.disabled.button,a.disabled.button:visited,ul.button-group li a.disabled,ul.button-group li button.disabled,ol.button-group li a.disabled,ol.button-group li button.disabled,.disabled[role~=button],button[disabled=disabled].disabled,input[disabled=disabled][type=submit].disabled,input[disabled=disabled][type=button].disabled,input[disabled=disabled][type=reset].disabled,[disabled=disabled].button.disabled,a[disabled=disabled].button.disabled:visited,ul.button-group li a[disabled=disabled].disabled,ul.button-group li button[disabled=disabled].disabled,ol.button-group li a[disabled=disabled].disabled,ol.button-group li button[disabled=disabled].disabled,[disabled=disabled][role~=button][aria-disabled],[disabled=disabled][role~=button].disabled,.disabled[role~=button][aria-readonly],[disabled=disabled][role~=button][aria-readonly]{color:#f3f3f3;background:#d9d9d9;border-color:#ccc;cursor:not-allowed;text-decoration:none;outline:0}button.alert,input[type=submit].alert,input[type=button].alert,input[type=reset].alert,.button.alert,a.button.alert,a.button.alert:visited,ul.button-group li a.alert,ul.button-group li button.alert,ol.button-group li a.alert,ol.button-group li button.alert,[role~=button].alert{background-color:#e8ca2b;color:#fff;border-color:#ecb06f}button.active.alert,input.active[type=submit].alert,input.active[type=button].alert,input.active[type=reset].alert,.active.button.alert,a.active.button.alert:visited,ul.button-group li a.active.alert,ul.button-group li button.active.alert,ol.button-group li a.active.alert,ol.button-group li button.active.alert,.active[role~=button].alert,button[aria-selected=true].alert,input[aria-selected=true][type=submit].alert,input[aria-selected=true][type=button].alert,input[aria-selected=true][type=reset].alert,[aria-selected=true].button.alert,a[aria-selected=true].button.alert:visited,ul.button-group li a[aria-selected=true].alert,ul.button-group li button[aria-selected=true].alert,ol.button-group li a[aria-selected=true].alert,ol.button-group li button[aria-selected=true].alert,[aria-selected=true][role~=button].alert,button[aria-pressed=true].alert,input[aria-pressed=true][type=submit].alert,input[aria-pressed=true][type=button].alert,input[aria-pressed=true][type=reset].alert,[aria-pressed=true].button.alert,a[aria-pressed=true].button.alert:visited,ul.button-group li a[aria-pressed=true].alert,ul.button-group li button[aria-pressed=true].alert,ol.button-group li a[aria-pressed=true].alert,ol.button-group li button[aria-pressed=true].alert,[aria-pressed=true][role~=button].alert,button.alert:hover,input[type=submit].alert:hover,input[type=button].alert:hover,input[type=reset].alert:hover,.button.alert:hover,a.button.alert:hover:visited,ul.button-group li a.alert:hover,ul.button-group li button.alert:hover,ol.button-group li a.alert:hover,ol.button-group li button.alert:hover,[role~=button].alert:hover,button.alert:focus,input[type=submit].alert:focus,input[type=button].alert:focus,input[type=reset].alert:focus,.button.alert:focus,a.button.alert:focus:visited,ul.button-group li a.alert:focus,ul.button-group li button.alert:focus,ol.button-group li a.alert:focus,ol.button-group li button.alert:focus,[role~=button].alert:focus{background:#e1c118}button.disabled.alert,input.disabled[type=submit].alert,input.disabled[type=button].alert,input.disabled[type=reset].alert,.disabled.button.alert,a.disabled.button.alert:visited,ul.button-group li a.disabled.alert,ul.button-group li button.disabled.alert,ol.button-group li a.disabled.alert,ol.button-group li button.disabled.alert,.disabled[role~=button].alert,button[disabled=disabled].alert,input[disabled=disabled][type=submit].alert,input[disabled=disabled][type=button].alert,input[disabled=disabled][type=reset].alert,[disabled=disabled].button.alert,a[disabled=disabled].button.alert:visited,ul.button-group li a[disabled=disabled].alert,ul.button-group li button[disabled=disabled].alert,ol.button-group li a[disabled=disabled].alert,ol.button-group li button[disabled=disabled].alert,[disabled=disabled][role~=button].alert{color:#d7cdbb;background:#c5b59b;border-color:#bba98b;cursor:not-allowed;text-decoration:none;outline:0}button.warning,input[type=submit].warning,input[type=button].warning,input[type=reset].warning,.button.warning,a.button.warning,a.button.warning:visited,ul.button-group li a.warning,ul.button-group li button.warning,ol.button-group li a.warning,ol.button-group li button.warning,[role~=button].warning{background-color:#fa8600;color:#fff;border-color:#e67900}button.active.warning,input.active[type=submit].warning,input.active[type=button].warning,input.active[type=reset].warning,.active.button.warning,a.active.button.warning:visited,ul.button-group li a.active.warning,ul.button-group li button.active.warning,ol.button-group li a.active.warning,ol.button-group li button.active.warning,.active[role~=button].warning,button[aria-selected=true].warning,input[aria-selected=true][type=submit].warning,input[aria-selected=true][type=button].warning,input[aria-selected=true][type=reset].warning,[aria-selected=true].button.warning,a[aria-selected=true].button.warning:visited,ul.button-group li a[aria-selected=true].warning,ul.button-group li button[aria-selected=true].warning,ol.button-group li a[aria-selected=true].warning,ol.button-group li button[aria-selected=true].warning,[aria-selected=true][role~=button].warning,button[aria-pressed=true].warning,input[aria-pressed=true][type=submit].warning,input[aria-pressed=true][type=button].warning,input[aria-pressed=true][type=reset].warning,[aria-pressed=true].button.warning,a[aria-pressed=true].button.warning:visited,ul.button-group li a[aria-pressed=true].warning,ul.button-group li button[aria-pressed=true].warning,ol.button-group li a[aria-pressed=true].warning,ol.button-group li button[aria-pressed=true].warning,[aria-pressed=true][role~=button].warning,button.warning:hover,input[type=submit].warning:hover,input[type=button].warning:hover,input[type=reset].warning:hover,.button.warning:hover,a.button.warning:hover:visited,ul.button-group li a.warning:hover,ul.button-group li button.warning:hover,ol.button-group li a.warning:hover,ol.button-group li button.warning:hover,[role~=button].warning:hover,button.warning:focus,input[type=submit].warning:focus,input[type=button].warning:focus,input[type=reset].warning:focus,.button.warning:focus,a.button.warning:focus:visited,ul.button-group li a.warning:focus,ul.button-group li button.warning:focus,ol.button-group li a.warning:focus,ol.button-group li button.warning:focus,[role~=button].warning:focus{background:#e07800}button.disabled.warning,input.disabled[type=submit].warning,input.disabled[type=button].warning,input.disabled[type=reset].warning,.disabled.button.warning,a.disabled.button.warning:visited,ul.button-group li a.disabled.warning,ul.button-group li button.disabled.warning,ol.button-group li a.disabled.warning,ol.button-group li button.disabled.warning,.disabled[role~=button].warning,button[disabled=disabled].warning,input[disabled=disabled][type=submit].warning,input[disabled=disabled][type=button].warning,input[disabled=disabled][type=reset].warning,[disabled=disabled].button.warning,a[disabled=disabled].button.warning:visited,ul.button-group li a[disabled=disabled].warning,ul.button-group li button[disabled=disabled].warning,ol.button-group li a[disabled=disabled].warning,ol.button-group li button[disabled=disabled].warning,[disabled=disabled][role~=button].warning{color:#f5d1a7;background:#f0b979;border-color:#eead62;cursor:not-allowed;text-decoration:none;outline:0}button.error,input[type=submit].error,input[type=button].error,input[type=reset].error,.button.error,a.button.error,a.button.error:visited,ul.button-group li a.error,ul.button-group li button.error,ol.button-group li a.error,ol.button-group li button.error,[role~=button].error,[role~=button][aria-invalid]{background-color:#b84c4c;color:#fff;border-color:#b14454}button.active.error,input.active[type=submit].error,input.active[type=button].error,input.active[type=reset].error,.active.button.error,a.active.button.error:visited,ul.button-group li a.active.error,ul.button-group li button.active.error,ol.button-group li a.active.error,ol.button-group li button.active.error,.active[role~=button].error,button[aria-selected=true].error,input[aria-selected=true][type=submit].error,input[aria-selected=true][type=button].error,input[aria-selected=true][type=reset].error,[aria-selected=true].button.error,a[aria-selected=true].button.error:visited,ul.button-group li a[aria-selected=true].error,ul.button-group li button[aria-selected=true].error,ol.button-group li a[aria-selected=true].error,ol.button-group li button[aria-selected=true].error,[aria-selected=true][role~=button].error,button[aria-pressed=true].error,input[aria-pressed=true][type=submit].error,input[aria-pressed=true][type=button].error,input[aria-pressed=true][type=reset].error,[aria-pressed=true].button.error,a[aria-pressed=true].button.error:visited,ul.button-group li a[aria-pressed=true].error,ul.button-group li button[aria-pressed=true].error,ol.button-group li a[aria-pressed=true].error,ol.button-group li button[aria-pressed=true].error,[aria-pressed=true][role~=button].error,button.error:hover,input[type=submit].error:hover,input[type=button].error:hover,input[type=reset].error:hover,.button.error:hover,a.button.error:hover:visited,ul.button-group li a.error:hover,ul.button-group li button.error:hover,ol.button-group li a.error:hover,ol.button-group li button.error:hover,[role~=button].error:hover,button.error:focus,input[type=submit].error:focus,input[type=button].error:focus,input[type=reset].error:focus,.button.error:focus,a.button.error:focus:visited,ul.button-group li a.error:focus,ul.button-group li button.error:focus,ol.button-group li a.error:focus,ol.button-group li button.error:focus,[role~=button].error:focus,.active[role~=button][aria-invalid],[aria-selected=true][role~=button][aria-invalid],[aria-pressed=true][role~=button][aria-invalid],[role~=button][aria-invalid]:hover,[role~=button][aria-invalid]:focus{background:#9a3d3d}button.disabled.error,input.disabled[type=submit].error,input.disabled[type=button].error,input.disabled[type=reset].error,.disabled.button.error,a.disabled.button.error:visited,ul.button-group li a.disabled.error,ul.button-group li button.disabled.error,ol.button-group li a.disabled.error,ol.button-group li button.disabled.error,.disabled[role~=button].error,button[disabled=disabled].error,input[disabled=disabled][type=submit].error,input[disabled=disabled][type=button].error,input[disabled=disabled][type=reset].error,[disabled=disabled].button.error,a[disabled=disabled].button.error:visited,ul.button-group li a[disabled=disabled].error,ul.button-group li button[disabled=disabled].error,ol.button-group li a[disabled=disabled].error,ol.button-group li button[disabled=disabled].error,[disabled=disabled][role~=button].error,.disabled[role~=button][aria-invalid],[disabled=disabled][role~=button][aria-invalid]{color:#d0b1b1;background:#bd9291;border-color:#b38281;cursor:not-allowed;text-decoration:none;outline:0}button.success,input[type=submit].success,input[type=button].success,input[type=reset].success,.button.success,a.button.success,a.button.success:visited,ul.button-group li a.success,ul.button-group li button.success,ol.button-group li a.success,ol.button-group li button.success,[role~=button].success{background-color:#67b646;color:#fff;border-color:#6da43f}button.active.success,input.active[type=submit].success,input.active[type=button].success,input.active[type=reset].success,.active.button.success,a.active.button.success:visited,ul.button-group li a.active.success,ul.button-group li button.active.success,ol.button-group li a.active.success,ol.button-group li button.active.success,.active[role~=button].success,button[aria-selected=true].success,input[aria-selected=true][type=submit].success,input[aria-selected=true][type=button].success,input[aria-selected=true][type=reset].success,[aria-selected=true].button.success,a[aria-selected=true].button.success:visited,ul.button-group li a[aria-selected=true].success,ul.button-group li button[aria-selected=true].success,ol.button-group li a[aria-selected=true].success,ol.button-group li button[aria-selected=true].success,[aria-selected=true][role~=button].success,button[aria-pressed=true].success,input[aria-pressed=true][type=submit].success,input[aria-pressed=true][type=button].success,input[aria-pressed=true][type=reset].success,[aria-pressed=true].button.success,a[aria-pressed=true].button.success:visited,ul.button-group li a[aria-pressed=true].success,ul.button-group li button[aria-pressed=true].success,ol.button-group li a[aria-pressed=true].success,ol.button-group li button[aria-pressed=true].success,[aria-pressed=true][role~=button].success,button.success:hover,input[type=submit].success:hover,input[type=button].success:hover,input[type=reset].success:hover,.button.success:hover,a.button.success:hover:visited,ul.button-group li a.success:hover,ul.button-group li button.success:hover,ol.button-group li a.success:hover,ol.button-group li button.success:hover,[role~=button].success:hover,button.success:focus,input[type=submit].success:focus,input[type=button].success:focus,input[type=reset].success:focus,.button.success:focus,a.button.success:focus:visited,ul.button-group li a.success:focus,ul.button-group li button.success:focus,ol.button-group li a.success:focus,ol.button-group li button.success:focus,[role~=button].success:focus{background:#549539}button.disabled.success,input.disabled[type=submit].success,input.disabled[type=button].success,input.disabled[type=reset].success,.disabled.button.success,a.disabled.button.success:visited,ul.button-group li a.disabled.success,ul.button-group li button.disabled.success,ol.button-group li a.disabled.success,ol.button-group li button.disabled.success,.disabled[role~=button].success,button[disabled=disabled].success,input[disabled=disabled][type=submit].success,input[disabled=disabled][type=button].success,input[disabled=disabled][type=reset].success,[disabled=disabled].button.success,a[disabled=disabled].button.success:visited,ul.button-group li a[disabled=disabled].success,ul.button-group li button[disabled=disabled].success,ol.button-group li a[disabled=disabled].success,ol.button-group li button[disabled=disabled].success,[disabled=disabled][role~=button].success{color:#9cb19c;background:#809b80;border-color:#719072;cursor:not-allowed;text-decoration:none;outline:0}button.question,input[type=submit].question,input[type=button].question,input[type=reset].question,.button.question,a.button.question,a.button.question:visited,ul.button-group li a.question,ul.button-group li button.question,ol.button-group li a.question,ol.button-group li button.question,[role~=button].question{background-color:#9b58b6;color:#fff;border-color:#ab28e1}button.active.question,input.active[type=submit].question,input.active[type=button].question,input.active[type=reset].question,.active.button.question,a.active.button.question:visited,ul.button-group li a.active.question,ul.button-group li button.active.question,ol.button-group li a.active.question,ol.button-group li button.active.question,.active[role~=button].question,button[aria-selected=true].question,input[aria-selected=true][type=submit].question,input[aria-selected=true][type=button].question,input[aria-selected=true][type=reset].question,[aria-selected=true].button.question,a[aria-selected=true].button.question:visited,ul.button-group li a[aria-selected=true].question,ul.button-group li button[aria-selected=true].question,ol.button-group li a[aria-selected=true].question,ol.button-group li button[aria-selected=true].question,[aria-selected=true][role~=button].question,button[aria-pressed=true].question,input[aria-pressed=true][type=submit].question,input[aria-pressed=true][type=button].question,input[aria-pressed=true][type=reset].question,[aria-pressed=true].button.question,a[aria-pressed=true].button.question:visited,ul.button-group li a[aria-pressed=true].question,ul.button-group li button[aria-pressed=true].question,ol.button-group li a[aria-pressed=true].question,ol.button-group li button[aria-pressed=true].question,[aria-pressed=true][role~=button].question,button.question:hover,input[type=submit].question:hover,input[type=button].question:hover,input[type=reset].question:hover,.button.question:hover,a.button.question:hover:visited,ul.button-group li a.question:hover,ul.button-group li button.question:hover,ol.button-group li a.question:hover,ol.button-group li button.question:hover,[role~=button].question:hover,button.question:focus,input[type=submit].question:focus,input[type=button].question:focus,input[type=reset].question:focus,.button.question:focus,a.button.question:focus:visited,ul.button-group li a.question:focus,ul.button-group li button.question:focus,ol.button-group li a.question:focus,ol.button-group li button.question:focus,[role~=button].question:focus{background:#83449c}button.disabled.question,input.disabled[type=submit].question,input.disabled[type=button].question,input.disabled[type=reset].question,.disabled.button.question,a.disabled.button.question:visited,ul.button-group li a.disabled.question,ul.button-group li button.disabled.question,ol.button-group li a.disabled.question,ol.button-group li button.disabled.question,.disabled[role~=button].question,button[disabled=disabled].question,input[disabled=disabled][type=submit].question,input[disabled=disabled][type=button].question,input[disabled=disabled][type=reset].question,[disabled=disabled].button.question,a[disabled=disabled].button.question:visited,ul.button-group li a[disabled=disabled].question,ul.button-group li button[disabled=disabled].question,ol.button-group li a[disabled=disabled].question,ol.button-group li button[disabled=disabled].question,[disabled=disabled][role~=button].question{color:#c9b9cf;background:#b19bbb;border-color:#a58cb0;cursor:not-allowed;text-decoration:none;outline:0}button.block,input[type=submit].block,input[type=button].block,input[type=reset].block,.button.block,a.button.block,a.button.block:visited,ul.button-group li a.block,ul.button-group li button.block,ol.button-group li a.block,ol.button-group li button.block,[role~=button].block{width:100%}.dropdown{position:relative}.dropdown>a:after,.dropdown>button:after,.dropdown .button:after{font-family:FontAwesome;font-weight:400;font-style:normal;text-decoration:inherit;-webkit-font-smoothing:antialiased;*margin-right:.3em;content:"\f0d7"}.dropdown ul{text-align:left;margin:0;padding:0;list-style:none;display:none;background:#fff;border:1px solid #ddd;position:absolute;left:0;width:100%;z-index:10;box-shadow:rgba(0,0,0,.25),5px,0,0;-o-border-radius:0 0 4px 4px;border-radius:0 0 4px 4px}.dropdown ul li a,.dropdown ul li button{width:100%;padding:.5em 1em;border:0;background:0 0;color:#2b2b2d;font-size:1em;font-weight:400;text-shadow:none;text-decoration:none;text-align:left;-o-border-radius:0;border-radius:0}.dropdown ul li a:hover,.dropdown ul li button:hover,.dropdown ul li a:focus,.dropdown ul li button:focus{background:#efefef}.dropdown ul li a.disabled,.dropdown ul li button.disabled,.dropdown ul li a[disabled=disabled],.dropdown ul li button[disabled=disabled]{color:silver;cursor:not-allowed;text-decoration:none;outline:0}.dropdown ul li:last-child{-o-border-radius:0 0 4px 4px;border-radius:0 0 4px 4px}.dropdown hr{margin:0;border-color:#ececec}ul.button-group:after,ol.button-group:after{content:""}ul.button-group li,ol.button-group li{float:left;margin:0}ul.button-group li:first-child a,ol.button-group li:first-child a,ul.button-group li:first-child button,ol.button-group li:first-child button{-o-border-radius:4px 0 0 4px;border-radius:4px 0 0 4px}ul.button-group li:last-child a,ol.button-group li:last-child a,ul.button-group li:last-child button,ol.button-group li:last-child button{-o-border-radius:0 4px 4px 0;border-radius:0 4px 4px 0}ul.button-group li a,ol.button-group li a,ul.button-group li button,ol.button-group li button{word-wrap:break-word;padding:.25em 1em;background:lightgrey;border:1px solid #c6c6c6;color:#5a5a5a;font-weight:700;text-decoration:none;cursor:pointer;line-height:1.5;-o-border-radius:0;border-radius:0;-webkit-transition:background-color .25s ease-out;transition:background-color .25s ease-out}ul.button-group li a.active,ol.button-group li a.active,ul.button-group li a[aria-selected=true],ol.button-group li a[aria-selected=true],ul.button-group li a[aria-pressed=true],ol.button-group li a[aria-pressed=true],ul.button-group li a:hover,ol.button-group li a:hover,ul.button-group li a:focus,ol.button-group li a:focus,ul.button-group li button.active,ol.button-group li button.active,ul.button-group li button[aria-selected=true],ol.button-group li button[aria-selected=true],ul.button-group li button[aria-pressed=true],ol.button-group li button[aria-pressed=true],ul.button-group li button:hover,ol.button-group li button:hover,ul.button-group li button:focus,ol.button-group li button:focus{background:#c6c6c6;text-decoration:none;border:1px solid #c6c6c6}ul.button-group li a.disabled,ol.button-group li a.disabled,ul.button-group li a[disabled=disabled],ol.button-group li a[disabled=disabled],ul.button-group li button.disabled,ol.button-group li button.disabled,ul.button-group li button[disabled=disabled],ol.button-group li button[disabled=disabled]{color:#f3f3f3;background:#d9d9d9;border-color:#ccc;cursor:not-allowed;text-decoration:none;outline:0}fieldset{background-color:#fafafa;border:1px solid #ddd;padding:10px;-o-border-radius:4px;border-radius:4px}legend{font-weight:900;margin-bottom:0}label{display:block;padding:.4em .2em}input,textarea,select{display:block;width:100%;max-width:100%;margin:0;padding:.4em;font-family:monospace;font-size:1em;line-height:1.5;height:2.25em;background:#fff;border:1px solid #ddd;-o-border-radius:4px;border-radius:4px}input[size],input[type=checkbox],input[type=radio]{height:auto}input[type=checkbox],input[type=radio]{border:0;-o-border-radius:0;border-radius:0}textarea{height:10em}.error input,.error select,.error textarea,.invalid input,.invalid select,.invalid textarea,input.invalid,input.error,select.invalid,select.error,textarea.invalid,textarea.error,input[aria-invalid=true],select[aria-invalid=true],textarea[aria-invalid=true]{background:#f2dede;color:#b94a48;border-color:#eed3d7}.error .prefix,.error .suffix,.invalid .prefix,.invalid .suffix{color:#fff;background:#b94a48;border-color:#b94a48}.success input,.success select,.success textarea,.valid input,.valid select,.valid textarea,input.valid,input.success,select.valid,select.success,textarea.valid,textarea.success,input[aria-invalid=false],select[aria-invalid=false],textarea[aria-invalid=false]{background:#dff0d8;color:#468847;border-color:#d6e9c6}.success .prefix,.success .suffix,.valid .prefix,.valid .suffix{color:#fff;background:#468847;border-color:#468847}.prefix,.suffix{display:block;position:relative;z-index:1;width:100%;overflow:hidden;white-space:nowrap;text-overflow:ellipsis;margin:0;padding:.4em;font-family:monospace;line-height:1.25;height:2.25em;text-align:center;color:gray;background:#dfdfdf;border:1px solid #d2d2d2}.prefix{left:2px;-o-border-radius:4px 0 0 4px;border-radius:4px 0 0 4px}.suffix{left:-2px;-o-border-radius:0 4px 4px 0;border-radius:0 4px 4px 0}.message,[role~=alert],[role~=status]{word-wrap:break-word;padding:.25em 1em .25em 2em;background:lightgrey;color:#5a5a5a;border:1px solid #ddd;font-weight:700;text-decoration:none;-o-border-radius:4px;border-radius:4px}.message:before,[role~=alert]:before,[role~=status]:before{font-family:FontAwesome;font-weight:400;font-style:normal;text-decoration:inherit;-webkit-font-smoothing:antialiased;*margin-right:.3em;content:"\f05a";position:absolute;top:50%;margin-top:-.9em;left:0;padding:.2em .3em;font-size:1.2em}.message.small,[role~=alert].small,[role~=status].small{font-size:.8em}.message.medium,[role~=alert].medium,[role~=status].medium{font-size:1em}.message.large,[role~=alert].large,[role~=status].large{font-size:1.5em}.message.square,[role~=alert].square,[role~=status].square{-o-border-radius:0;border-radius:0}.message.round,[role~=alert].round,[role~=status].round{-o-border-radius:9999em;border-radius:9999em}.message.info,[role~=alert].info,[role~=status].info{background-color:#d9edf7;color:#3a87ad;border-color:#bce8f1}.message.alert,[role~=alert].alert,[role~=status].alert{background-color:#fcf8e3;color:#c09853;border-color:#fbeee0}.message.alert:before,[role~=alert].alert:before,[role~=status].alert:before{content:"\f06a"}.message.warning,[role~=alert].warning,[role~=status].warning{background:#ffe5c7;color:#ff971e;border-color:#ffdbb3}.message.warning:before,[role~=alert].warning:before,[role~=status].warning:before{content:"\f071"}.message.error,[role~=alert].error,[role~=status].error{background:#f2dede;color:#b94a48;border-color:#eed3d7}.message.error:before,[role~=alert].error:before,[role~=status].error:before{content:"\f057"}.message.success,[role~=alert].success,[role~=status].success{background:#dff0d8;color:#468847;border-color:#d6e9c6}.message.success:before,[role~=alert].success:before,[role~=status].success:before{content:"\f058"}.message.question,[role~=alert].question,[role~=status].question{background-color:#f0e6f4;color:#9855b4;border-color:#f1dbfa}.message.question:before,[role~=alert].question:before,[role~=status].question:before{content:"\f059"}.message.dismissible,[role~=alert].dismissible,[role~=status].dismissible{padding-right:2em;cursor:pointer}.message.dismissible:after,[role~=alert].dismissible:after,[role~=status].dismissible:after{font-family:FontAwesome;font-weight:400;font-style:normal;text-decoration:inherit;-webkit-font-smoothing:antialiased;*margin-right:.3em;content:"\f00d";position:absolute;top:50%;margin-top:-.9em;right:0;padding:.2em .3em;font-size:1.2em;cursor:pointer}.message.green,[role~=alert].green,[role~=status].green{border-color:#2ecc71;background-color:#54d98c;color:#fff}.message.blue,[role~=alert].blue,[role~=status].blue{border-color:#3498db;background-color:#5faee3;color:#fff}.message.purple,[role~=alert].purple,[role~=status].purple{border-color:#9b59b6;background-color:#b07cc6;color:#fff}.message.yellow,[role~=alert].yellow,[role~=status].yellow{border-color:#f1c40f;background-color:#f4d03f;color:#fff}.message.orange,[role~=alert].orange,[role~=status].orange{border-color:#e67e22;background-color:#eb9950;color:#fff}.message.red,[role~=alert].red,[role~=status].red{border-color:#e74c3c;background-color:#ed7669;color:#fff}.message.pink,[role~=alert].pink,[role~=status].pink{border-color:#f02475;background-color:#f35493;color:#fff}.message.turquoise,[role~=alert].turquoise,[role~=status].turquoise{border-color:#1abc9c;background-color:#28e1bd;color:#fff}.message.asphalt,[role~=alert].asphalt,[role~=status].asphalt{border-color:#34495e;background-color:#46627f;color:#fff}.message.charcoal,[role~=alert].charcoal,[role~=status].charcoal{border-color:#383636;background-color:#524f4f;color:#fff}.message.white,[role~=alert].white,[role~=status].white{border-color:#fff;background-color:#fff;color:#111}.message.black,[role~=alert].black,[role~=status].black{border-color:#111;background-color:#2b2b2b;color:#fff}.nav,[role~=navigation]{clear:both;background:lightgrey;color:#5a5a5a;border:1px solid #c6c6c6;-o-border-radius:4px;border-radius:4px}.nav:after,[role~=navigation]:after{content:" ";display:block;height:0;font-size:0;clear:both;visibility:hidden}.nav a,.nav a:visited,.nav button{background:lightgrey;color:#5a5a5a;cursor:pointer;padding:.25em 1em;text-decoration:none;cursor:pointer;line-height:1.5;border:0;border-top:1px solid #c6c6c6;-o-border-radius:0;border-radius:0}.nav.vertical a,.nav.vertical button:not(.hamburger){display:block;width:100%;text-align:left}.nav button.hamburger{display:none;background:0 0;color:#616161;border:0}.nav button.hamburger:hover,.nav button.hamburger:focus{background:none!important}.nav a.on,.nav a.on:visited,.nav button.on,.nav a:hover,.nav button:hover,.nav a:focus,.nav button:focus,.nav li[role=menu][aria-expanded=true]>a,.nav li[role=menu][aria-expanded=true]>button,.nav a.active,.nav a.active:visited,.nav button.active{background:#c6c6c6;color:#616161;text-decoration:none}.nav li ul{display:none;position:absolute;z-index:9;min-width:100%;border:1px solid #c6c6c6;font-size:.9em;-o-border-radius:0 0 4px 4px;border-radius:0 0 4px 4px}.nav.vertical li ul{position:static;border:0;-o-border-radius:0!important;border-radius:0!important}.nav.green,.nav.green li[role=menu] ul{background-color:#2ecc71;color:#fff;border-color:#29b765}.nav.green li[role=menu] ul a,.nav.green li[role=menu] ul button,.nav.green a,.nav.green button{background-color:#2ecc71;color:#fff;border-color:#29b765}.nav.green li[role=menu] ul a.active,.nav.green li[role=menu] ul button.active,.nav.green a.active,.nav.green button.active,.nav.green li[role=menu] ul a:hover,.nav.green li[role=menu] ul button:hover,.nav.green a:hover,.nav.green button:hover,.nav.green li[role=menu] ul a:focus,.nav.green li[role=menu] ul button:focus,.nav.green a:focus,.nav.green button:focus{background:#29b765;border-color:#25a25a}.nav.green li[role=menu] ul a.disabled,.nav.green li[role=menu] ul button.disabled,.nav.green a.disabled,.nav.green button.disabled{color:#a0d9b8;background:#7ccb9d;border-color:#69c490;cursor:not-allowed;text-decoration:none;outline:0}.nav.blue,.nav.blue li[role=menu] ul{background-color:#3498db;color:#fff;border-color:#258cd1}.nav.blue li[role=menu] ul a,.nav.blue li[role=menu] ul button,.nav.blue a,.nav.blue button{background-color:#3498db;color:#fff;border-color:#258cd1}.nav.blue li[role=menu] ul a.active,.nav.blue li[role=menu] ul button.active,.nav.blue a.active,.nav.blue button.active,.nav.blue li[role=menu] ul a:hover,.nav.blue li[role=menu] ul button:hover,.nav.blue a:hover,.nav.blue button:hover,.nav.blue li[role=menu] ul a:focus,.nav.blue li[role=menu] ul button:focus,.nav.blue a:focus,.nav.blue button:focus{background:#258cd1;border-color:#217dbb}.nav.blue li[role=menu] ul a.disabled,.nav.blue li[role=menu] ul button.disabled,.nav.blue a.disabled,.nav.blue button.disabled{color:#abcde3;background:#85b6d6;border-color:#72aad0;cursor:not-allowed;text-decoration:none;outline:0}.nav.purple,.nav.purple li[role=menu] ul{background-color:#9b59b6;color:#fff;border-color:#8f4bab}.nav.purple li[role=menu] ul a,.nav.purple li[role=menu] ul button,.nav.purple a,.nav.purple button{background-color:#9b59b6;color:#fff;border-color:#8f4bab}.nav.purple li[role=menu] ul a.active,.nav.purple li[role=menu] ul button.active,.nav.purple a.active,.nav.purple button.active,.nav.purple li[role=menu] ul a:hover,.nav.purple li[role=menu] ul button:hover,.nav.purple a:hover,.nav.purple button:hover,.nav.purple li[role=menu] ul a:focus,.nav.purple li[role=menu] ul button:focus,.nav.purple a:focus,.nav.purple button:focus{background:#8f4bab;border-color:#804399}.nav.purple li[role=menu] ul a.disabled,.nav.purple li[role=menu] ul button.disabled,.nav.purple a.disabled,.nav.purple button.disabled{color:#ccbdd2;background:#b49ebd;border-color:#a88fb3;cursor:not-allowed;text-decoration:none;outline:0}.nav.yellow,.nav.yellow li[role=menu] ul{background-color:#f1c40f;color:#fff;border-color:#dab10d}.nav.yellow li[role=menu] ul a,.nav.yellow li[role=menu] ul button,.nav.yellow a,.nav.yellow button{background-color:#f1c40f;color:#fff;border-color:#dab10d}.nav.yellow li[role=menu] ul a.active,.nav.yellow li[role=menu] ul button.active,.nav.yellow a.active,.nav.yellow button.active,.nav.yellow li[role=menu] ul a:hover,.nav.yellow li[role=menu] ul button:hover,.nav.yellow a:hover,.nav.yellow button:hover,.nav.yellow li[role=menu] ul a:focus,.nav.yellow li[role=menu] ul button:focus,.nav.yellow a:focus,.nav.yellow button:focus{background:#dab10d;border-color:#c29d0b}.nav.yellow li[role=menu] ul a.disabled,.nav.yellow li[role=menu] ul button.disabled,.nav.yellow a.disabled,.nav.yellow button.disabled{color:#ebda94;background:#e3cb69;border-color:#e0c453;cursor:not-allowed;text-decoration:none;outline:0}.nav.orange,.nav.orange li[role=menu] ul{background-color:#e67e22;color:#fff;border-color:#d67118}.nav.orange li[role=menu] ul a,.nav.orange li[role=menu] ul button,.nav.orange a,.nav.orange button{background-color:#e67e22;color:#fff;border-color:#d67118}.nav.orange li[role=menu] ul a.active,.nav.orange li[role=menu] ul button.active,.nav.orange a.active,.nav.orange button.active,.nav.orange li[role=menu] ul a:hover,.nav.orange li[role=menu] ul button:hover,.nav.orange a:hover,.nav.orange button:hover,.nav.orange li[role=menu] ul a:focus,.nav.orange li[role=menu] ul button:focus,.nav.orange a:focus,.nav.orange button:focus{background:#d67118;border-color:#bf6516}.nav.orange li[role=menu] ul a.disabled,.nav.orange li[role=menu] ul button.disabled,.nav.orange a.disabled,.nav.orange button.disabled{color:#e7c2a0;background:#dda778;border-color:#d89a63;cursor:not-allowed;text-decoration:none;outline:0}.nav.red,.nav.red li[role=menu] ul{background-color:#e74c3c;color:#fff;border-color:#e43725}.nav.red li[role=menu] ul a,.nav.red li[role=menu] ul button,.nav.red a,.nav.red button{background-color:#e74c3c;color:#fff;border-color:#e43725}.nav.red li[role=menu] ul a.active,.nav.red li[role=menu] ul button.active,.nav.red a.active,.nav.red button.active,.nav.red li[role=menu] ul a:hover,.nav.red li[role=menu] ul button:hover,.nav.red a:hover,.nav.red button:hover,.nav.red li[role=menu] ul a:focus,.nav.red li[role=menu] ul button:focus,.nav.red a:focus,.nav.red button:focus{background:#e43725;border-color:#d62c1a}.nav.red li[role=menu] ul a.disabled,.nav.red li[role=menu] ul button.disabled,.nav.red a.disabled,.nav.red button.disabled{color:#ecbcb7;background:#e1968e;border-color:#dc837a;cursor:not-allowed;text-decoration:none;outline:0}.nav.pink,.nav.pink li[role=menu] ul{background-color:#f02475;color:#fff;border-color:#ea1067}.nav.pink li[role=menu] ul a,.nav.pink li[role=menu] ul button,.nav.pink a,.nav.pink button{background-color:#f02475;color:#fff;border-color:#ea1067}.nav.pink li[role=menu] ul a.active,.nav.pink li[role=menu] ul button.active,.nav.pink a.active,.nav.pink button.active,.nav.pink li[role=menu] ul a:hover,.nav.pink li[role=menu] ul button:hover,.nav.pink a:hover,.nav.pink button:hover,.nav.pink li[role=menu] ul a:focus,.nav.pink li[role=menu] ul button:focus,.nav.pink a:focus,.nav.pink button:focus{background:#ea1067;border-color:#d30e5c}.nav.pink li[role=menu] ul a.disabled,.nav.pink li[role=menu] ul button.disabled,.nav.pink a.disabled,.nav.pink button.disabled{color:#eea6c2;background:#e57ba5;border-color:#e16697;cursor:not-allowed;text-decoration:none;outline:0}.nav.turquoise,.nav.turquoise li[role=menu] ul{background-color:#1abc9c;color:#fff;border-color:#17a689}.nav.turquoise li[role=menu] ul a,.nav.turquoise li[role=menu] ul button,.nav.turquoise a,.nav.turquoise button{background-color:#1abc9c;color:#fff;border-color:#17a689}.nav.turquoise li[role=menu] ul a.active,.nav.turquoise li[role=menu] ul button.active,.nav.turquoise a.active,.nav.turquoise button.active,.nav.turquoise li[role=menu] ul a:hover,.nav.turquoise li[role=menu] ul button:hover,.nav.turquoise a:hover,.nav.turquoise button:hover,.nav.turquoise li[role=menu] ul a:focus,.nav.turquoise li[role=menu] ul button:focus,.nav.turquoise a:focus,.nav.turquoise button:focus{background:#17a689;border-color:#148f77}.nav.turquoise li[role=menu] ul a.disabled,.nav.turquoise li[role=menu] ul button.disabled,.nav.turquoise a.disabled,.nav.turquoise button.disabled{color:#7cdac7;background:#54ceb6;border-color:#40c9ae;cursor:not-allowed;text-decoration:none;outline:0}.nav.asphalt,.nav.asphalt li[role=menu] ul{background-color:#34495e;color:#fff;border-color:#2b3c4e}.nav.asphalt li[role=menu] ul a,.nav.asphalt li[role=menu] ul button,.nav.asphalt a,.nav.asphalt button{background-color:#34495e;color:#fff;border-color:#2b3c4e}.nav.asphalt li[role=menu] ul a.active,.nav.asphalt li[role=menu] ul button.active,.nav.asphalt a.active,.nav.asphalt button.active,.nav.asphalt li[role=menu] ul a:hover,.nav.asphalt li[role=menu] ul button:hover,.nav.asphalt a:hover,.nav.asphalt button:hover,.nav.asphalt li[role=menu] ul a:focus,.nav.asphalt li[role=menu] ul button:focus,.nav.asphalt a:focus,.nav.asphalt button:focus{background:#2b3c4e;border-color:#222f3d}.nav.asphalt li[role=menu] ul a.disabled,.nav.asphalt li[role=menu] ul button.disabled,.nav.asphalt a.disabled,.nav.asphalt button.disabled{color:#7e8993;background:#656f79;border-color:#5a626b;cursor:not-allowed;text-decoration:none;outline:0}.nav.charcoal,.nav.charcoal li[role=menu] ul{background-color:#383636;color:#fff;border-color:#2b2929}.nav.charcoal li[role=menu] ul a,.nav.charcoal li[role=menu] ul button,.nav.charcoal a,.nav.charcoal button{background-color:#383636;color:#fff;border-color:#2b2929}.nav.charcoal li[role=menu] ul a.active,.nav.charcoal li[role=menu] ul button.active,.nav.charcoal a.active,.nav.charcoal button.active,.nav.charcoal li[role=menu] ul a:hover,.nav.charcoal li[role=menu] ul button:hover,.nav.charcoal a:hover,.nav.charcoal button:hover,.nav.charcoal li[role=menu] ul a:focus,.nav.charcoal li[role=menu] ul button:focus,.nav.charcoal a:focus,.nav.charcoal button:focus{background:#2b2929;border-color:#1e1d1d}.nav.charcoal li[role=menu] ul a.disabled,.nav.charcoal li[role=menu] ul button.disabled,.nav.charcoal a.disabled,.nav.charcoal button.disabled{color:#777;background:#5d5d5d;border-color:#515151;cursor:not-allowed;text-decoration:none;outline:0}.nav.white,.nav.white li[role=menu] ul{background-color:#fff;color:#111;border-color:#f2f2f2}.nav.white li[role=menu] ul a,.nav.white li[role=menu] ul button,.nav.white a,.nav.white button{background-color:#fff;color:#111;border-color:#f2f2f2}.nav.white li[role=menu] ul a.active,.nav.white li[role=menu] ul button.active,.nav.white a.active,.nav.white button.active,.nav.white li[role=menu] ul a:hover,.nav.white li[role=menu] ul button:hover,.nav.white a:hover,.nav.white button:hover,.nav.white li[role=menu] ul a:focus,.nav.white li[role=menu] ul button:focus,.nav.white a:focus,.nav.white button:focus{background:#f2f2f2;border-color:#e6e6e6}.nav.white li[role=menu] ul a.disabled,.nav.white li[role=menu] ul button.disabled,.nav.white a.disabled,.nav.white button.disabled{color:#fff;background:#fff;border-color:#fff;cursor:not-allowed;text-decoration:none;outline:0}.nav.black,.nav.black li[role=menu] ul{background-color:#111;color:#fff;border-color:#040404}.nav.black li[role=menu] ul a,.nav.black li[role=menu] ul button,.nav.black a,.nav.black button{background-color:#111;color:#fff;border-color:#040404}.nav.black li[role=menu] ul a.active,.nav.black li[role=menu] ul button.active,.nav.black a.active,.nav.black button.active,.nav.black li[role=menu] ul a:hover,.nav.black li[role=menu] ul button:hover,.nav.black a:hover,.nav.black button:hover,.nav.black li[role=menu] ul a:focus,.nav.black li[role=menu] ul button:focus,.nav.black a:focus,.nav.black button:focus{background:#040404;border-color:#000}.nav.black li[role=menu] ul a.disabled,.nav.black li[role=menu] ul button.disabled,.nav.black a.disabled,.nav.black button.disabled{color:#515151;background:#373737;border-color:#2b2b2b;cursor:not-allowed;text-decoration:none;outline:0}.no-js .nav ul li:hover>ul,.no-js .nav ul li:focus>ul{display:block}.nav.green button.hamburger{color:#fff}.nav.green li[role=menu][aria-expanded=true]>a,.nav.green li[role=menu][aria-expanded=true]>button{background:#29b765;border-color:#25a25a;color:#fff}.nav.blue button.hamburger{color:#fff}.nav.blue li[role=menu][aria-expanded=true]>a,.nav.blue li[role=menu][aria-expanded=true]>button{background:#258cd1;border-color:#217dbb;color:#fff}.nav.purple button.hamburger{color:#fff}.nav.purple li[role=menu][aria-expanded=true]>a,.nav.purple li[role=menu][aria-expanded=true]>button{background:#8f4bab;border-color:#804399;color:#fff}.nav.yellow button.hamburger{color:#fff}.nav.yellow li[role=menu][aria-expanded=true]>a,.nav.yellow li[role=menu][aria-expanded=true]>button{background:#dab10d;border-color:#c29d0b;color:#fff}.nav.orange button.hamburger{color:#fff}.nav.orange li[role=menu][aria-expanded=true]>a,.nav.orange li[role=menu][aria-expanded=true]>button{background:#d67118;border-color:#bf6516;color:#fff}.nav.red button.hamburger{color:#fff}.nav.red li[role=menu][aria-expanded=true]>a,.nav.red li[role=menu][aria-expanded=true]>button{background:#e43725;border-color:#d62c1a;color:#fff}.nav.pink button.hamburger{color:#fff}.nav.pink li[role=menu][aria-expanded=true]>a,.nav.pink li[role=menu][aria-expanded=true]>button{background:#ea1067;border-color:#d30e5c;color:#fff}.nav.turquoise button.hamburger{color:#fff}.nav.turquoise li[role=menu][aria-expanded=true]>a,.nav.turquoise li[role=menu][aria-expanded=true]>button{background:#17a689;border-color:#148f77;color:#fff}.nav.asphalt button.hamburger{color:#fff}.nav.asphalt li[role=menu][aria-expanded=true]>a,.nav.asphalt li[role=menu][aria-expanded=true]>button{background:#2b3c4e;border-color:#222f3d;color:#fff}.nav.charcoal button.hamburger{color:#fff}.nav.charcoal li[role=menu][aria-expanded=true]>a,.nav.charcoal li[role=menu][aria-expanded=true]>button{background:#2b2929;border-color:#1e1d1d;color:#fff}.nav.white button.hamburger{color:#111}.nav.white li[role=menu][aria-expanded=true]>a,.nav.white li[role=menu][aria-expanded=true]>button{background:#f2f2f2;border-color:#e6e6e6;color:#111}.nav.black button.hamburger{color:#fff}.nav.black li[role=menu][aria-expanded=true]>a,.nav.black li[role=menu][aria-expanded=true]>button{background:#040404;border-color:#000;color:#fff}.nav.vertical>ul>li:first-child>a,.nav.vertical>ul>li:first-child>button{border-top-width:0}.nav.vertical>ul>li>a,.nav.vertical>ul>li>button{border-top-width:1px}.nav.vertical li li a,.nav.vertical li li button{text-indent:10px}.nav.vertical li li li a,.nav.vertical li li li button{text-indent:20px}.nav.vertical li[role=menu]>a:after,.nav.vertical li[role=menu]>button:after{content:"\f0d7"}.nav.vertical li[role=menu][aria-pressed=true]>a:after,.nav.vertical li[role=menu][aria-pressed=true]>button:after{content:"\f0d8"}.nav.vertical li li{-o-border-radius:0;border-radius:0}.nav.vertical li li:last-child a,.nav.vertical li li:last-child button{-o-border-radius:0;border-radius:0}.nav.vertical li li ul li:first-child a,.nav.vertical li li ul li:first-child button{-o-border-radius:0;border-radius:0}.nav li[role=menu]>a:after,.nav li[role=menu]>button:after{margin-left:.5em;content:"\f0d7";font-family:FontAwesome;font-weight:400;font-style:normal;text-decoration:inherit;-webkit-font-smoothing:antialiased}.nav li ul>li:first-child>a,.nav li ul>li:first-child>button{border-top-width:0}.nav li ul a,.nav li ul button{display:block;width:100%}.nav li li{float:none}.nav li li:last-child a,.nav li li:last-child button{-o-border-radius:0 0 4px 4px;border-radius:0 0 4px 4px}.nav li li[role=menu]>a:after,.nav li li[role=menu]>button:after{content:"\f0da"}.nav li li ul{top:0;left:100%;min-width:none;font-size:1em;-o-border-radius:0 4px 4px;border-radius:0 4px 4px}.nav li li ul li:first-child a,.nav li li ul li:first-child button{-o-border-radius:0 4px 0 0;border-radius:0 4px 0 0}.nav li li ul li:last-child a,.nav li li ul li:last-child button{-o-border-radius:0 0 4px 4px;border-radius:0 0 4px 4px}.nav ul[aria-expanded=true]{display:block}.nav>ul>li>a,.nav>ul>li>button{border-top-width:0}.nav>ul>li:first-child>a,.nav>ul>li:first-child>button{border-top-width:0}table{border:1px solid #ddd;width:100%;max-width:100%;padding:0;margin:0;font-size:1em}table.align-center th th,table.align-center td th,table.align-center th td,table.align-center td td{text-align:center}table.responsive{table-layout:fixed}table th,table td{padding:10px;background:#fcfcfc;white-space:normal;word-wrap:break-word;text-align:left}table td{font-size:.85em}table tr:nth-child(even) th,table tr:nth-child(even) td{background:#f2f2f2}table thead tr th{font-size:1em;font-weight:700}table thead tr th,table thead tr td{background:lightgrey;color:#5a5a5a}table.info thead tr th,table.info thead tr td{background:#d9edf7;color:#3a87ad}table.success thead tr th,table.success thead tr td{background:#dff0d8;color:#468847}table.alert thead tr th,table.alert thead tr td{background:#fcf8e3;color:#c09853}table.question thead tr th,table.question thead tr td{background:#f0e6f4;color:#9855b4}table.warning thead tr th,table.warning thead tr td{background:#ffe5c7;color:#ff971e}table.error thead tr th,table.error thead tr td{background:#f2dede;color:#b94a48}table.green thead tr th,table.green thead tr td{background:#2ecc71;color:#fff}table.blue thead tr th,table.blue thead tr td{background:#3498db;color:#fff}table.purple thead tr th,table.purple thead tr td{background:#9b59b6;color:#fff}table.yellow thead tr th,table.yellow thead tr td{background:#f1c40f;color:#fff}table.orange thead tr th,table.orange thead tr td{background:#e67e22;color:#fff}table.red thead tr th,table.red thead tr td{background:#e74c3c;color:#fff}table.pink thead tr th,table.pink thead tr td{background:#f02475;color:#fff}table.turquoise thead tr th,table.turquoise thead tr td{background:#1abc9c;color:#fff}table.asphalt thead tr th,table.asphalt thead tr td{background:#34495e;color:#fff}table.charcoal thead tr th,table.charcoal thead tr td{background:#383636;color:#fff}table.white thead tr th,table.white thead tr td{background:#fff;color:#111}table.black thead tr th,table.black thead tr td{background:#111;color:#fff}table thead tr td{font-weight:400}table tbody tr th{font-size:.95em}table tfoot tr th{font-size:.85em}table tfoot tr td{background:#e0e0e0;color:#5a5a5a;font-size:.75em}.tabs>ul li,[role~=tab]{margin-bottom:-1px;padding:.25em 1em;background:lightgrey;border:1px solid #b9b9b9;color:#5a5a5a;font-family:Helvetica,Arial,Geneva,sans-serif;font-weight:700;text-decoration:none;cursor:pointer;line-height:1.5;width:auto;height:auto;text-align:center;-o-border-radius:4px 4px 0 0;border-radius:4px 4px 0 0}.tabs>ul li:hover,[role~=tab]:hover,.tabs>ul li:focus,[role~=tab]:focus{-webkit-transition:background-color .25s ease-out;transition:background-color .25s ease-out}.tabs>ul li:hover,[role~=tab]:hover,.tabs>ul li:focus,[role~=tab]:focus,.tabs.accordion>ul li.active{background:#c6c6c6;text-decoration:none;color:#616161}.tabs>ul li.active{background:#fff;text-decoration:none;color:#2b2b2d;border-bottom-color:#fff;-webkit-transition:none;transition:none}.tabs>div,[role~=tabpanel]{display:none;background:#fff;border:1px solid #b9b9b9;color:#2b2b2d;-o-border-radius:0 4px 4px;border-radius:0 4px 4px}.tabs>div.active,.active[role~=tabpanel],.tabs>div.on,.on[role~=tabpanel]{display:block}.tabs>ul li.green{background-color:#2ecc71;color:#fff;border-color:#29b765}.tabs>ul li.active.green,.tabs>ul li[aria-selected=true].green,.tabs>ul li[aria-pressed=true].green,.tabs>ul li.green:hover,.tabs>ul li.green:focus{background:#29b765;border-color:#25a25a}.tabs>ul li.disabled.green,.tabs>ul li[disabled=disabled].green{color:#a0d9b8;background:#7ccb9d;border-color:#69c490;cursor:not-allowed;text-decoration:none;outline:0}.tabs>ul li.blue{background-color:#3498db;color:#fff;border-color:#258cd1}.tabs>ul li.active.blue,.tabs>ul li[aria-selected=true].blue,.tabs>ul li[aria-pressed=true].blue,.tabs>ul li.blue:hover,.tabs>ul li.blue:focus{background:#258cd1;border-color:#217dbb}.tabs>ul li.disabled.blue,.tabs>ul li[disabled=disabled].blue{color:#abcde3;background:#85b6d6;border-color:#72aad0;cursor:not-allowed;text-decoration:none;outline:0}.tabs>ul li.purple{background-color:#9b59b6;color:#fff;border-color:#8f4bab}.tabs>ul li.active.purple,.tabs>ul li[aria-selected=true].purple,.tabs>ul li[aria-pressed=true].purple,.tabs>ul li.purple:hover,.tabs>ul li.purple:focus{background:#8f4bab;border-color:#804399}.tabs>ul li.disabled.purple,.tabs>ul li[disabled=disabled].purple{color:#ccbdd2;background:#b49ebd;border-color:#a88fb3;cursor:not-allowed;text-decoration:none;outline:0}.tabs>ul li.yellow{background-color:#f1c40f;color:#fff;border-color:#dab10d}.tabs>ul li.active.yellow,.tabs>ul li[aria-selected=true].yellow,.tabs>ul li[aria-pressed=true].yellow,.tabs>ul li.yellow:hover,.tabs>ul li.yellow:focus{background:#dab10d;border-color:#c29d0b}.tabs>ul li.disabled.yellow,.tabs>ul li[disabled=disabled].yellow{color:#ebda94;background:#e3cb69;border-color:#e0c453;cursor:not-allowed;text-decoration:none;outline:0}.tabs>ul li.orange{background-color:#e67e22;color:#fff;border-color:#d67118}.tabs>ul li.active.orange,.tabs>ul li[aria-selected=true].orange,.tabs>ul li[aria-pressed=true].orange,.tabs>ul li.orange:hover,.tabs>ul li.orange:focus{background:#d67118;border-color:#bf6516}.tabs>ul li.disabled.orange,.tabs>ul li[disabled=disabled].orange{color:#e7c2a0;background:#dda778;border-color:#d89a63;cursor:not-allowed;text-decoration:none;outline:0}.tabs>ul li.red{background-color:#e74c3c;color:#fff;border-color:#e43725}.tabs>ul li.active.red,.tabs>ul li[aria-selected=true].red,.tabs>ul li[aria-pressed=true].red,.tabs>ul li.red:hover,.tabs>ul li.red:focus{background:#e43725;border-color:#d62c1a}.tabs>ul li.disabled.red,.tabs>ul li[disabled=disabled].red{color:#ecbcb7;background:#e1968e;border-color:#dc837a;cursor:not-allowed;text-decoration:none;outline:0}.tabs>ul li.pink{background-color:#f02475;color:#fff;border-color:#ea1067}.tabs>ul li.active.pink,.tabs>ul li[aria-selected=true].pink,.tabs>ul li[aria-pressed=true].pink,.tabs>ul li.pink:hover,.tabs>ul li.pink:focus{background:#ea1067;border-color:#d30e5c}.tabs>ul li.disabled.pink,.tabs>ul li[disabled=disabled].pink{color:#eea6c2;background:#e57ba5;border-color:#e16697;cursor:not-allowed;text-decoration:none;outline:0}.tabs>ul li.turquoise{background-color:#1abc9c;color:#fff;border-color:#17a689}.tabs>ul li.active.turquoise,.tabs>ul li[aria-selected=true].turquoise,.tabs>ul li[aria-pressed=true].turquoise,.tabs>ul li.turquoise:hover,.tabs>ul li.turquoise:focus{background:#17a689;border-color:#148f77}.tabs>ul li.disabled.turquoise,.tabs>ul li[disabled=disabled].turquoise{color:#7cdac7;background:#54ceb6;border-color:#40c9ae;cursor:not-allowed;text-decoration:none;outline:0}.tabs>ul li.asphalt{background-color:#34495e;color:#fff;border-color:#2b3c4e}.tabs>ul li.active.asphalt,.tabs>ul li[aria-selected=true].asphalt,.tabs>ul li[aria-pressed=true].asphalt,.tabs>ul li.asphalt:hover,.tabs>ul li.asphalt:focus{background:#2b3c4e;border-color:#222f3d}.tabs>ul li.disabled.asphalt,.tabs>ul li[disabled=disabled].asphalt{color:#7e8993;background:#656f79;border-color:#5a626b;cursor:not-allowed;text-decoration:none;outline:0}.tabs>ul li.charcoal{background-color:#383636;color:#fff;border-color:#2b2929}.tabs>ul li.active.charcoal,.tabs>ul li[aria-selected=true].charcoal,.tabs>ul li[aria-pressed=true].charcoal,.tabs>ul li.charcoal:hover,.tabs>ul li.charcoal:focus{background:#2b2929;border-color:#1e1d1d}.tabs>ul li.disabled.charcoal,.tabs>ul li[disabled=disabled].charcoal{color:#777;background:#5d5d5d;border-color:#515151;cursor:not-allowed;text-decoration:none;outline:0}.tabs>ul li.white{background-color:#fff;color:#111;border-color:#f2f2f2}.tabs>ul li.active.white,.tabs>ul li[aria-selected=true].white,.tabs>ul li[aria-pressed=true].white,.tabs>ul li.white:hover,.tabs>ul li.white:focus{background:#f2f2f2;border-color:#e6e6e6}.tabs>ul li.disabled.white,.tabs>ul li[disabled=disabled].white{color:#fff;background:#fff;border-color:#fff;cursor:not-allowed;text-decoration:none;outline:0}.tabs>ul li.black{background-color:#111;color:#fff;border-color:#040404}.tabs>ul li.active.black,.tabs>ul li[aria-selected=true].black,.tabs>ul li[aria-pressed=true].black,.tabs>ul li.black:hover,.tabs>ul li.black:focus{background:#040404;border-color:#000}.tabs>ul li.disabled.black,.tabs>ul li[disabled=disabled].black{color:#515151;background:#373737;border-color:#2b2b2b;cursor:not-allowed;text-decoration:none;outline:0}.no-js .tabs>div{display:block}.tabs:not(.accordion).vertical>ul{overflow:auto}.tabs.vertical{-o-border-radius:4px;border-radius:4px}.tabs.vertical>ul{background:#b9b9b9;-o-border-radius:4px 0 0 4px;border-radius:4px 0 0 4px}.tabs.vertical>ul li{border-right:0}.tabs.vertical>ul li:not(:first-child){-o-border-radius:0;border-radius:0}.tabs.vertical>ul li:first-child{-o-border-radius:4px 0 0;border-radius:4px 0 0}.tabs.vertical>div{clear:none;-o-border-radius:4px;border-radius:4px}.tabs.accordion>ul li{-o-border-radius:0;border-radius:0}.tabs.accordion>ul li:first-child{-o-border-radius:4px 4px 0 0;border-radius:4px 4px 0 0}.tabs.accordion>ul li:last-of-type{-o-border-radius:0 0 4px 4px;border-radius:0 0 4px 4px}.tabs.accordion>ul li.active{border-bottom:1px solid #b9b9b9}.tabs.accordion>ul>div{-o-border-radius:0;border-radius:0}.tabs.accordion>ul>div:last-child{-o-border-radius:0 0 4px 4px;border-radius:0 0 4px 4px}body{-webkit-backface-visibility:hidden}.animated{-webkit-animation-duration:1s;animation-duration:1s;-webkit-animation-fill-mode:both;animation-fill-mode:both}@-webkit-keyframes bounce{0%,20%,50%,80%,100%{-webkit-transform:translateY(0);transform:translateY(0)}40%{-webkit-transform:translateY(-30px);transform:translateY(-30px)}60%{-webkit-transform:translateY(-15px);transform:translateY(-15px)}}@keyframes bounce{0%,20%,50%,80%,100%{-webkit-transform:translateY(0);transform:translateY(0)}40%{-webkit-transform:translateY(-30px);transform:translateY(-30px)}60%{-webkit-transform:translateY(-15px);transform:translateY(-15px)}}.bounce{-webkit-animation-name:bounce;animation-name:bounce}@-webkit-keyframes bounceIn{0%{opacity:0;-webkit-transform:scale(0.3);transform:scale(0.3)}50%{opacity:1;-webkit-transform:scale(1.05);transform:scale(1.05)}70%{-webkit-transform:scale(0.9);transform:scale(0.9)}100%{-webkit-transform:scale(1);transform:scale(1)}}@keyframes bounceIn{0%{opacity:0;-webkit-transform:scale(0.3);transform:scale(0.3)}50%{opacity:1;-webkit-transform:scale(1.05);transform:scale(1.05)}70%{-webkit-transform:scale(0.9);transform:scale(0.9)}100%{-webkit-transform:scale(1);transform:scale(1)}}.bounceIn{-webkit-animation-name:bounceIn;animation-name:bounceIn}@-webkit-keyframes bounceInUp{0%{opacity:0;-webkit-transform:translateY(2000px);transform:translateY(2000px)}60%{opacity:1;-webkit-transform:translateY(-30px);transform:translateY(-30px)}80%{-webkit-transform:translateY(10px);transform:translateY(10px)}100%{-webkit-transform:translateY(0);transform:translateY(0)}}@keyframes bounceInUp{0%{opacity:0;-webkit-transform:translateY(2000px);transform:translateY(2000px)}60%{opacity:1;-webkit-transform:translateY(-30px);transform:translateY(-30px)}80%{-webkit-transform:translateY(10px);transform:translateY(10px)}100%{-webkit-transform:translateY(0);transform:translateY(0)}}.bounceInUp{-webkit-animation-name:bounceInUp;animation-name:bounceInUp}@-webkit-keyframes bounceInDown{0%{opacity:0;-webkit-transform:translateY(-2000px);transform:translateY(-2000px)}60%{opacity:1;-webkit-transform:translateY(30px);transform:translateY(30px)}80%{-webkit-transform:translateY(-10px);transform:translateY(-10px)}100%{-webkit-transform:translateY(0);transform:translateY(0)}}@keyframes bounceInDown{0%{opacity:0;-webkit-transform:translateY(-2000px);transform:translateY(-2000px)}60%{opacity:1;-webkit-transform:translateY(30px);transform:translateY(30px)}80%{-webkit-transform:translateY(-10px);transform:translateY(-10px)}100%{-webkit-transform:translateY(0);transform:translateY(0)}}.bounceInDown{-webkit-animation-name:bounceInDown;animation-name:bounceInDown}@-webkit-keyframes bounceInLeft{0%{opacity:0;-webkit-transform:translateX(-2000px);transform:translateX(-2000px)}60%{opacity:1;-webkit-transform:translateX(30px);transform:translateX(30px)}80%{-webkit-transform:translateX(-10px);transform:translateX(-10px)}100%{-webkit-transform:translateX(0);transform:translateX(0)}}@keyframes bounceInLeft{0%{opacity:0;-webkit-transform:translateX(-2000px);transform:translateX(-2000px)}60%{opacity:1;-webkit-transform:translateX(30px);transform:translateX(30px)}80%{-webkit-transform:translateX(-10px);transform:translateX(-10px)}100%{-webkit-transform:translateX(0);transform:translateX(0)}}.bounceInLeft{-webkit-animation-name:bounceInLeft;animation-name:bounceInLeft}@-webkit-keyframes bounceInRight{0%{opacity:0;-webkit-transform:translateX(2000px);transform:translateX(2000px)}60%{opacity:1;-webkit-transform:translateX(-30px);transform:translateX(-30px)}80%{-webkit-transform:translateX(10px);transform:translateX(10px)}100%{-webkit-transform:translateX(0);transform:translateX(0)}}@keyframes bounceInRight{0%{opacity:0;-webkit-transform:translateX(2000px);transform:translateX(2000px)}60%{opacity:1;-webkit-transform:translateX(-30px);transform:translateX(-30px)}80%{-webkit-transform:translateX(10px);transform:translateX(10px)}100%{-webkit-transform:translateX(0);transform:translateX(0)}}.bounceInRight{-webkit-animation-name:bounceInRight;animation-name:bounceInRight}@-webkit-keyframes bounceOut{0%{-webkit-transform:scale(1);transform:scale(1)}25%{-webkit-transform:scale(0.95);transform:scale(0.95)}50%{opacity:1;-webkit-transform:scale(1.1);transform:scale(1.1)}100%{opacity:0;-webkit-transform:scale(0.3);transform:scale(0.3)}}@keyframes bounceOut{0%{-webkit-transform:scale(1);transform:scale(1)}25%{-webkit-transform:scale(0.95);transform:scale(0.95)}50%{opacity:1;-webkit-transform:scale(1.1);transform:scale(1.1)}100%{opacity:0;-webkit-transform:scale(0.3);transform:scale(0.3)}}.bounceOut{-webkit-animation-name:bounceOut;animation-name:bounceOut}@-webkit-keyframes bounceOutUp{0%{-webkit-transform:translateY(0);transform:translateY(0)}20%{opacity:1;-webkit-transform:translateY(20px);transform:translateY(20px)}100%{opacity:0;-webkit-transform:translateY(-2000px);transform:translateY(-2000px)}}@keyframes bounceOutUp{0%{-webkit-transform:translateY(0);transform:translateY(0)}20%{opacity:1;-webkit-transform:translateY(20px);transform:translateY(20px)}100%{opacity:0;-webkit-transform:translateY(-2000px);transform:translateY(-2000px)}}.bounceOutUp{-webkit-animation-name:bounceOutUp;animation-name:bounceOutUp}@-webkit-keyframes bounceOutDown{0%{-webkit-transform:translateY(0);transform:translateY(0)}20%{opacity:1;-webkit-transform:translateY(-20px);transform:translateY(-20px)}100%{opacity:0;-webkit-transform:translateY(2000px);transform:translateY(2000px)}}@keyframes bounceOutDown{0%{-webkit-transform:translateY(0);transform:translateY(0)}20%{opacity:1;-webkit-transform:translateY(-20px);transform:translateY(-20px)}100%{opacity:0;-webkit-transform:translateY(2000px);transform:translateY(2000px)}}.bounceOutDown{-webkit-animation-name:bounceOutDown;animation-name:bounceOutDown}@-webkit-keyframes bounceOutLeft{0%{-webkit-transform:translateX(0);transform:translateX(0)}20%{opacity:1;-webkit-transform:translateX(20px);transform:translateX(20px)}100%{opacity:0;-webkit-transform:translateX(-2000px);transform:translateX(-2000px)}}@keyframes bounceOutLeft{0%{-webkit-transform:translateX(0);transform:translateX(0)}20%{opacity:1;-webkit-transform:translateX(20px);transform:translateX(20px)}100%{opacity:0;-webkit-transform:translateX(-2000px);transform:translateX(-2000px)}}.bounceOutLeft{-webkit-animation-name:bounceOutLeft;animation-name:bounceOutLeft}@-webkit-keyframes bounceOutRight{0%{-webkit-transform:translateX(0);transform:translateX(0)}20%{opacity:1;-webkit-transform:translateX(-20px);transform:translateX(-20px)}100%{opacity:0;-webkit-transform:translateX(2000px);transform:translateX(2000px)}}@keyframes bounceOutRight{0%{-webkit-transform:translateX(0);transform:translateX(0)}20%{opacity:1;-webkit-transform:translateX(-20px);transform:translateX(-20px)}100%{opacity:0;-webkit-transform:translateX(2000px);transform:translateX(2000px)}}.bounceOutRight{-webkit-animation-name:bounceOutRight;animation-name:bounceOutRight}@-webkit-keyframes rotateIn{0%{-webkit-transform-origin:center center;transform-origin:center center;-webkit-transform:rotate(-200deg);transform:rotate(-200deg);opacity:0}100%{-webkit-transform-origin:center center;transform-origin:center center;-webkit-transform:rotate(0);transform:rotate(0);opacity:1}}@keyframes rotateIn{0%{-webkit-transform-origin:center center;transform-origin:center center;-webkit-transform:rotate(-200deg);transform:rotate(-200deg);opacity:0}100%{-webkit-transform-origin:center center;transform-origin:center center;-webkit-transform:rotate(0);transform:rotate(0);opacity:1}}@-webkit-keyframes fadeIn{0%{opacity:0}100%{opacity:1}}@keyframes fadeIn{0%{opacity:0}100%{opacity:1}}.fadeIn{-webkit-animation-name:fadeIn;animation-name:fadeIn}@-webkit-keyframes fadeInUp{0%{opacity:0;-webkit-transform:translateY(20px);transform:translateY(20px)}100%{opacity:1;-webkit-transform:translateY(0);transform:translateY(0)}}@keyframes fadeInUp{0%{opacity:0;-webkit-transform:translateY(20px);transform:translateY(20px)}100%{opacity:1;-webkit-transform:translateY(0);transform:translateY(0)}}.fadeInUp{-webkit-animation-name:fadeInUp;animation-name:fadeInUp}@-webkit-keyframes fadeInDown{0%{opacity:0;-webkit-transform:translateY(-20px);transform:translateY(-20px)}100%{opacity:1;-webkit-transform:translateY(0);transform:translateY(0)}}@keyframes fadeInDown{0%{opacity:0;-webkit-transform:translateY(-20px);transform:translateY(-20px)}100%{opacity:1;-webkit-transform:translateY(0);transform:translateY(0)}}.fadeInDown{-webkit-animation-name:fadeInDown;animation-name:fadeInDown}@-webkit-keyframes fadeInLeft{0%{opacity:0;-webkit-transform:translateX(-20px);transform:translateX(-20px)}100%{opacity:1;-webkit-transform:translateX(0);transform:translateX(0)}}@keyframes fadeInLeft{0%{opacity:0;-webkit-transform:translateX(-20px);transform:translateX(-20px)}100%{opacity:1;-webkit-transform:translateX(0);transform:translateX(0)}}.fadeInLeft{-webkit-animation-name:fadeInLeft;animation-name:fadeInLeft}@-webkit-keyframes fadeInRight{0%{opacity:0;-webkit-transform:translateX(20px);transform:translateX(20px)}100%{opacity:1;-webkit-transform:translateX(0);transform:translateX(0)}}@keyframes fadeInRight{0%{opacity:0;-webkit-transform:translateX(20px);transform:translateX(20px)}100%{opacity:1;-webkit-transform:translateX(0);transform:translateX(0)}}.fadeInRight{-webkit-animation-name:fadeInRight;animation-name:fadeInRight}@-webkit-keyframes fadeInUpBig{0%{opacity:0;-webkit-transform:translateY(2000px);transform:translateY(2000px)}100%{opacity:1;-webkit-transform:translateY(0);transform:translateY(0)}}@keyframes fadeInUpBig{0%{opacity:0;-webkit-transform:translateY(2000px);transform:translateY(2000px)}100%{opacity:1;-webkit-transform:translateY(0);transform:translateY(0)}}.fadeInUpBig{-webkit-animation-name:fadeInUpBig;animation-name:fadeInUpBig}@-webkit-keyframes fadeInDownBig{0%{opacity:0;-webkit-transform:translateY(-2000px);transform:translateY(-2000px)}100%{opacity:1;-webkit-transform:translateY(0);transform:translateY(0)}}@keyframes fadeInDownBig{0%{opacity:0;-webkit-transform:translateY(-2000px);transform:translateY(-2000px)}100%{opacity:1;-webkit-transform:translateY(0);transform:translateY(0)}}.fadeInDownBig{-webkit-animation-name:fadeInDownBig;animation-name:fadeInDownBig}@-webkit-keyframes fadeInLeftBig{0%{opacity:0;-webkit-transform:translateX(-2000px);transform:translateX(-2000px)}100%{opacity:1;-webkit-transform:translateX(0);transform:translateX(0)}}@keyframes fadeInLeftBig{0%{opacity:0;-webkit-transform:translateX(-2000px);transform:translateX(-2000px)}100%{opacity:1;-webkit-transform:translateX(0);transform:translateX(0)}}.fadeInLeftBig{-webkit-animation-name:fadeInLeftBig;animation-name:fadeInLeftBig}@-webkit-keyframes fadeInRightBig{0%{opacity:0;-webkit-transform:translateX(2000px);transform:translateX(2000px)}100%{opacity:1;-webkit-transform:translateX(0);transform:translateX(0)}}@keyframes fadeInRightBig{0%{opacity:0;-webkit-transform:translateX(2000px);transform:translateX(2000px)}100%{opacity:1;-webkit-transform:translateX(0);transform:translateX(0)}}.fadeInRightBig{-webkit-animation-name:fadeInRightBig;animation-name:fadeInRightBig}@-webkit-keyframes fadeOut{0%{opacity:1}100%{opacity:0}}@keyframes fadeOut{0%{opacity:1}100%{opacity:0}}.fadeOut,.dismiss{-webkit-animation-name:fadeOut;animation-name:fadeOut}@-webkit-keyframes fadeOutUp{0%{opacity:1;-webkit-transform:translateY(0);transform:translateY(0)}100%{opacity:0;-webkit-transform:translateY(-20px);transform:translateY(-20px)}}@keyframes fadeOutUp{0%{opacity:1;-webkit-transform:translateY(0);transform:translateY(0)}100%{opacity:0;-webkit-transform:translateY(-20px);transform:translateY(-20px)}}.fadeOutUp{-webkit-animation-name:fadeOutUp;animation-name:fadeOutUp}@-webkit-keyframes fadeOutDown{0%{opacity:1;-webkit-transform:translateY(0);transform:translateY(0)}100%{opacity:0;-webkit-transform:translateY(20px);transform:translateY(20px)}}@keyframes fadeOutDown{0%{opacity:1;-webkit-transform:translateY(0);transform:translateY(0)}100%{opacity:0;-webkit-transform:translateY(20px);transform:translateY(20px)}}.fadeOutDown{-webkit-animation-name:fadeOutDown;animation-name:fadeOutDown}@-webkit-keyframes fadeOutLeft{0%{opacity:1;-webkit-transform:translateX(0);transform:translateX(0)}100%{opacity:0;-webkit-transform:translateX(-20px);transform:translateX(-20px)}}@keyframes fadeOutLeft{0%{opacity:1;-webkit-transform:translateX(0);transform:translateX(0)}100%{opacity:0;-webkit-transform:translateX(-20px);transform:translateX(-20px)}}.fadeOutLeft{-webkit-animation-name:fadeOutLeft;animation-name:fadeOutLeft}@-webkit-keyframes fadeOutRight{0%{opacity:1;-webkit-transform:translateX(0);transform:translateX(0)}100%{opacity:0;-webkit-transform:translateX(20px);transform:translateX(20px)}}@keyframes fadeOutRight{0%{opacity:1;-webkit-transform:translateX(0);transform:translateX(0)}100%{opacity:0;-webkit-transform:translateX(20px);transform:translateX(20px)}}.fadeOutRight{-webkit-animation-name:fadeOutRight;animation-name:fadeOutRight}@-webkit-keyframes fadeOutUpBig{0%{opacity:1;-webkit-transform:translateY(0);transform:translateY(0)}100%{opacity:0;-webkit-transform:translateY(-2000px);transform:translateY(-2000px)}}@keyframes fadeOutUpBig{0%{opacity:1;-webkit-transform:translateY(0);transform:translateY(0)}100%{opacity:0;-webkit-transform:translateY(-2000px);transform:translateY(-2000px)}}.fadeOutUpBig{-webkit-animation-name:fadeOutUpBig;animation-name:fadeOutUpBig}@-webkit-keyframes fadeOutDownBig{0%{opacity:1;-webkit-transform:translateY(0);transform:translateY(0)}100%{opacity:0;-webkit-transform:translateY(2000px);transform:translateY(2000px)}}@keyframes fadeOutDownBig{0%{opacity:1;-webkit-transform:translateY(0);transform:translateY(0)}100%{opacity:0;-webkit-transform:translateY(2000px);transform:translateY(2000px)}}.fadeOutDownBig{-webkit-animation-name:fadeOutDownBig;animation-name:fadeOutDownBig}@-webkit-keyframes fadeOutLeftBig{0%{opacity:1;-webkit-transform:translateX(0);transform:translateX(0)}100%{opacity:0;-webkit-transform:translateX(-2000px);transform:translateX(-2000px)}}@keyframes fadeOutLeftBig{0%{opacity:1;-webkit-transform:translateX(0);transform:translateX(0)}100%{opacity:0;-webkit-transform:translateX(-2000px);transform:translateX(-2000px)}}.fadeOutLeftBig{-webkit-animation-name:fadeOutLeftBig;animation-name:fadeOutLeftBig}@-webkit-keyframes fadeOutRightBig{0%{opacity:1;-webkit-transform:translateX(0);transform:translateX(0)}100%{opacity:0;-webkit-transform:translateX(2000px);transform:translateX(2000px)}}@keyframes fadeOutRightBig{0%{opacity:1;-webkit-transform:translateX(0);transform:translateX(0)}100%{opacity:0;-webkit-transform:translateX(2000px);transform:translateX(2000px)}}.fadeOutRightBig{-webkit-animation-name:fadeOutRightBig;animation-name:fadeOutRightBig}@-webkit-keyframes flash{0%,50%,100%{opacity:1}25%,75%{opacity:0}}@keyframes flash{0%,50%,100%{opacity:1}25%,75%{opacity:0}}.flash{-webkit-animation-name:flash;animation-name:flash}@-webkit-keyframes flip{0%{-webkit-transform:perspective(400px) rotateY(0);transform:perspective(400px) rotateY(0);-webkit-animation-timing-function:ease-out;animation-timing-function:ease-out}40%{-webkit-transform:perspective(400px) translateZ(150px) rotateY(170deg);transform:perspective(400px) translateZ(150px) rotateY(170deg);-webkit-animation-timing-function:ease-out;animation-timing-function:ease-out}50%{-webkit-transform:perspective(400px) translateZ(150px) rotateY(190deg) scale(1);transform:perspective(400px) translateZ(150px) rotateY(190deg) scale(1);-webkit-animation-timing-function:ease-in;animation-timing-function:ease-in}80%{-webkit-transform:perspective(400px) rotateY(360deg) scale(0.95);transform:perspective(400px) rotateY(360deg) scale(0.95);-webkit-animation-timing-function:ease-in;animation-timing-function:ease-in}100%{-webkit-transform:perspective(400px) scale(1);transform:perspective(400px) scale(1);-webkit-animation-timing-function:ease-in;animation-timing-function:ease-in}}@keyframes flip{0%{-webkit-transform:perspective(400px) rotateY(0);transform:perspective(400px) rotateY(0);-webkit-animation-timing-function:ease-out;animation-timing-function:ease-out}40%{-webkit-transform:perspective(400px) translateZ(150px) rotateY(170deg);transform:perspective(400px) translateZ(150px) rotateY(170deg);-webkit-animation-timing-function:ease-out;animation-timing-function:ease-out}50%{-webkit-transform:perspective(400px) translateZ(150px) rotateY(190deg) scale(1);transform:perspective(400px) translateZ(150px) rotateY(190deg) scale(1);-webkit-animation-timing-function:ease-in;animation-timing-function:ease-in}80%{-webkit-transform:perspective(400px) rotateY(360deg) scale(0.95);transform:perspective(400px) rotateY(360deg) scale(0.95);-webkit-animation-timing-function:ease-in;animation-timing-function:ease-in}100%{-webkit-transform:perspective(400px) scale(1);transform:perspective(400px) scale(1);-webkit-animation-timing-function:ease-in;animation-timing-function:ease-in}}.flip{-webkit-backface-visibility:visible!important;backface-visibility:visible!important;-webkit-animation-name:flip;animation-name:flip}@-webkit-keyframes flipInX{0%{-webkit-transform:perspective(400px) rotateX(90deg);transform:perspective(400px) rotateX(90deg);opacity:0}40%{-webkit-transform:perspective(400px) rotateX(-10deg);transform:perspective(400px) rotateX(-10deg)}70%{-webkit-transform:perspective(400px) rotateX(10deg);transform:perspective(400px) rotateX(10deg)}100%{-webkit-transform:perspective(400px) rotateX(0deg);transform:perspective(400px) rotateX(0deg);opacity:1}}@keyframes flipInX{0%{-webkit-transform:perspective(400px) rotateX(90deg);transform:perspective(400px) rotateX(90deg);opacity:0}40%{-webkit-transform:perspective(400px) rotateX(-10deg);transform:perspective(400px) rotateX(-10deg)}70%{-webkit-transform:perspective(400px) rotateX(10deg);transform:perspective(400px) rotateX(10deg)}100%{-webkit-transform:perspective(400px) rotateX(0deg);transform:perspective(400px) rotateX(0deg);opacity:1}}.flipInX{-webkit-backface-visibility:visible!important;backface-visibility:visible!important;-webkit-animation-name:flipInX;animation-name:flipInX}@-webkit-keyframes flipOutX{0%{-webkit-transform:perspective(400px) rotateX(0deg);transform:perspective(400px) rotateX(0deg);opacity:1}100%{-webkit-transform:perspective(400px) rotateX(90deg);transform:perspective(400px) rotateX(90deg);opacity:0}}@keyframes flipOutX{0%{-webkit-transform:perspective(400px) rotateX(0deg);transform:perspective(400px) rotateX(0deg);opacity:1}100%{-webkit-transform:perspective(400px) rotateX(90deg);transform:perspective(400px) rotateX(90deg);opacity:0}}.flipOutX{-webkit-animation-name:flipOutX;animation-name:flipOutX;-webkit-backface-visibility:visible!important;backface-visibility:visible!important}@-webkit-keyframes flipInY{0%{-webkit-transform:perspective(400px) rotateY(90deg);transform:perspective(400px) rotateY(90deg);opacity:0}40%{-webkit-transform:perspective(400px) rotateY(-10deg);transform:perspective(400px) rotateY(-10deg)}70%{-webkit-transform:perspective(400px) rotateY(10deg);transform:perspective(400px) rotateY(10deg)}100%{-webkit-transform:perspective(400px) rotateY(0deg);transform:perspective(400px) rotateY(0deg);opacity:1}}@keyframes flipInY{0%{-webkit-transform:perspective(400px) rotateY(90deg);transform:perspective(400px) rotateY(90deg);opacity:0}40%{-webkit-transform:perspective(400px) rotateY(-10deg);transform:perspective(400px) rotateY(-10deg)}70%{-webkit-transform:perspective(400px) rotateY(10deg);transform:perspective(400px) rotateY(10deg)}100%{-webkit-transform:perspective(400px) rotateY(0deg);transform:perspective(400px) rotateY(0deg);opacity:1}}.flipInY{-webkit-backface-visibility:visible!important;backface-visibility:visible!important;-webkit-animation-name:flipInY;animation-name:flipInY}@-webkit-keyframes flipOutY{0%{-webkit-transform:perspective(400px) rotateY(0deg);transform:perspective(400px) rotateY(0deg);opacity:1}100%{-webkit-transform:perspective(400px) rotateY(90deg);transform:perspective(400px) rotateY(90deg);opacity:0}}@keyframes flipOutY{0%{-webkit-transform:perspective(400px) rotateY(0deg);transform:perspective(400px) rotateY(0deg);opacity:1}100%{-webkit-transform:perspective(400px) rotateY(90deg);transform:perspective(400px) rotateY(90deg);opacity:0}}.flipOutY{-webkit-backface-visibility:visible!important;backface-visibility:visible!important;-webkit-animation-name:flipOutY;animation-name:flipOutY}@-webkit-keyframes hinge{0%{-webkit-transform:rotate(0);transform:rotate(0);-webkit-transform-origin:top left;transform-origin:top left;-webkit-animation-timing-function:ease-in-out;animation-timing-function:ease-in-out}20%,60%{-webkit-transform:rotate(80deg);transform:rotate(80deg);-webkit-transform-origin:top left;transform-origin:top left;-webkit-animation-timing-function:ease-in-out;animation-timing-function:ease-in-out}40%{-webkit-transform:rotate(60deg);transform:rotate(60deg);-webkit-transform-origin:top left;transform-origin:top left;-webkit-animation-timing-function:ease-in-out;animation-timing-function:ease-in-out}80%{-webkit-transform:rotate(60deg) translateY(0);transform:rotate(60deg) translateY(0);opacity:1;-webkit-transform-origin:top left;transform-origin:top left;-webkit-animation-timing-function:ease-in-out;animation-timing-function:ease-in-out}100%{-webkit-transform:translateY(700px);transform:translateY(700px);opacity:0}}@keyframes hinge{0%{-webkit-transform:rotate(0);transform:rotate(0);-webkit-transform-origin:top left;transform-origin:top left;-webkit-animation-timing-function:ease-in-out;animation-timing-function:ease-in-out}20%,60%{-webkit-transform:rotate(80deg);transform:rotate(80deg);-webkit-transform-origin:top left;transform-origin:top left;-webkit-animation-timing-function:ease-in-out;animation-timing-function:ease-in-out}40%{-webkit-transform:rotate(60deg);transform:rotate(60deg);-webkit-transform-origin:top left;transform-origin:top left;-webkit-animation-timing-function:ease-in-out;animation-timing-function:ease-in-out}80%{-webkit-transform:rotate(60deg) translateY(0);transform:rotate(60deg) translateY(0);opacity:1;-webkit-transform-origin:top left;transform-origin:top left;-webkit-animation-timing-function:ease-in-out;animation-timing-function:ease-in-out}100%{-webkit-transform:translateY(700px);transform:translateY(700px);opacity:0}}.hinge{-webkit-animation-name:hinge;animation-name:hinge}.animated.hinge{-webkit-animation-duration:2s;animation-duration:2s}@-webkit-keyframes lightSpeedIn{0%{-webkit-transform:translateX(100%) skewX(-30deg);transform:translateX(100%) skewX(-30deg);opacity:0}60%{-webkit-transform:translateX(-20%) skewX(30deg);transform:translateX(-20%) skewX(30deg);opacity:1}80%{-webkit-transform:translateX(0%) skewX(-15deg);transform:translateX(0%) skewX(-15deg);opacity:1}100%{-webkit-transform:translateX(0%) skewX(0deg);transform:translateX(0%) skewX(0deg);opacity:1}}@keyframes lightSpeedIn{0%{-webkit-transform:translateX(100%) skewX(-30deg);transform:translateX(100%) skewX(-30deg);opacity:0}60%{-webkit-transform:translateX(-20%) skewX(30deg);transform:translateX(-20%) skewX(30deg);opacity:1}80%{-webkit-transform:translateX(0%) skewX(-15deg);transform:translateX(0%) skewX(-15deg);opacity:1}100%{-webkit-transform:translateX(0%) skewX(0deg);transform:translateX(0%) skewX(0deg);opacity:1}}.lightSpeedIn{-webkit-animation-name:lightSpeedIn;animation-name:lightSpeedIn;-webkit-animation-timing-function:ease-out;animation-timing-function:ease-out}.animated.lightSpeedIn{-webkit-animation-duration:.5s;animation-duration:.5s}@-webkit-keyframes lightSpeedOut{0%{-webkit-transform:translateX(0%) skewX(0deg);transform:translateX(0%) skewX(0deg);opacity:1}100%{-webkit-transform:translateX(100%) skewX(-30deg);transform:translateX(100%) skewX(-30deg);opacity:0}}@keyframes lightSpeedOut{0%{-webkit-transform:translateX(0%) skewX(0deg);transform:translateX(0%) skewX(0deg);opacity:1}100%{-webkit-transform:translateX(100%) skewX(-30deg);transform:translateX(100%) skewX(-30deg);opacity:0}}.lightSpeedOut{-webkit-animation-name:lightSpeedOut;animation-name:lightSpeedOut;-webkit-animation-timing-function:ease-in;animation-timing-function:ease-in}.animated.lightSpeedOut{-webkit-animation-duration:.25s;animation-duration:.25s}@-webkit-keyframes pulse{0%{-webkit-transform:scale(1);transform:scale(1)}50%{-webkit-transform:scale(1.1);transform:scale(1.1)}100%{-webkit-transform:scale(1);transform:scale(1)}}@keyframes pulse{0%{-webkit-transform:scale(1);transform:scale(1)}50%{-webkit-transform:scale(1.1);transform:scale(1.1)}100%{-webkit-transform:scale(1);transform:scale(1)}}.pulse{-webkit-animation-name:pulse;animation-name:pulse}@-webkit-keyframes rollIn{0%{opacity:0;-webkit-transform:translateX(-100%) rotate(-120deg);transform:translateX(-100%) rotate(-120deg)}100%{opacity:1;-webkit-transform:translateX(0px) rotate(0deg);transform:translateX(0px) rotate(0deg)}}@keyframes rollIn{0%{opacity:0;-webkit-transform:translateX(-100%) rotate(-120deg);transform:translateX(-100%) rotate(-120deg)}100%{opacity:1;-webkit-transform:translateX(0px) rotate(0deg);transform:translateX(0px) rotate(0deg)}}.rollIn{-webkit-animation-name:rollIn;animation-name:rollIn}@-webkit-keyframes rollOut{0%{opacity:1;-webkit-transform:translateX(0px) rotate(0deg);transform:translateX(0px) rotate(0deg)}100%{opacity:0;-webkit-transform:translateX(100%) rotate(120deg);transform:translateX(100%) rotate(120deg)}}@keyframes rollOut{0%{opacity:1;-webkit-transform:translateX(0px) rotate(0deg);transform:translateX(0px) rotate(0deg)}100%{opacity:0;-webkit-transform:translateX(100%) rotate(120deg);transform:translateX(100%) rotate(120deg)}}.rollOut{-webkit-animation-name:rollOut;animation-name:rollOut}@keyframes lightSpeedIn{0%{-webkit-transform:translateX(100%) skewX(-30deg);transform:translateX(100%) skewX(-30deg);opacity:0}60%{-webkit-transform:translateX(-20%) skewX(30deg);transform:translateX(-20%) skewX(30deg);opacity:1}80%{-webkit-transform:translateX(0%) skewX(-15deg);transform:translateX(0%) skewX(-15deg);opacity:1}100%{-webkit-transform:translateX(0%) skewX(0deg);transform:translateX(0%) skewX(0deg);opacity:1}}.rotateIn{-webkit-animation-name:rotateIn;animation-name:rotateIn}@-webkit-keyframes rotateInUpLeft{0%{-webkit-transform-origin:left bottom;transform-origin:left bottom;-webkit-transform:rotate(90deg);transform:rotate(90deg);opacity:0}100%{-webkit-transform-origin:left bottom;transform-origin:left bottom;-webkit-transform:rotate(0);transform:rotate(0);opacity:1}}@keyframes rotateInUpLeft{0%{-webkit-transform-origin:left bottom;transform-origin:left bottom;-webkit-transform:rotate(90deg);transform:rotate(90deg);opacity:0}100%{-webkit-transform-origin:left bottom;transform-origin:left bottom;-webkit-transform:rotate(0);transform:rotate(0);opacity:1}}.rotateInUpLeft{-webkit-animation-name:rotateInUpLeft;animation-name:rotateInUpLeft}@-webkit-keyframes rotateInDownLeft{0%{-webkit-transform-origin:left bottom;transform-origin:left bottom;-webkit-transform:rotate(-90deg);transform:rotate(-90deg);opacity:0}100%{-webkit-transform-origin:left bottom;transform-origin:left bottom;-webkit-transform:rotate(0);transform:rotate(0);opacity:1}}@keyframes rotateInDownLeft{0%{-webkit-transform-origin:left bottom;transform-origin:left bottom;-webkit-transform:rotate(-90deg);transform:rotate(-90deg);opacity:0}100%{-webkit-transform-origin:left bottom;transform-origin:left bottom;-webkit-transform:rotate(0);transform:rotate(0);opacity:1}}.rotateInDownLeft{-webkit-animation-name:rotateInDownLeft;animation-name:rotateInDownLeft}@-webkit-keyframes rotateInUpRight{0%{-webkit-transform-origin:right bottom;transform-origin:right bottom;-webkit-transform:rotate(-90deg);transform:rotate(-90deg);opacity:0}100%{-webkit-transform-origin:right bottom;transform-origin:right bottom;-webkit-transform:rotate(0);transform:rotate(0);opacity:1}}@keyframes rotateInUpRight{0%{-webkit-transform-origin:right bottom;transform-origin:right bottom;-webkit-transform:rotate(-90deg);transform:rotate(-90deg);opacity:0}100%{-webkit-transform-origin:right bottom;transform-origin:right bottom;-webkit-transform:rotate(0);transform:rotate(0);opacity:1}}.rotateInUpRight{-webkit-animation-name:rotateInUpRight;animation-name:rotateInUpRight}@-webkit-keyframes rotateInDownRight{0%{-webkit-transform-origin:right bottom;transform-origin:right bottom;-webkit-transform:rotate(90deg);transform:rotate(90deg);opacity:0}100%{-webkit-transform-origin:right bottom;transform-origin:right bottom;-webkit-transform:rotate(0);transform:rotate(0);opacity:1}}@keyframes rotateInDownRight{0%{-webkit-transform-origin:right bottom;transform-origin:right bottom;-webkit-transform:rotate(90deg);transform:rotate(90deg);opacity:0}100%{-webkit-transform-origin:right bottom;transform-origin:right bottom;-webkit-transform:rotate(0);transform:rotate(0);opacity:1}}.rotateInDownRight{-webkit-animation-name:rotateInDownRight;animation-name:rotateInDownRight}@-webkit-keyframes rotateOut{0%{-webkit-transform-origin:center center;transform-origin:center center;-webkit-transform:rotate(0);transform:rotate(0);opacity:1}100%{-webkit-transform-origin:center center;transform-origin:center center;-webkit-transform:rotate(200deg);transform:rotate(200deg);opacity:0}}@keyframes rotateOut{0%{-webkit-transform-origin:center center;transform-origin:center center;-webkit-transform:rotate(0);transform:rotate(0);opacity:1}100%{-webkit-transform-origin:center center;transform-origin:center center;-webkit-transform:rotate(200deg);transform:rotate(200deg);opacity:0}}.rotateOut{-webkit-animation-name:rotateOut;animation-name:rotateOut}@-webkit-keyframes rotateOutUpLeft{0%{-webkit-transform-origin:left bottom;transform-origin:left bottom;-webkit-transform:rotate(0);transform:rotate(0);opacity:1}100%{-webkit-transform-origin:left bottom;transform-origin:left bottom;-webkit-transform:rotate(-90deg);transform:rotate(-90deg);opacity:0}}@keyframes rotateOutUpLeft{0%{-webkit-transform-origin:left bottom;transform-origin:left bottom;-webkit-transform:rotate(0);transform:rotate(0);opacity:1}100%{-webkit-transform-origin:left bottom;transform-origin:left bottom;-webkit-transform:rotate(-90deg);transform:rotate(-90deg);opacity:0}}.rotateOutUpLeft{-webkit-animation-name:rotateOutUpLeft;animation-name:rotateOutUpLeft}@-webkit-keyframes rotateOutDownLeft{0%{-webkit-transform-origin:left bottom;transform-origin:left bottom;-webkit-transform:rotate(0);transform:rotate(0);opacity:1}100%{-webkit-transform-origin:left bottom;transform-origin:left bottom;-webkit-transform:rotate(90deg);transform:rotate(90deg);opacity:0}}@keyframes rotateOutDownLeft{0%{-webkit-transform-origin:left bottom;transform-origin:left bottom;-webkit-transform:rotate(0);transform:rotate(0);opacity:1}100%{-webkit-transform-origin:left bottom;transform-origin:left bottom;-webkit-transform:rotate(90deg);transform:rotate(90deg);opacity:0}}.rotateOutDownLeft{-webkit-animation-name:rotateOutDownLeft;animation-name:rotateOutDownLeft}@-webkit-keyframes rotateOutUpRight{0%{-webkit-transform-origin:right bottom;transform-origin:right bottom;-webkit-transform:rotate(0);transform:rotate(0);opacity:1}100%{-webkit-transform-origin:right bottom;transform-origin:right bottom;-webkit-transform:rotate(90deg);transform:rotate(90deg);opacity:0}}@keyframes rotateOutUpRight{0%{-webkit-transform-origin:right bottom;transform-origin:right bottom;-webkit-transform:rotate(0);transform:rotate(0);opacity:1}100%{-webkit-transform-origin:right bottom;transform-origin:right bottom;-webkit-transform:rotate(90deg);transform:rotate(90deg);opacity:0}}.rotateOutUpRight{-webkit-animation-name:rotateOutUpRight;animation-name:rotateOutUpRight}@-webkit-keyframes rotateOutDownRight{0%{-webkit-transform-origin:right bottom;transform-origin:right bottom;-webkit-transform:rotate(0);transform:rotate(0);opacity:1}100%{-webkit-transform-origin:right bottom;transform-origin:right bottom;-webkit-transform:rotate(-90deg);transform:rotate(-90deg);opacity:0}}@keyframes rotateOutDownRight{0%{-webkit-transform-origin:right bottom;transform-origin:right bottom;-webkit-transform:rotate(0);transform:rotate(0);opacity:1}100%{-webkit-transform-origin:right bottom;transform-origin:right bottom;-webkit-transform:rotate(-90deg);transform:rotate(-90deg);opacity:0}}.rotateOutDownRight{-webkit-animation-name:rotateOutDownRight;animation-name:rotateOutDownRight}@-webkit-keyframes shake{0%,100%{-webkit-transform:translateX(0);transform:translateX(0)}10%,30%,50%,70%,90%{-webkit-transform:translateX(-10px);transform:translateX(-10px)}20%,40%,60%,80%{-webkit-transform:translateX(10px);transform:translateX(10px)}}@keyframes shake{0%,100%{-webkit-transform:translateX(0);transform:translateX(0)}10%,30%,50%,70%,90%{-webkit-transform:translateX(-10px);transform:translateX(-10px)}20%,40%,60%,80%{-webkit-transform:translateX(10px);transform:translateX(10px)}}.shake{-webkit-animation-name:shake;animation-name:shake}.spin{display:inline;-webkit-animation:spin 2s linear;animation:spin 2s linear;-webkit-animation-iteration-count:infinite;animation-iteration-count:infinite}.spin-once{display:inline;-webkit-animation:spin 2s linear;animation:spin 2s linear;-webkit-animation-iteration-count:1;animation-iteration-count:1}.spin-twice{display:inline;-webkit-animation:spin 2s linear;animation:spin 2s linear;-webkit-animation-iteration-count:2;animation-iteration-count:2}.spin-thrice{display:inline;-webkit-animation:spin 2s linear;animation:spin 2s linear;-webkit-animation-iteration-count:3;animation-iteration-count:3}.spin-fast{display:inline;-webkit-animation:spin 1s linear;animation:spin 1s linear;-webkit-animation-iteration-count:infinite;animation-iteration-count:infinite}.spin-fast-once{display:inline;-webkit-animation:spin 1s linear;animation:spin 1s linear;-webkit-animation-iteration-count:1;animation-iteration-count:1}.spin-fast-twice{display:inline;-webkit-animation:spin 1s linear;animation:spin 1s linear;-webkit-animation-iteration-count:2;animation-iteration-count:2}.spin-fast-thrice{display:inline;-webkit-animation:spin 1s linear;animation:spin 1s linear;-webkit-animation-iteration-count:3;animation-iteration-count:3}.spin-slow{display:inline;-webkit-animation:spin 5s linear;animation:spin 5s linear;-webkit-animation-iteration-count:infinite;animation-iteration-count:infinite}.spin-slow-once{display:inline;-webkit-animation:spin 5s linear;animation:spin 5s linear;-webkit-animation-iteration-count:1;animation-iteration-count:1}.spin-slow-twice{display:inline;-webkit-animation:spin 5s linear;animation:spin 5s linear;-webkit-animation-iteration-count:2;animation-iteration-count:2}.spin-slow-thrice{display:inline;-webkit-animation:spin 5s linear;animation:spin 5s linear;-webkit-animation-iteration-count:3;animation-iteration-count:3}@keyframes spin{0%{-webkit-transform:rotate(0deg);transform:rotate(0deg)}100%{-webkit-transform:rotate(359deg);transform:rotate(359deg)}}@-webkit-keyframes swing{20%{-webkit-transform:rotate(15deg);transform:rotate(15deg)}40%{-webkit-transform:rotate(-10deg);transform:rotate(-10deg)}60%{-webkit-transform:rotate(5deg);transform:rotate(5deg)}80%{-webkit-transform:rotate(-5deg);transform:rotate(-5deg)}100%{-webkit-transform:rotate(0deg);transform:rotate(0deg)}}@keyframes swing{20%{-webkit-transform:rotate(15deg);transform:rotate(15deg)}40%{-webkit-transform:rotate(-10deg);transform:rotate(-10deg)}60%{-webkit-transform:rotate(5deg);transform:rotate(5deg)}80%{-webkit-transform:rotate(-5deg);transform:rotate(-5deg)}100%{-webkit-transform:rotate(0deg);transform:rotate(0deg)}}.swing{-webkit-transform-origin:top center;transform-origin:top center;-webkit-animation-name:swing;animation-name:swing}@-webkit-keyframes tada{0%{-webkit-transform:scale(1);transform:scale(1)}10%,20%{-webkit-transform:scale(0.9) rotate(-3deg);transform:scale(0.9) rotate(-3deg)}30%,50%,70%,90%{-webkit-transform:scale(1.1) rotate(3deg);transform:scale(1.1) rotate(3deg)}40%,60%,80%{-webkit-transform:scale(1.1) rotate(-3deg);transform:scale(1.1) rotate(-3deg)}100%{-webkit-transform:scale(1) rotate(0);transform:scale(1) rotate(0)}}@keyframes tada{0%{-webkit-transform:scale(1);transform:scale(1)}10%,20%{-webkit-transform:scale(0.9) rotate(-3deg);transform:scale(0.9) rotate(-3deg)}30%,50%,70%,90%{-webkit-transform:scale(1.1) rotate(3deg);transform:scale(1.1) rotate(3deg)}40%,60%,80%{-webkit-transform:scale(1.1) rotate(-3deg);transform:scale(1.1) rotate(-3deg)}100%{-webkit-transform:scale(1) rotate(0);transform:scale(1) rotate(0)}}.tada{-webkit-animation-name:tada;animation-name:tada}@-webkit-keyframes wiggle{0%{-webkit-transform:skewX(9deg);transform:skewX(9deg)}10%{-webkit-transform:skewX(-8deg);transform:skewX(-8deg)}20%{-webkit-transform:skewX(7deg);transform:skewX(7deg)}30%{-webkit-transform:skewX(-6deg);transform:skewX(-6deg)}40%{-webkit-transform:skewX(5deg);transform:skewX(5deg)}50%{-webkit-transform:skewX(-4deg);transform:skewX(-4deg)}60%{-webkit-transform:skewX(3deg);transform:skewX(3deg)}70%{-webkit-transform:skewX(-2deg);transform:skewX(-2deg)}80%{-webkit-transform:skewX(1deg);transform:skewX(1deg)}90%{-webkit-transform:skewX(0deg);transform:skewX(0deg)}100%{-webkit-transform:skewX(0deg);transform:skewX(0deg)}}@keyframes wiggle{0%{-webkit-transform:skewX(9deg);transform:skewX(9deg)}10%{-webkit-transform:skewX(-8deg);transform:skewX(-8deg)}20%{-webkit-transform:skewX(7deg);transform:skewX(7deg)}30%{-webkit-transform:skewX(-6deg);transform:skewX(-6deg)}40%{-webkit-transform:skewX(5deg);transform:skewX(5deg)}50%{-webkit-transform:skewX(-4deg);transform:skewX(-4deg)}60%{-webkit-transform:skewX(3deg);transform:skewX(3deg)}70%{-webkit-transform:skewX(-2deg);transform:skewX(-2deg)}80%{-webkit-transform:skewX(1deg);transform:skewX(1deg)}90%{-webkit-transform:skewX(0deg);transform:skewX(0deg)}100%{-webkit-transform:skewX(0deg);transform:skewX(0deg)}}.wiggle{-webkit-animation-name:wiggle;animation-name:wiggle;-webkit-animation-timing-function:ease-in;animation-timing-function:ease-in}.animated.wiggle{-webkit-animation-duration:.75s;animation-duration:.75s}@-webkit-keyframes wobble{0%{-webkit-transform:translateX(0%);transform:translateX(0%)}15%{-webkit-transform:translateX(-25%) rotate(-5deg);transform:translateX(-25%) rotate(-5deg)}30%{-webkit-transform:translateX(20%) rotate(3deg);transform:translateX(20%) rotate(3deg)}45%{-webkit-transform:translateX(-15%) rotate(-3deg);transform:translateX(-15%) rotate(-3deg)}60%{-webkit-transform:translateX(10%) rotate(2deg);transform:translateX(10%) rotate(2deg)}75%{-webkit-transform:translateX(-5%) rotate(-1deg);transform:translateX(-5%) rotate(-1deg)}100%{-webkit-transform:translateX(0%);transform:translateX(0%)}}@keyframes wobble{0%{-webkit-transform:translateX(0%);transform:translateX(0%)}15%{-webkit-transform:translateX(-25%) rotate(-5deg);transform:translateX(-25%) rotate(-5deg)}30%{-webkit-transform:translateX(20%) rotate(3deg);transform:translateX(20%) rotate(3deg)}45%{-webkit-transform:translateX(-15%) rotate(-3deg);transform:translateX(-15%) rotate(-3deg)}60%{-webkit-transform:translateX(10%) rotate(2deg);transform:translateX(10%) rotate(2deg)}75%{-webkit-transform:translateX(-5%) rotate(-1deg);transform:translateX(-5%) rotate(-1deg)}100%{-webkit-transform:translateX(0%);transform:translateX(0%)}}.wobble{-webkit-animation-name:wobble;animation-name:wobble}@-webkit-keyframes slideInDown{0%{opacity:0;-webkit-transform:translateY(-2000px);transform:translateY(-2000px)}100%{-webkit-transform:translateY(0);transform:translateY(0)}}@keyframes slideInDown{0%{opacity:0;-webkit-transform:translateY(-2000px);transform:translateY(-2000px)}100%{-webkit-transform:translateY(0);transform:translateY(0)}}.slideInDown{-webkit-animation-name:slideInDown;animation-name:slideInDown}@-webkit-keyframes slideInLeft{0%{opacity:0;-webkit-transform:translateX(-2000px);transform:translateX(-2000px)}100%{-webkit-transform:translateX(0);transform:translateX(0)}}@keyframes slideInLeft{0%{opacity:0;-webkit-transform:translateX(-2000px);transform:translateX(-2000px)}100%{-webkit-transform:translateX(0);transform:translateX(0)}}.slideInLeft{-webkit-animation-name:slideInLeft;animation-name:slideInLeft}@-webkit-keyframes slideInRight{0%{opacity:0;-webkit-transform:translateX(2000px);transform:translateX(2000px)}100%{-webkit-transform:translateX(0);transform:translateX(0)}}@keyframes slideInRight{0%{opacity:0;-webkit-transform:translateX(2000px);transform:translateX(2000px)}100%{-webkit-transform:translateX(0);transform:translateX(0)}}.slideInRight{-webkit-animation-name:slideInRight;animation-name:slideInRight}@-webkit-keyframes slideOutLeft{0%{-webkit-transform:translateX(0);transform:translateX(0)}100%{opacity:0;-webkit-transform:translateX(-2000px);transform:translateX(-2000px)}}@keyframes slideOutLeft{0%{-webkit-transform:translateX(0);transform:translateX(0)}100%{opacity:0;-webkit-transform:translateX(-2000px);transform:translateX(-2000px)}}.slideOutLeft{-webkit-animation-name:slideOutLeft;animation-name:slideOutLeft}@-webkit-keyframes slideOutRight{0%{-webkit-transform:translateX(0);transform:translateX(0)}100%{opacity:0;-webkit-transform:translateX(2000px);transform:translateX(2000px)}}@keyframes slideOutRight{0%{-webkit-transform:translateX(0);transform:translateX(0)}100%{opacity:0;-webkit-transform:translateX(2000px);transform:translateX(2000px)}}.slideOutRight{-webkit-animation-name:slideOutRight;animation-name:slideOutRight}@-webkit-keyframes slideOutUp{0%{-webkit-transform:translateY(0);transform:translateY(0)}100%{opacity:0;-webkit-transform:translateY(-2000px);transform:translateY(-2000px)}}@keyframes slideOutUp{0%{-webkit-transform:translateY(0);transform:translateY(0)}100%{opacity:0;-webkit-transform:translateY(-2000px);transform:translateY(-2000px)}}.slideOutUp{-webkit-animation-name:slideOutUp;animation-name:slideOutUp}.desktop-only{display:block}.ipad-only{display:none!important}.small-tablet-only{display:none!important}.mobile-only{display:none!important}@media (min-width:1025px){.desktop-only{display:block!important}.ipad-only{display:none!important}.small-tablet-only{display:none!important}.mobile-only{display:none!important}.hide-on-desktop{display:none!important}.no-border-desktop{border:0}.small-desktop{font-size:.8em}.medium-desktop{font-size:1em}.large-desktop{font-size:1.5em}.zero-desktop.padded,.zero-desktop.half-padded,.zero-desktop.double-padded,.zero-desktop.triple-padded{padding:0!important;margin:0!important}.zero-desktop.pad-top,.zero-desktop.half-pad-top,.zero-desktop.double-pad-top,.zero-desktop.triple-pad-top{padding:0!important;margin:0!important}.zero-desktop.pad-right,.zero-desktop.half-pad-right,.zero-desktop.double-pad-right,.zero-desktop.triple-pad-right{padding:0!important;margin:0!important}.zero-desktop.pad-bottom,.zero-desktop.half-pad-bottom,.zero-desktop.double-pad-bottom,.zero-desktop.triple-pad-bottom{padding:0!important;margin:0!important}.zero-desktop.pad-left,.zero-desktop.half-pad-left,.zero-desktop.double-pad-left,.zero-desktop.triple-pad-left{padding:0!important;margin:0!important}.zero-desktop.gapped,.zero-desktop.half-gapped,.zero-desktop.double-gapped,.zero-desktop.triple-gapped{padding:0!important;margin:0!important}.zero-desktop.gap-top,.zero-desktop.half-gap-top,.zero-desktop.double-gap-top,.zero-desktop.triple-gap-top{padding:0!important;margin:0!important}.zero-desktop.gap-right,.zero-desktop.half-gap-right,.zero-desktop.double-gap-right,.zero-desktop.triple-gap-right{padding:0!important;margin:0!important}.zero-desktop.gap-bottom,.zero-desktop.half-gap-bottom,.zero-desktop.double-gap-bottom,.zero-desktop.triple-gap-bottom{padding:0!important;margin:0!important}.zero-desktop.gap-left,.zero-desktop.half-gap-left,.zero-desktop.double-gap-left,.zero-desktop.triple-gap-left{padding:0!important;margin:0!important}.padded-desktop{padding:10px}.pad-top-desktop{padding-top:10px}.pad-right-desktop{padding-right:10px}.pad-bottom-desktop{padding-bottom:10px}.pad-left-desktop{padding-left:10px}.no-padding-desktop.padded,.no-padding-desktop.half-padded,.no-padding-desktop.double-padded,.no-padding-desktop.triple-padded,.no-padded-desktop.padded,.no-padded-desktop.half-padded,.no-padded-desktop.double-padded,.no-padded-desktop.triple-padded,.no-pad-desktop.padded,.no-pad-desktop.half-padded,.no-pad-desktop.double-padded,.no-pad-desktop.triple-padded{padding:0!important}.no-padding-desktop.pad-top,.no-padding-desktop.half-pad-top,.no-padding-desktop.double-pad-top,.no-padding-desktop.triple-pad-top,.no-padded-desktop.pad-top,.no-padded-desktop.half-pad-top,.no-padded-desktop.double-pad-top,.no-padded-desktop.triple-pad-top,.no-pad-desktop.pad-top,.no-pad-desktop.half-pad-top,.no-pad-desktop.double-pad-top,.no-pad-desktop.triple-pad-top{padding:0!important}.no-padding-desktop.pad-right,.no-padding-desktop.half-pad-right,.no-padding-desktop.double-pad-right,.no-padding-desktop.triple-pad-right,.no-padded-desktop.pad-right,.no-padded-desktop.half-pad-right,.no-padded-desktop.double-pad-right,.no-padded-desktop.triple-pad-right,.no-pad-desktop.pad-right,.no-pad-desktop.half-pad-right,.no-pad-desktop.double-pad-right,.no-pad-desktop.triple-pad-right{padding:0!important}.no-padding-desktop.pad-bottom,.no-padding-desktop.half-pad-bottom,.no-padding-desktop.double-pad-bottom,.no-padding-desktop.triple-pad-bottom,.no-padded-desktop.pad-bottom,.no-padded-desktop.half-pad-bottom,.no-padded-desktop.double-pad-bottom,.no-padded-desktop.triple-pad-bottom,.no-pad-desktop.pad-bottom,.no-pad-desktop.half-pad-bottom,.no-pad-desktop.double-pad-bottom,.no-pad-desktop.triple-pad-bottom{padding:0!important}.no-padding-desktop.pad-left,.no-padding-desktop.half-pad-left,.no-padding-desktop.double-pad-left,.no-padding-desktop.triple-pad-left,.no-padded-desktop.pad-left,.no-padded-desktop.half-pad-left,.no-padded-desktop.double-pad-left,.no-padded-desktop.triple-pad-left,.no-pad-desktop.pad-left,.no-pad-desktop.half-pad-left,.no-pad-desktop.double-pad-left,.no-pad-desktop.triple-pad-left{padding:0!important}.no-pad-top-desktop{padding-top:0}.no-pad-right-desktop{padding-right:0}.no-pad-bottom-desktop{padding-bottom:0}.no-pad-left-desktop{padding-left:0}.gapped-desktop{margin:10px}.gap-top-desktop{margin-top:10px}.gap-right-desktop{margin-right:10px}.gap-bottom-desktop{margin-bottom:10px}.gap-left-desktop{margin-left:10px}.no-margin-desktop.gapped,.no-margin-desktop.half-gapped,.no-margin-desktop.double-gapped,.no-margin-desktop.triple-gapped,.no-gapped-desktop.gapped,.no-gapped-desktop.half-gapped,.no-gapped-desktop.double-gapped,.no-gapped-desktop.triple-gapped,.no-gap-desktop.gapped,.no-gap-desktop.half-gapped,.no-gap-desktop.double-gapped,.no-gap-desktop.triple-gapped{margin:0!important}.no-margin-desktop.gap-top,.no-margin-desktop.half-gap-top,.no-margin-desktop.double-gap-top,.no-margin-desktop.triple-gap-top,.no-gapped-desktop.gap-top,.no-gapped-desktop.half-gap-top,.no-gapped-desktop.double-gap-top,.no-gapped-desktop.triple-gap-top,.no-gap-desktop.gap-top,.no-gap-desktop.half-gap-top,.no-gap-desktop.double-gap-top,.no-gap-desktop.triple-gap-top{margin:0!important}.no-margin-desktop.gap-right,.no-margin-desktop.half-gap-right,.no-margin-desktop.double-gap-right,.no-margin-desktop.triple-gap-right,.no-gapped-desktop.gap-right,.no-gapped-desktop.half-gap-right,.no-gapped-desktop.double-gap-right,.no-gapped-desktop.triple-gap-right,.no-gap-desktop.gap-right,.no-gap-desktop.half-gap-right,.no-gap-desktop.double-gap-right,.no-gap-desktop.triple-gap-right{margin:0!important}.no-margin-desktop.gap-bottom,.no-margin-desktop.half-gap-bottom,.no-margin-desktop.double-gap-bottom,.no-margin-desktop.triple-gap-bottom,.no-gapped-desktop.gap-bottom,.no-gapped-desktop.half-gap-bottom,.no-gapped-desktop.double-gap-bottom,.no-gapped-desktop.triple-gap-bottom,.no-gap-desktop.gap-bottom,.no-gap-desktop.half-gap-bottom,.no-gap-desktop.double-gap-bottom,.no-gap-desktop.triple-gap-bottom{margin:0!important}.no-margin-desktop.gap-left,.no-margin-desktop.half-gap-left,.no-margin-desktop.double-gap-left,.no-margin-desktop.triple-gap-left,.no-gapped-desktop.gap-left,.no-gapped-desktop.half-gap-left,.no-gapped-desktop.double-gap-left,.no-gapped-desktop.triple-gap-left,.no-gap-desktop.gap-left,.no-gap-desktop.half-gap-left,.no-gap-desktop.double-gap-left,.no-gap-desktop.triple-gap-left{margin:0!important}.no-gap-top-desktop{margin-top:0}.no-gap-right-desktop{margin-right:0}.no-gap-bottom-desktop{margin-bottom:0}.no-gap-left-desktop{margin-left:0}.offset-desktop{margin:-10px}.offset-top-desktop{margin-top:-10px}.offset-right-desktop{margin-right:-10px}.offset-bottom-desktop{margin-bottom:-10px}.offset-left-desktop{margin-left:-10px}.no-offset-top-desktop{margin-top:0}.no-offset-right-desktop{margin-right:0}.no-offset-bottom-desktop{margin-bottom:0}.no-offset-left-desktop{margin-left:0}.half-padded-desktop{padding:5px}.double-padded-desktop{padding:20px}.triple-padded-desktop{padding:30px}.half-pad-top-desktop{padding-top:5px}.double-pad-top-desktop{padding-top:20px}.triple-pad-top-desktop{padding-top:30px}.half-pad-right-desktop{padding-right:5px}.double-pad-right-desktop{padding-right:20px}.triple-pad-right-desktop{padding-right:30px}.half-pad-bottom-desktop{padding-bottom:5px}.double-pad-bottom-desktop{padding-bottom:20px}.triple-pad-bottom-desktop{padding-bottom:30px}.half-pad-left-desktop{padding-left:5px}.double-pad-left-desktop{padding-left:20px}.triple-pad-left-desktop{padding-left:30px}.half-gapped-desktop{margin:5px}.double-gapped-desktop{margin:20px}.triple-gapped-desktop{margin:30px}.half-gap-top-desktop{margin-top:5px}.double-gap-top-desktop{margin-top:20px}.triple-gap-top-desktop{margin-top:30px}.half-gap-right-desktop{margin-right:5px}.double-gap-right-desktop{margin-right:20px}.triple-gap-right-desktop{margin-right:30px}.half-gap-bottom-desktop{margin-bottom:5px}.double-gap-bottom-desktop{margin-bottom:20px}.triple-gap-bottom-desktop{margin-bottom:30px}.half-gap-left-desktop{margin-left:5px}.double-gap-left-desktop{margin-left:20px}.triple-gap-left-desktop{margin-left:30px}.half-offset-desktop{margin:-5px}.double-offset-desktop{margin:-20px}.triple-offset-desktop{margin:-30px}.half-offset-top-desktop{margin-top:-5px}.double-offset-top-desktop{margin-top:-20px}.triple-offset-top-desktop{margin-top:-30px}.half-offset-right-desktop{margin-right:-5px}.double-offset-right-desktop{margin-right:-20px}.triple-offset-right-desktop{margin-right:-30px}.half-offset-bottom-desktop{margin-bottom:-5px}.double-offset-bottom-desktop{margin-bottom:-20px}.triple-offset-bottom-desktop{margin-bottom:-30px}.half-offset-left-desktop{margin-left:-5px}.double-offset-left-desktop{margin-left:-20px}.triple-offset-left-desktop{margin-left:-30px}.fixed-desktop{position:fixed}.fixed-top-desktop{position:fixed;top:0}.fixed-right-desktop{position:fixed;right:0}.fixed-bottom-desktop{position:fixed;bottom:0}.fixed-left-desktop{position:fixed;left:0}.fixed-middle-desktop{position:fixed;top:50%}.fixed-center-desktop{position:fixed;left:50%}.absolute-desktop{position:absolute}.absolute-top-desktop{position:absolute;top:0}.absolute-right-desktop{position:absolute;right:0}.absolute-bottom-desktop{position:absolute;bottom:0}.absolute-left-desktop{position:absolute;left:0}.absolute-middle-desktop{position:absolute;top:50%}.absolute-center-desktop{position:absolute;left:50%}.static-desktop{position:static}.inline-desktop{display:inline}.block-desktop{display:block}.clear-desktop{clear:both}.clear-desktop:after{content:" ";display:block;height:0;font-size:0;clear:both;visibility:hidden}.center-desktop{display:block;margin:0 auto}.pull-right-desktop{float:right}.pull-left-desktop{float:left}.pull-none-desktop{float:none}.bordered-desktop{border:1px solid #ddd}.border-top-desktop{border-top:1px solid #ddd}.border-right-desktop{border-right:1px solid #ddd}.border-bottom-desktop{border-bottom:1px solid #ddd}.border-left-desktop{border-left:1px solid #ddd}.align-top-desktop{display:table-cell;vertical-align:top}.align-right-desktop{text-align:right}.align-bottom-desktop{display:table-cell;vertical-align:bottom}.align-left-desktop{text-align:left}.align-center-desktop{text-align:center}.justify-desktop{text-align:justify}.truncate-desktop{overflow:hidden;white-space:nowrap;text-overflow:ellipsis}.rotate-90-desktop{display:inline-block;width:auto;-webkit-transform:rotate(90deg);transform:rotate(90deg)}.rotate-90-ctr-desktop{display:inline-block;width:auto;-webkit-transform:rotate(-90deg);transform:rotate(-90deg)}.rotate-180-desktop{display:inline-block;width:auto;-webkit-transform:rotate(180deg);transform:rotate(180deg)}.rotate-180-ctr-desktop{display:inline-block;width:auto;-webkit-transform:rotate(-180deg);transform:rotate(-180deg)}.rotate-270-desktop{display:inline-block;width:auto;-webkit-transform:rotate(270deg);transform:rotate(270deg)}.rotate-270-ctr-desktop{display:inline-block;width:auto;-webkit-transform:rotate(-270deg);transform:rotate(-270deg)}.rotate-360-desktop{display:inline-block;width:auto;-webkit-transform:rotate(360deg);transform:rotate(360deg)}.rotate-360-ctr-desktop{display:inline-block;width:auto;-webkit-transform:rotate(-360deg);transform:rotate(-360deg)}.one-column-desktop,.one-columns-desktop{-webkit-column-count:1;-moz-column-count:1;column-count:1}.two-column-desktop,.two-columns-desktop{-webkit-column-count:2;-moz-column-count:2;column-count:2}.three-column-desktop,.three-columns-desktop{-webkit-column-count:3;-moz-column-count:3;column-count:3}.four-column-desktop,.four-columns-desktop{-webkit-column-count:4;-moz-column-count:4;column-count:4}.five-column-desktop,.five-columns-desktop{-webkit-column-count:5;-moz-column-count:5;column-count:5}}@media (min-width:768px) and (max-width:1024px){.desktop-only{display:none!important}.ipad-only{display:block!important}.small-tablet-only{display:none!important}.mobile-only{display:none!important}.hide-on-ipad{display:none!important}.no-border-ipad{border:0}.small-ipad{font-size:.8em}.medium-ipad{font-size:1em}.large-ipad{font-size:1.5em}.zero-ipad.padded,.zero-ipad.half-padded,.zero-ipad.double-padded,.zero-ipad.triple-padded{padding:0!important;margin:0!important}.zero-ipad.pad-top,.zero-ipad.half-pad-top,.zero-ipad.double-pad-top,.zero-ipad.triple-pad-top{padding:0!important;margin:0!important}.zero-ipad.pad-right,.zero-ipad.half-pad-right,.zero-ipad.double-pad-right,.zero-ipad.triple-pad-right{padding:0!important;margin:0!important}.zero-ipad.pad-bottom,.zero-ipad.half-pad-bottom,.zero-ipad.double-pad-bottom,.zero-ipad.triple-pad-bottom{padding:0!important;margin:0!important}.zero-ipad.pad-left,.zero-ipad.half-pad-left,.zero-ipad.double-pad-left,.zero-ipad.triple-pad-left{padding:0!important;margin:0!important}.zero-ipad.gapped,.zero-ipad.half-gapped,.zero-ipad.double-gapped,.zero-ipad.triple-gapped{padding:0!important;margin:0!important}.zero-ipad.gap-top,.zero-ipad.half-gap-top,.zero-ipad.double-gap-top,.zero-ipad.triple-gap-top{padding:0!important;margin:0!important}.zero-ipad.gap-right,.zero-ipad.half-gap-right,.zero-ipad.double-gap-right,.zero-ipad.triple-gap-right{padding:0!important;margin:0!important}.zero-ipad.gap-bottom,.zero-ipad.half-gap-bottom,.zero-ipad.double-gap-bottom,.zero-ipad.triple-gap-bottom{padding:0!important;margin:0!important}.zero-ipad.gap-left,.zero-ipad.half-gap-left,.zero-ipad.double-gap-left,.zero-ipad.triple-gap-left{padding:0!important;margin:0!important}.padded-ipad{padding:10px}.pad-top-ipad{padding-top:10px}.pad-right-ipad{padding-right:10px}.pad-bottom-ipad{padding-bottom:10px}.pad-left-ipad{padding-left:10px}.no-padding-ipad.padded,.no-padding-ipad.half-padded,.no-padding-ipad.double-padded,.no-padding-ipad.triple-padded,.no-padded-ipad.padded,.no-padded-ipad.half-padded,.no-padded-ipad.double-padded,.no-padded-ipad.triple-padded,.no-pad-ipad.padded,.no-pad-ipad.half-padded,.no-pad-ipad.double-padded,.no-pad-ipad.triple-padded{padding:0!important}.no-padding-ipad.pad-top,.no-padding-ipad.half-pad-top,.no-padding-ipad.double-pad-top,.no-padding-ipad.triple-pad-top,.no-padded-ipad.pad-top,.no-padded-ipad.half-pad-top,.no-padded-ipad.double-pad-top,.no-padded-ipad.triple-pad-top,.no-pad-ipad.pad-top,.no-pad-ipad.half-pad-top,.no-pad-ipad.double-pad-top,.no-pad-ipad.triple-pad-top{padding:0!important}.no-padding-ipad.pad-right,.no-padding-ipad.half-pad-right,.no-padding-ipad.double-pad-right,.no-padding-ipad.triple-pad-right,.no-padded-ipad.pad-right,.no-padded-ipad.half-pad-right,.no-padded-ipad.double-pad-right,.no-padded-ipad.triple-pad-right,.no-pad-ipad.pad-right,.no-pad-ipad.half-pad-right,.no-pad-ipad.double-pad-right,.no-pad-ipad.triple-pad-right{padding:0!important}.no-padding-ipad.pad-bottom,.no-padding-ipad.half-pad-bottom,.no-padding-ipad.double-pad-bottom,.no-padding-ipad.triple-pad-bottom,.no-padded-ipad.pad-bottom,.no-padded-ipad.half-pad-bottom,.no-padded-ipad.double-pad-bottom,.no-padded-ipad.triple-pad-bottom,.no-pad-ipad.pad-bottom,.no-pad-ipad.half-pad-bottom,.no-pad-ipad.double-pad-bottom,.no-pad-ipad.triple-pad-bottom{padding:0!important}.no-padding-ipad.pad-left,.no-padding-ipad.half-pad-left,.no-padding-ipad.double-pad-left,.no-padding-ipad.triple-pad-left,.no-padded-ipad.pad-left,.no-padded-ipad.half-pad-left,.no-padded-ipad.double-pad-left,.no-padded-ipad.triple-pad-left,.no-pad-ipad.pad-left,.no-pad-ipad.half-pad-left,.no-pad-ipad.double-pad-left,.no-pad-ipad.triple-pad-left{padding:0!important}.no-pad-top-ipad{padding-top:0}.no-pad-right-ipad{padding-right:0}.no-pad-bottom-ipad{padding-bottom:0}.no-pad-left-ipad{padding-left:0}.gapped-ipad{margin:10px}.gap-top-ipad{margin-top:10px}.gap-right-ipad{margin-right:10px}.gap-bottom-ipad{margin-bottom:10px}.gap-left-ipad{margin-left:10px}.no-margin-ipad.gapped,.no-margin-ipad.half-gapped,.no-margin-ipad.double-gapped,.no-margin-ipad.triple-gapped,.no-gapped-ipad.gapped,.no-gapped-ipad.half-gapped,.no-gapped-ipad.double-gapped,.no-gapped-ipad.triple-gapped,.no-gap-ipad.gapped,.no-gap-ipad.half-gapped,.no-gap-ipad.double-gapped,.no-gap-ipad.triple-gapped{margin:0!important}.no-margin-ipad.gap-top,.no-margin-ipad.half-gap-top,.no-margin-ipad.double-gap-top,.no-margin-ipad.triple-gap-top,.no-gapped-ipad.gap-top,.no-gapped-ipad.half-gap-top,.no-gapped-ipad.double-gap-top,.no-gapped-ipad.triple-gap-top,.no-gap-ipad.gap-top,.no-gap-ipad.half-gap-top,.no-gap-ipad.double-gap-top,.no-gap-ipad.triple-gap-top{margin:0!important}.no-margin-ipad.gap-right,.no-margin-ipad.half-gap-right,.no-margin-ipad.double-gap-right,.no-margin-ipad.triple-gap-right,.no-gapped-ipad.gap-right,.no-gapped-ipad.half-gap-right,.no-gapped-ipad.double-gap-right,.no-gapped-ipad.triple-gap-right,.no-gap-ipad.gap-right,.no-gap-ipad.half-gap-right,.no-gap-ipad.double-gap-right,.no-gap-ipad.triple-gap-right{margin:0!important}.no-margin-ipad.gap-bottom,.no-margin-ipad.half-gap-bottom,.no-margin-ipad.double-gap-bottom,.no-margin-ipad.triple-gap-bottom,.no-gapped-ipad.gap-bottom,.no-gapped-ipad.half-gap-bottom,.no-gapped-ipad.double-gap-bottom,.no-gapped-ipad.triple-gap-bottom,.no-gap-ipad.gap-bottom,.no-gap-ipad.half-gap-bottom,.no-gap-ipad.double-gap-bottom,.no-gap-ipad.triple-gap-bottom{margin:0!important}.no-margin-ipad.gap-left,.no-margin-ipad.half-gap-left,.no-margin-ipad.double-gap-left,.no-margin-ipad.triple-gap-left,.no-gapped-ipad.gap-left,.no-gapped-ipad.half-gap-left,.no-gapped-ipad.double-gap-left,.no-gapped-ipad.triple-gap-left,.no-gap-ipad.gap-left,.no-gap-ipad.half-gap-left,.no-gap-ipad.double-gap-left,.no-gap-ipad.triple-gap-left{margin:0!important}.no-gap-top-ipad{margin-top:0}.no-gap-right-ipad{margin-right:0}.no-gap-bottom-ipad{margin-bottom:0}.no-gap-left-ipad{margin-left:0}.offset-ipad{margin:-10px}.offset-top-ipad{margin-top:-10px}.offset-right-ipad{margin-right:-10px}.offset-bottom-ipad{margin-bottom:-10px}.offset-left-ipad{margin-left:-10px}.no-offset-top-ipad{margin-top:0}.no-offset-right-ipad{margin-right:0}.no-offset-bottom-ipad{margin-bottom:0}.no-offset-left-ipad{margin-left:0}.half-padded-ipad{padding:5px}.double-padded-ipad{padding:20px}.triple-padded-ipad{padding:30px}.half-pad-top-ipad{padding-top:5px}.double-pad-top-ipad{padding-top:20px}.triple-pad-top-ipad{padding-top:30px}.half-pad-right-ipad{padding-right:5px}.double-pad-right-ipad{padding-right:20px}.triple-pad-right-ipad{padding-right:30px}.half-pad-bottom-ipad{padding-bottom:5px}.double-pad-bottom-ipad{padding-bottom:20px}.triple-pad-bottom-ipad{padding-bottom:30px}.half-pad-left-ipad{padding-left:5px}.double-pad-left-ipad{padding-left:20px}.triple-pad-left-ipad{padding-left:30px}.half-gapped-ipad{margin:5px}.double-gapped-ipad{margin:20px}.triple-gapped-ipad{margin:30px}.half-gap-top-ipad{margin-top:5px}.double-gap-top-ipad{margin-top:20px}.triple-gap-top-ipad{margin-top:30px}.half-gap-right-ipad{margin-right:5px}.double-gap-right-ipad{margin-right:20px}.triple-gap-right-ipad{margin-right:30px}.half-gap-bottom-ipad{margin-bottom:5px}.double-gap-bottom-ipad{margin-bottom:20px}.triple-gap-bottom-ipad{margin-bottom:30px}.half-gap-left-ipad{margin-left:5px}.double-gap-left-ipad{margin-left:20px}.triple-gap-left-ipad{margin-left:30px}.half-offset-ipad{margin:-5px}.double-offset-ipad{margin:-20px}.triple-offset-ipad{margin:-30px}.half-offset-top-ipad{margin-top:-5px}.double-offset-top-ipad{margin-top:-20px}.triple-offset-top-ipad{margin-top:-30px}.half-offset-right-ipad{margin-right:-5px}.double-offset-right-ipad{margin-right:-20px}.triple-offset-right-ipad{margin-right:-30px}.half-offset-bottom-ipad{margin-bottom:-5px}.double-offset-bottom-ipad{margin-bottom:-20px}.triple-offset-bottom-ipad{margin-bottom:-30px}.half-offset-left-ipad{margin-left:-5px}.double-offset-left-ipad{margin-left:-20px}.triple-offset-left-ipad{margin-left:-30px}.fixed-ipad{position:fixed}.fixed-top-ipad{position:fixed;top:0}.fixed-right-ipad{position:fixed;right:0}.fixed-bottom-ipad{position:fixed;bottom:0}.fixed-left-ipad{position:fixed;left:0}.fixed-middle-ipad{position:fixed;top:50%}.fixed-center-ipad{position:fixed;left:50%}.absolute-ipad{position:absolute}.absolute-top-ipad{position:absolute;top:0}.absolute-right-ipad{position:absolute;right:0}.absolute-bottom-ipad{position:absolute;bottom:0}.absolute-left-ipad{position:absolute;left:0}.absolute-middle-ipad{position:absolute;top:50%}.absolute-center-ipad{position:absolute;left:50%}.static-ipad{position:static}.inline-ipad{display:inline}.block-ipad{display:block}.clear-ipad{clear:both}.clear-ipad:after{content:" ";display:block;height:0;font-size:0;clear:both;visibility:hidden}.center-ipad{display:block;margin:0 auto}.pull-right-ipad{float:right}.pull-left-ipad{float:left}.pull-none-ipad{float:none}.bordered-ipad{border:1px solid #ddd}.border-top-ipad{border-top:1px solid #ddd}.border-right-ipad{border-right:1px solid #ddd}.border-bottom-ipad{border-bottom:1px solid #ddd}.border-left-ipad{border-left:1px solid #ddd}.align-top-ipad{display:table-cell;vertical-align:top}.align-right-ipad{text-align:right}.align-bottom-ipad{display:table-cell;vertical-align:bottom}.align-left-ipad{text-align:left}.align-center-ipad{text-align:center}.justify-ipad{text-align:justify}.truncate-ipad{overflow:hidden;white-space:nowrap;text-overflow:ellipsis}.rotate-90-ipad{display:inline-block;width:auto;-webkit-transform:rotate(90deg);transform:rotate(90deg)}.rotate-90-ctr-ipad{display:inline-block;width:auto;-webkit-transform:rotate(-90deg);transform:rotate(-90deg)}.rotate-180-ipad{display:inline-block;width:auto;-webkit-transform:rotate(180deg);transform:rotate(180deg)}.rotate-180-ctr-ipad{display:inline-block;width:auto;-webkit-transform:rotate(-180deg);transform:rotate(-180deg)}.rotate-270-ipad{display:inline-block;width:auto;-webkit-transform:rotate(270deg);transform:rotate(270deg)}.rotate-270-ctr-ipad{display:inline-block;width:auto;-webkit-transform:rotate(-270deg);transform:rotate(-270deg)}.rotate-360-ipad{display:inline-block;width:auto;-webkit-transform:rotate(360deg);transform:rotate(360deg)}.rotate-360-ipad{display:inline-block;width:auto;-webkit-transform:rotate(-360deg);transform:rotate(-360deg)}.one-column-ipad,.one-columns-ipad{-webkit-column-count:1;-moz-column-count:1;column-count:1}.two-column-ipad,.two-columns-ipad{-webkit-column-count:2;-moz-column-count:2;column-count:2}.three-column-ipad,.three-columns-ipad{-webkit-column-count:3;-moz-column-count:3;column-count:3}.four-column-ipad,.four-columns-ipad{-webkit-column-count:4;-moz-column-count:4;column-count:4}.five-column-ipad,.five-columns-ipad{-webkit-column-count:5;-moz-column-count:5;column-count:5}.nav:not(.nocollapse):not(.small-tablet):not(.mobile):before{float:left;display:inline-block;content:attr(title);padding:.25em .5em;font-weight:700}.nav:not(.nocollapse):not(.small-tablet):not(.mobile) button.hamburger{display:inline-block}.nav:not(.nocollapse):not(.small-tablet):not(.mobile) button.hamburger:before{font-family:FontAwesome;font-weight:400;font-style:normal;text-decoration:inherit;-webkit-font-smoothing:antialiased;*margin-right:.3em;content:"\f03a"}.nav:not(.nocollapse):not(.small-tablet):not(.mobile) button.hamburger[aria-expanded=true]:before{content:"\f151"}.nav:not(.nocollapse):not(.small-tablet):not(.mobile)>ul{display:none}.nav:not(.nocollapse):not(.small-tablet):not(.mobile)>ul>li>a,.nav:not(.nocollapse):not(.small-tablet):not(.mobile)>ul>li>button{border-top-width:1px}.nav:not(.nocollapse):not(.small-tablet):not(.mobile)>ul[aria-expanded=true]{display:block}.nav:not(.nocollapse):not(.small-tablet):not(.mobile) li{float:none;display:block}.nav:not(.nocollapse):not(.small-tablet):not(.mobile) li ul{position:static}.nav:not(.nocollapse):not(.small-tablet):not(.mobile) li li a,.nav:not(.nocollapse):not(.small-tablet):not(.mobile) li li button{text-indent:10px}.nav:not(.nocollapse):not(.small-tablet):not(.mobile) li li li a,.nav:not(.nocollapse):not(.small-tablet):not(.mobile) li li li button{text-indent:20px}.nav:not(.nocollapse):not(.small-tablet):not(.mobile) li[role=menu] ul{border:0}.nav:not(.nocollapse):not(.small-tablet):not(.mobile) li[role=menu] ul>li:first-child>a,.nav:not(.nocollapse):not(.small-tablet):not(.mobile) li[role=menu] ul>li:first-child>button{border-top-width:1px}.nav:not(.nocollapse):not(.small-tablet):not(.mobile) li[role=menu]>a:after,.nav:not(.nocollapse):not(.small-tablet):not(.mobile) li[role=menu]>button:after{float:none;content:"\f0d7"}.nav:not(.nocollapse):not(.small-tablet):not(.mobile) li[role=menu][aria-pressed=true]>a:after,.nav:not(.nocollapse):not(.small-tablet):not(.mobile) li[role=menu][aria-pressed=true]>button:after{content:"\f0d8"}.nav:not(.nocollapse):not(.small-tablet):not(.mobile) a,.nav:not(.nocollapse):not(.small-tablet):not(.mobile) button:not(.hamburger){display:block;width:100%;text-align:left}.nav:not(.nocollapse):not(.small-tablet):not(.mobile).green li ul a,.nav:not(.nocollapse):not(.small-tablet):not(.mobile).green li[role=menu] ul a,.nav:not(.nocollapse):not(.small-tablet):not(.mobile).green a,.nav:not(.nocollapse):not(.small-tablet):not(.mobile).green li ul button,.nav:not(.nocollapse):not(.small-tablet):not(.mobile).green li[role=menu] ul button,.nav:not(.nocollapse):not(.small-tablet):not(.mobile).green button{-o-border-radius:0;border-radius:0}.nav:not(.nocollapse):not(.small-tablet):not(.mobile).blue li ul a,.nav:not(.nocollapse):not(.small-tablet):not(.mobile).blue li[role=menu] ul a,.nav:not(.nocollapse):not(.small-tablet):not(.mobile).blue a,.nav:not(.nocollapse):not(.small-tablet):not(.mobile).blue li ul button,.nav:not(.nocollapse):not(.small-tablet):not(.mobile).blue li[role=menu] ul button,.nav:not(.nocollapse):not(.small-tablet):not(.mobile).blue button{-o-border-radius:0;border-radius:0}.nav:not(.nocollapse):not(.small-tablet):not(.mobile).purple li ul a,.nav:not(.nocollapse):not(.small-tablet):not(.mobile).purple li[role=menu] ul a,.nav:not(.nocollapse):not(.small-tablet):not(.mobile).purple a,.nav:not(.nocollapse):not(.small-tablet):not(.mobile).purple li ul button,.nav:not(.nocollapse):not(.small-tablet):not(.mobile).purple li[role=menu] ul button,.nav:not(.nocollapse):not(.small-tablet):not(.mobile).purple button{-o-border-radius:0;border-radius:0}.nav:not(.nocollapse):not(.small-tablet):not(.mobile).yellow li ul a,.nav:not(.nocollapse):not(.small-tablet):not(.mobile).yellow li[role=menu] ul a,.nav:not(.nocollapse):not(.small-tablet):not(.mobile).yellow a,.nav:not(.nocollapse):not(.small-tablet):not(.mobile).yellow li ul button,.nav:not(.nocollapse):not(.small-tablet):not(.mobile).yellow li[role=menu] ul button,.nav:not(.nocollapse):not(.small-tablet):not(.mobile).yellow button{-o-border-radius:0;border-radius:0}.nav:not(.nocollapse):not(.small-tablet):not(.mobile).orange li ul a,.nav:not(.nocollapse):not(.small-tablet):not(.mobile).orange li[role=menu] ul a,.nav:not(.nocollapse):not(.small-tablet):not(.mobile).orange a,.nav:not(.nocollapse):not(.small-tablet):not(.mobile).orange li ul button,.nav:not(.nocollapse):not(.small-tablet):not(.mobile).orange li[role=menu] ul button,.nav:not(.nocollapse):not(.small-tablet):not(.mobile).orange button{-o-border-radius:0;border-radius:0}.nav:not(.nocollapse):not(.small-tablet):not(.mobile).red li ul a,.nav:not(.nocollapse):not(.small-tablet):not(.mobile).red li[role=menu] ul a,.nav:not(.nocollapse):not(.small-tablet):not(.mobile).red a,.nav:not(.nocollapse):not(.small-tablet):not(.mobile).red li ul button,.nav:not(.nocollapse):not(.small-tablet):not(.mobile).red li[role=menu] ul button,.nav:not(.nocollapse):not(.small-tablet):not(.mobile).red button{-o-border-radius:0;border-radius:0}.nav:not(.nocollapse):not(.small-tablet):not(.mobile).pink li ul a,.nav:not(.nocollapse):not(.small-tablet):not(.mobile).pink li[role=menu] ul a,.nav:not(.nocollapse):not(.small-tablet):not(.mobile).pink a,.nav:not(.nocollapse):not(.small-tablet):not(.mobile).pink li ul button,.nav:not(.nocollapse):not(.small-tablet):not(.mobile).pink li[role=menu] ul button,.nav:not(.nocollapse):not(.small-tablet):not(.mobile).pink button{-o-border-radius:0;border-radius:0}.nav:not(.nocollapse):not(.small-tablet):not(.mobile).turquoise li ul a,.nav:not(.nocollapse):not(.small-tablet):not(.mobile).turquoise li[role=menu] ul a,.nav:not(.nocollapse):not(.small-tablet):not(.mobile).turquoise a,.nav:not(.nocollapse):not(.small-tablet):not(.mobile).turquoise li ul button,.nav:not(.nocollapse):not(.small-tablet):not(.mobile).turquoise li[role=menu] ul button,.nav:not(.nocollapse):not(.small-tablet):not(.mobile).turquoise button{-o-border-radius:0;border-radius:0}.nav:not(.nocollapse):not(.small-tablet):not(.mobile).asphalt li ul a,.nav:not(.nocollapse):not(.small-tablet):not(.mobile).asphalt li[role=menu] ul a,.nav:not(.nocollapse):not(.small-tablet):not(.mobile).asphalt a,.nav:not(.nocollapse):not(.small-tablet):not(.mobile).asphalt li ul button,.nav:not(.nocollapse):not(.small-tablet):not(.mobile).asphalt li[role=menu] ul button,.nav:not(.nocollapse):not(.small-tablet):not(.mobile).asphalt button{-o-border-radius:0;border-radius:0}.nav:not(.nocollapse):not(.small-tablet):not(.mobile).charcoal li ul a,.nav:not(.nocollapse):not(.small-tablet):not(.mobile).charcoal li[role=menu] ul a,.nav:not(.nocollapse):not(.small-tablet):not(.mobile).charcoal a,.nav:not(.nocollapse):not(.small-tablet):not(.mobile).charcoal li ul button,.nav:not(.nocollapse):not(.small-tablet):not(.mobile).charcoal li[role=menu] ul button,.nav:not(.nocollapse):not(.small-tablet):not(.mobile).charcoal button{-o-border-radius:0;border-radius:0}.nav:not(.nocollapse):not(.small-tablet):not(.mobile).white li ul a,.nav:not(.nocollapse):not(.small-tablet):not(.mobile).white li[role=menu] ul a,.nav:not(.nocollapse):not(.small-tablet):not(.mobile).white a,.nav:not(.nocollapse):not(.small-tablet):not(.mobile).white li ul button,.nav:not(.nocollapse):not(.small-tablet):not(.mobile).white li[role=menu] ul button,.nav:not(.nocollapse):not(.small-tablet):not(.mobile).white button{-o-border-radius:0;border-radius:0}.nav:not(.nocollapse):not(.small-tablet):not(.mobile).black li ul a,.nav:not(.nocollapse):not(.small-tablet):not(.mobile).black li[role=menu] ul a,.nav:not(.nocollapse):not(.small-tablet):not(.mobile).black a,.nav:not(.nocollapse):not(.small-tablet):not(.mobile).black li ul button,.nav:not(.nocollapse):not(.small-tablet):not(.mobile).black li[role=menu] ul button,.nav:not(.nocollapse):not(.small-tablet):not(.mobile).black button{-o-border-radius:0;border-radius:0}}@media (max-width:767px){.half:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.halves:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.third:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.thirds:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.fourth:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.fourths:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.fifth:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.fifths:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.sixth:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.sixths:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.seventh:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.sevenths:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.eighth:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.eighths:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.ninth:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.ninths:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.tenth:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.tenths:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.eleventh:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.elevenths:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.twelfth:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.twelfths:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet){width:100%!important;float:none;clear:both}.half.right-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.halves.right-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.third.right-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.thirds.right-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.third.right-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.thirds.right-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.fourth.right-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.fourths.right-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.fourth.right-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.fourths.right-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.fourth.right-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.fourths.right-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.fifth.right-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.fifths.right-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.fifth.right-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.fifths.right-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.fifth.right-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.fifths.right-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.fifth.right-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.fifths.right-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.sixth.right-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.sixths.right-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.sixth.right-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.sixths.right-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.sixth.right-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.sixths.right-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.sixth.right-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.sixths.right-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.sixth.right-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.sixths.right-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.seventh.right-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.sevenths.right-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.seventh.right-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.sevenths.right-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.seventh.right-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.sevenths.right-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.seventh.right-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.sevenths.right-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.seventh.right-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.sevenths.right-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.seventh.right-six:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.sevenths.right-six:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.eighth.right-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.eighths.right-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.eighth.right-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.eighths.right-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.eighth.right-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.eighths.right-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.eighth.right-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.eighths.right-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.eighth.right-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.eighths.right-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.eighth.right-six:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.eighths.right-six:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.eighth.right-seven:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.eighths.right-seven:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.ninth.right-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.ninths.right-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.ninth.right-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.ninths.right-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.ninth.right-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.ninths.right-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.ninth.right-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.ninths.right-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.ninth.right-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.ninths.right-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.ninth.right-six:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.ninths.right-six:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.ninth.right-seven:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.ninths.right-seven:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.ninth.right-eight:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.ninths.right-eight:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.tenth.right-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.tenths.right-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.tenth.right-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.tenths.right-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.tenth.right-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.tenths.right-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.tenth.right-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.tenths.right-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.tenth.right-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.tenths.right-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.tenth.right-six:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.tenths.right-six:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.tenth.right-seven:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.tenths.right-seven:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.tenth.right-eight:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.tenths.right-eight:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.tenth.right-nine:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.tenths.right-nine:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.eleventh.right-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.elevenths.right-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.eleventh.right-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.elevenths.right-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.eleventh.right-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.elevenths.right-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.eleventh.right-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.elevenths.right-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.eleventh.right-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.elevenths.right-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.eleventh.right-six:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.elevenths.right-six:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.eleventh.right-seven:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.elevenths.right-seven:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.eleventh.right-eight:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.elevenths.right-eight:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.eleventh.right-nine:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.elevenths.right-nine:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.eleventh.right-ten:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.elevenths.right-ten:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.twelfth.right-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.twelfths.right-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.twelfth.right-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.twelfths.right-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.twelfth.right-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.twelfths.right-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.twelfth.right-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.twelfths.right-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.twelfth.right-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.twelfths.right-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.twelfth.right-six:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.twelfths.right-six:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.twelfth.right-seven:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.twelfths.right-seven:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.twelfth.right-eight:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.twelfths.right-eight:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.twelfth.right-nine:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.twelfths.right-nine:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.twelfth.right-ten:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.twelfths.right-ten:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.twelfth.right-eleven:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.twelfths.right-eleven:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet){left:0}.half.left-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.halves.left-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.third.left-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.thirds.left-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.third.left-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.thirds.left-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.fourth.left-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.fourths.left-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.fourth.left-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.fourths.left-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.fourth.left-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.fourths.left-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.fifth.left-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.fifths.left-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.fifth.left-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.fifths.left-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.fifth.left-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.fifths.left-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.fifth.left-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.fifths.left-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.sixth.left-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.sixths.left-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.sixth.left-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.sixths.left-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.sixth.left-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.sixths.left-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.sixth.left-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.sixths.left-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.sixth.left-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.sixths.left-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.seventh.left-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.sevenths.left-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.seventh.left-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.sevenths.left-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.seventh.left-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.sevenths.left-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.seventh.left-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.sevenths.left-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.seventh.left-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.sevenths.left-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.seventh.left-six:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.sevenths.left-six:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.eighth.left-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.eighths.left-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.eighth.left-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.eighths.left-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.eighth.left-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.eighths.left-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.eighth.left-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.eighths.left-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.eighth.left-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.eighths.left-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.eighth.left-six:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.eighths.left-six:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.eighth.left-seven:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.eighths.left-seven:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.ninth.left-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.ninths.left-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.ninth.left-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.ninths.left-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.ninth.left-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.ninths.left-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.ninth.left-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.ninths.left-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.ninth.left-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.ninths.left-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.ninth.left-six:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.ninths.left-six:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.ninth.left-seven:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.ninths.left-seven:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.ninth.left-eight:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.ninths.left-eight:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.tenth.left-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.tenths.left-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.tenth.left-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.tenths.left-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.tenth.left-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.tenths.left-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.tenth.left-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.tenths.left-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.tenth.left-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.tenths.left-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.tenth.left-six:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.tenths.left-six:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.tenth.left-seven:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.tenths.left-seven:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.tenth.left-eight:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.tenths.left-eight:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.tenth.left-nine:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.tenths.left-nine:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.eleventh.left-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.elevenths.left-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.eleventh.left-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.elevenths.left-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.eleventh.left-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.elevenths.left-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.eleventh.left-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.elevenths.left-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.eleventh.left-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.elevenths.left-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.eleventh.left-six:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.elevenths.left-six:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.eleventh.left-seven:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.elevenths.left-seven:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.eleventh.left-eight:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.elevenths.left-eight:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.eleventh.left-nine:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.elevenths.left-nine:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.eleventh.left-ten:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.elevenths.left-ten:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.twelfth.left-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.twelfths.left-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.twelfth.left-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.twelfths.left-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.twelfth.left-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.twelfths.left-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.twelfth.left-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.twelfths.left-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.twelfth.left-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.twelfths.left-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.twelfth.left-six:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.twelfths.left-six:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.twelfth.left-seven:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.twelfths.left-seven:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.twelfth.left-eight:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.twelfths.left-eight:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.twelfth.left-nine:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.twelfths.left-nine:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.twelfth.left-ten:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.twelfths.left-ten:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.twelfth.left-eleven:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.twelfths.left-eleven:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet){right:0}.half.skip-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.halves.skip-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.third.skip-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.thirds.skip-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.third.skip-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.thirds.skip-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.fourth.skip-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.fourths.skip-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.fourth.skip-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.fourths.skip-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.fourth.skip-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.fourths.skip-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.fifth.skip-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.fifths.skip-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.fifth.skip-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.fifths.skip-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.fifth.skip-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.fifths.skip-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.fifth.skip-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.fifths.skip-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.sixth.skip-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.sixths.skip-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.sixth.skip-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.sixths.skip-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.sixth.skip-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.sixths.skip-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.sixth.skip-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.sixths.skip-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.sixth.skip-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.sixths.skip-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.seventh.skip-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.sevenths.skip-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.seventh.skip-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.sevenths.skip-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.seventh.skip-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.sevenths.skip-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.seventh.skip-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.sevenths.skip-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.seventh.skip-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.sevenths.skip-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.seventh.skip-six:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.sevenths.skip-six:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.eighth.skip-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.eighths.skip-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.eighth.skip-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.eighths.skip-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.eighth.skip-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.eighths.skip-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.eighth.skip-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.eighths.skip-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.eighth.skip-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.eighths.skip-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.eighth.skip-six:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.eighths.skip-six:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.eighth.skip-seven:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.eighths.skip-seven:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.ninth.skip-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.ninths.skip-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.ninth.skip-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.ninths.skip-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.ninth.skip-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.ninths.skip-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.ninth.skip-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.ninths.skip-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.ninth.skip-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.ninths.skip-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.ninth.skip-six:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.ninths.skip-six:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.ninth.skip-seven:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.ninths.skip-seven:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.ninth.skip-eight:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.ninths.skip-eight:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.tenth.skip-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.tenths.skip-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.tenth.skip-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.tenths.skip-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.tenth.skip-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.tenths.skip-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.tenth.skip-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.tenths.skip-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.tenth.skip-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.tenths.skip-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.tenth.skip-six:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.tenths.skip-six:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.tenth.skip-seven:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.tenths.skip-seven:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.tenth.skip-eight:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.tenths.skip-eight:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.tenth.skip-nine:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.tenths.skip-nine:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.eleventh.skip-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.elevenths.skip-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.eleventh.skip-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.elevenths.skip-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.eleventh.skip-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.elevenths.skip-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.eleventh.skip-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.elevenths.skip-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.eleventh.skip-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.elevenths.skip-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.eleventh.skip-six:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.elevenths.skip-six:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.eleventh.skip-seven:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.elevenths.skip-seven:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.eleventh.skip-eight:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.elevenths.skip-eight:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.eleventh.skip-nine:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.elevenths.skip-nine:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.eleventh.skip-ten:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.elevenths.skip-ten:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.twelfth.skip-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.twelfths.skip-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.twelfth.skip-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.twelfths.skip-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.twelfth.skip-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.twelfths.skip-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.twelfth.skip-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.twelfths.skip-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.twelfth.skip-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.twelfths.skip-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.twelfth.skip-six:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.twelfths.skip-six:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.twelfth.skip-seven:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.twelfths.skip-seven:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.twelfth.skip-eight:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.twelfths.skip-eight:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.twelfth.skip-nine:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.twelfths.skip-nine:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.twelfth.skip-ten:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.twelfths.skip-ten:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.twelfth.skip-eleven:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet),.twelfths.skip-eleven:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile):not(.small-tablet):not(.one-up-small-tablet):not(.two-up-small-tablet):not(.three-up-small-tablet):not(.four-up-small-tablet):not(.five-up-small-tablet){margin-left:0}.desktop-only{display:none!important}.ipad-only{display:none!important}.small-tablet-only{display:block!important}.mobile-only{display:none!important}.hide-on-small-tablet{display:none!important}.no-border-small-tablet{border:0}.small-small-tablet{font-size:.8em}.medium-small-tablet{font-size:1em}.large-small-tablet{font-size:1.5em}.zero-small-tablet.padded,.zero-small-tablet.half-padded,.zero-small-tablet.double-padded,.zero-small-tablet.triple-padded{padding:0!important;margin:0!important}.zero-small-tablet.pad-top,.zero-small-tablet.half-pad-top,.zero-small-tablet.double-pad-top,.zero-small-tablet.triple-pad-top{padding:0!important;margin:0!important}.zero-small-tablet.pad-right,.zero-small-tablet.half-pad-right,.zero-small-tablet.double-pad-right,.zero-small-tablet.triple-pad-right{padding:0!important;margin:0!important}.zero-small-tablet.pad-bottom,.zero-small-tablet.half-pad-bottom,.zero-small-tablet.double-pad-bottom,.zero-small-tablet.triple-pad-bottom{padding:0!important;margin:0!important}.zero-small-tablet.pad-left,.zero-small-tablet.half-pad-left,.zero-small-tablet.double-pad-left,.zero-small-tablet.triple-pad-left{padding:0!important;margin:0!important}.zero-small-tablet.gapped,.zero-small-tablet.half-gapped,.zero-small-tablet.double-gapped,.zero-small-tablet.triple-gapped{padding:0!important;margin:0!important}.zero-small-tablet.gap-top,.zero-small-tablet.half-gap-top,.zero-small-tablet.double-gap-top,.zero-small-tablet.triple-gap-top{padding:0!important;margin:0!important}.zero-small-tablet.gap-right,.zero-small-tablet.half-gap-right,.zero-small-tablet.double-gap-right,.zero-small-tablet.triple-gap-right{padding:0!important;margin:0!important}.zero-small-tablet.gap-bottom,.zero-small-tablet.half-gap-bottom,.zero-small-tablet.double-gap-bottom,.zero-small-tablet.triple-gap-bottom{padding:0!important;margin:0!important}.zero-small-tablet.gap-left,.zero-small-tablet.half-gap-left,.zero-small-tablet.double-gap-left,.zero-small-tablet.triple-gap-left{padding:0!important;margin:0!important}.padded-small-tablet{padding:10px}.pad-top-small-tablet{padding-top:10px}.pad-right-small-tablet{padding-right:10px}.pad-bottom-small-tablet{padding-bottom:10px}.pad-left-small-tablet{padding-left:10px}.no-padding-small-tablet.padded,.no-padding-small-tablet.half-padded,.no-padding-small-tablet.double-padded,.no-padding-small-tablet.triple-padded,.no-padded-small-tablet.padded,.no-padded-small-tablet.half-padded,.no-padded-small-tablet.double-padded,.no-padded-small-tablet.triple-padded,.no-pad-small-tablet.padded,.no-pad-small-tablet.half-padded,.no-pad-small-tablet.double-padded,.no-pad-small-tablet.triple-padded{padding:0!important}.no-padding-small-tablet.pad-top,.no-padding-small-tablet.half-pad-top,.no-padding-small-tablet.double-pad-top,.no-padding-small-tablet.triple-pad-top,.no-padded-small-tablet.pad-top,.no-padded-small-tablet.half-pad-top,.no-padded-small-tablet.double-pad-top,.no-padded-small-tablet.triple-pad-top,.no-pad-small-tablet.pad-top,.no-pad-small-tablet.half-pad-top,.no-pad-small-tablet.double-pad-top,.no-pad-small-tablet.triple-pad-top{padding:0!important}.no-padding-small-tablet.pad-right,.no-padding-small-tablet.half-pad-right,.no-padding-small-tablet.double-pad-right,.no-padding-small-tablet.triple-pad-right,.no-padded-small-tablet.pad-right,.no-padded-small-tablet.half-pad-right,.no-padded-small-tablet.double-pad-right,.no-padded-small-tablet.triple-pad-right,.no-pad-small-tablet.pad-right,.no-pad-small-tablet.half-pad-right,.no-pad-small-tablet.double-pad-right,.no-pad-small-tablet.triple-pad-right{padding:0!important}.no-padding-small-tablet.pad-bottom,.no-padding-small-tablet.half-pad-bottom,.no-padding-small-tablet.double-pad-bottom,.no-padding-small-tablet.triple-pad-bottom,.no-padded-small-tablet.pad-bottom,.no-padded-small-tablet.half-pad-bottom,.no-padded-small-tablet.double-pad-bottom,.no-padded-small-tablet.triple-pad-bottom,.no-pad-small-tablet.pad-bottom,.no-pad-small-tablet.half-pad-bottom,.no-pad-small-tablet.double-pad-bottom,.no-pad-small-tablet.triple-pad-bottom{padding:0!important}.no-padding-small-tablet.pad-left,.no-padding-small-tablet.half-pad-left,.no-padding-small-tablet.double-pad-left,.no-padding-small-tablet.triple-pad-left,.no-padded-small-tablet.pad-left,.no-padded-small-tablet.half-pad-left,.no-padded-small-tablet.double-pad-left,.no-padded-small-tablet.triple-pad-left,.no-pad-small-tablet.pad-left,.no-pad-small-tablet.half-pad-left,.no-pad-small-tablet.double-pad-left,.no-pad-small-tablet.triple-pad-left{padding:0!important}.no-pad-top-small-tablet{padding-top:0}.no-pad-right-small-tablet{padding-right:0}.no-pad-bottom-small-tablet{padding-bottom:0}.no-pad-left-small-tablet{padding-left:0}.gapped-small-tablet{margin:10px}.gap-top-small-tablet{margin-top:10px}.gap-right-small-tablet{margin-right:10px}.gap-bottom-small-tablet{margin-bottom:10px}.gap-left-small-tablet{margin-left:10px}.no-margin-small-tablet.gapped,.no-margin-small-tablet.half-gapped,.no-margin-small-tablet.double-gapped,.no-margin-small-tablet.triple-gapped,.no-gapped-small-tablet.gapped,.no-gapped-small-tablet.half-gapped,.no-gapped-small-tablet.double-gapped,.no-gapped-small-tablet.triple-gapped,.no-gap-small-tablet.gapped,.no-gap-small-tablet.half-gapped,.no-gap-small-tablet.double-gapped,.no-gap-small-tablet.triple-gapped{margin:0!important}.no-margin-small-tablet.gap-top,.no-margin-small-tablet.half-gap-top,.no-margin-small-tablet.double-gap-top,.no-margin-small-tablet.triple-gap-top,.no-gapped-small-tablet.gap-top,.no-gapped-small-tablet.half-gap-top,.no-gapped-small-tablet.double-gap-top,.no-gapped-small-tablet.triple-gap-top,.no-gap-small-tablet.gap-top,.no-gap-small-tablet.half-gap-top,.no-gap-small-tablet.double-gap-top,.no-gap-small-tablet.triple-gap-top{margin:0!important}.no-margin-small-tablet.gap-right,.no-margin-small-tablet.half-gap-right,.no-margin-small-tablet.double-gap-right,.no-margin-small-tablet.triple-gap-right,.no-gapped-small-tablet.gap-right,.no-gapped-small-tablet.half-gap-right,.no-gapped-small-tablet.double-gap-right,.no-gapped-small-tablet.triple-gap-right,.no-gap-small-tablet.gap-right,.no-gap-small-tablet.half-gap-right,.no-gap-small-tablet.double-gap-right,.no-gap-small-tablet.triple-gap-right{margin:0!important}.no-margin-small-tablet.gap-bottom,.no-margin-small-tablet.half-gap-bottom,.no-margin-small-tablet.double-gap-bottom,.no-margin-small-tablet.triple-gap-bottom,.no-gapped-small-tablet.gap-bottom,.no-gapped-small-tablet.half-gap-bottom,.no-gapped-small-tablet.double-gap-bottom,.no-gapped-small-tablet.triple-gap-bottom,.no-gap-small-tablet.gap-bottom,.no-gap-small-tablet.half-gap-bottom,.no-gap-small-tablet.double-gap-bottom,.no-gap-small-tablet.triple-gap-bottom{margin:0!important}.no-margin-small-tablet.gap-left,.no-margin-small-tablet.half-gap-left,.no-margin-small-tablet.double-gap-left,.no-margin-small-tablet.triple-gap-left,.no-gapped-small-tablet.gap-left,.no-gapped-small-tablet.half-gap-left,.no-gapped-small-tablet.double-gap-left,.no-gapped-small-tablet.triple-gap-left,.no-gap-small-tablet.gap-left,.no-gap-small-tablet.half-gap-left,.no-gap-small-tablet.double-gap-left,.no-gap-small-tablet.triple-gap-left{margin:0!important}.no-gap-top-small-tablet{margin-top:0}.no-gap-right-small-tablet{margin-right:0}.no-gap-bottom-small-tablet{margin-bottom:0}.no-gap-left-small-tablet{margin-left:0}.offset-small-tablet{margin:-10px}.offset-top-small-tablet{margin-top:-10px}.offset-right-small-tablet{margin-right:-10px}.offset-bottom-small-tablet{margin-bottom:-10px}.offset-left-small-tablet{margin-left:-10px}.no-offset-top-small-tablet{margin-top:0}.no-offset-right-small-tablet{margin-right:0}.no-offset-bottom-small-tablet{margin-bottom:0}.no-offset-left-small-tablet{margin-left:0}.half-padded-small-tablet{padding:5px}.double-padded-small-tablet{padding:20px}.triple-padded-small-tablet{padding:30px}.half-pad-top-small-tablet{padding-top:5px}.double-pad-top-small-tablet{padding-top:20px}.triple-pad-top-small-tablet{padding-top:30px}.half-pad-right-small-tablet{padding-right:5px}.double-pad-right-small-tablet{padding-right:20px}.triple-pad-right-small-tablet{padding-right:30px}.half-pad-bottom-small-tablet{padding-bottom:5px}.double-pad-bottom-small-tablet{padding-bottom:20px}.triple-pad-bottom-small-tablet{padding-bottom:30px}.half-pad-left-small-tablet{padding-left:5px}.double-pad-left-small-tablet{padding-left:20px}.triple-pad-left-small-tablet{padding-left:30px}.half-gapped-small-tablet{margin:5px}.double-gapped-small-tablet{margin:20px}.triple-gapped-small-tablet{margin:30px}.half-gap-top-small-tablet{margin-top:5px}.double-gap-top-small-tablet{margin-top:20px}.triple-gap-top-small-tablet{margin-top:30px}.half-gap-right-small-tablet{margin-right:5px}.double-gap-right-small-tablet{margin-right:20px}.triple-gap-right-small-tablet{margin-right:30px}.half-gap-bottom-small-tablet{margin-bottom:5px}.double-gap-bottom-small-tablet{margin-bottom:20px}.triple-gap-bottom-small-tablet{margin-bottom:30px}.half-gap-left-small-tablet{margin-left:5px}.double-gap-left-small-tablet{margin-left:20px}.triple-gap-left-small-tablet{margin-left:30px}.half-offset-small-tablet{margin:-5px}.double-offset-small-tablet{margin:-20px}.triple-offset-small-tablet{margin:-30px}.half-offset-top-small-tablet{margin-top:-5px}.double-offset-top-small-tablet{margin-top:-20px}.triple-offset-top-small-tablet{margin-top:-30px}.half-offset-right-small-tablet{margin-right:-5px}.double-offset-right-small-tablet{margin-right:-20px}.triple-offset-right-small-tablet{margin-right:-30px}.half-offset-bottom-small-tablet{margin-bottom:-5px}.double-offset-bottom-small-tablet{margin-bottom:-20px}.triple-offset-bottom-small-tablet{margin-bottom:-30px}.half-offset-left-small-tablet{margin-left:-5px}.double-offset-left-small-tablet{margin-left:-20px}.triple-offset-left-small-tablet{margin-left:-30px}.fixed-small-tablet{position:fixed}.fixed-top-small-tablet{position:fixed;top:0}.fixed-right-small-tablet{position:fixed;right:0}.fixed-bottom-small-tablet{position:fixed;bottom:0}.fixed-left-small-tablet{position:fixed;left:0}.fixed-middle-small-tablet{position:fixed;top:50%}.fixed-center-small-tablet{position:fixed;left:50%}.absolute-small-tablet{position:absolute}.absolute-top-small-tablet{position:absolute;top:0}.absolute-right-small-tablet{position:absolute;right:0}.absolute-bottom-small-tablet{position:absolute;bottom:0}.absolute-left-small-tablet{position:absolute;left:0}.absolute-middle-small-tablet{position:absolute;top:50%}.absolute-center-small-tablet{position:absolute;left:50%}.static-small-tablet{position:static}.inline-small-tablet{display:inline}.block-small-tablet{display:block}.clear-small-tablet{clear:both}.clear-small-tablet:after{content:" ";display:block;height:0;font-size:0;clear:both;visibility:hidden}.center-small-tablet{display:block;margin:0 auto}.pull-right-small-tablet{float:right}.pull-left-small-tablet{float:left}.pull-none-small-tablet{float:none}.bordered-small-tablet{border:1px solid #ddd}.border-top-small-tablet{border-top:1px solid #ddd}.border-right-small-tablet{border-right:1px solid #ddd}.border-bottom-small-tablet{border-bottom:1px solid #ddd}.border-left-small-tablet{border-left:1px solid #ddd}.align-top-small-tablet{display:table-cell;vertical-align:top}.align-right-small-tablet{text-align:right}.align-bottom-small-tablet{display:table-cell;vertical-align:bottom}.align-left-small-tablet{text-align:left}.align-center-small-tablet{text-align:center}.justify-small-tablet{text-align:justify}.truncate-small-tablet{overflow:hidden;white-space:nowrap;text-overflow:ellipsis}.rotate-90-small-tablet{display:inline-block;width:auto;-webkit-transform:rotate(90deg);transform:rotate(90deg)}.rotate-90-ctr-small-tablet{display:inline-block;width:auto;-webkit-transform:rotate(-90deg);transform:rotate(-90deg)}.rotate-180-small-tablet{display:inline-block;width:auto;-webkit-transform:rotate(180deg);transform:rotate(180deg)}.rotate-180-ctr-small-tablet{display:inline-block;width:auto;-webkit-transform:rotate(-180deg);transform:rotate(-180deg)}.rotate-270-small-tablet{display:inline-block;width:auto;-webkit-transform:rotate(270deg);transform:rotate(270deg)}.rotate-270-ctr-small-tablet{display:inline-block;width:auto;-webkit-transform:rotate(-270deg);transform:rotate(-270deg)}.rotate-360-small-tablet{display:inline-block;width:auto;-webkit-transform:rotate(360deg);transform:rotate(360deg)}.rotate-360-ctr-small-tablet{display:inline-block;width:auto;-webkit-transform:rotate(-360deg);transform:rotate(-360deg)}.one-column-small-tablet,.one-columns-small-tablet{-webkit-column-count:1;-moz-column-count:1;column-count:1}.two-column-small-tablet,.two-columns-small-tablet{-webkit-column-count:2;-moz-column-count:2;column-count:2}.three-column-small-tablet,.three-columns-small-tablet{-webkit-column-count:3;-moz-column-count:3;column-count:3}.four-column-small-tablet,.four-columns-small-tablet{-webkit-column-count:4;-moz-column-count:4;column-count:4}.five-column-small-tablet,.five-columns-small-tablet{-webkit-column-count:5;-moz-column-count:5;column-count:5}.nav:not(.nocollapse):not(.mobile):before{float:left;display:inline-block;content:attr(title);padding:.25em .5em;font-weight:700}.nav:not(.nocollapse):not(.mobile) button.hamburger{display:inline-block}.nav:not(.nocollapse):not(.mobile) button.hamburger:before{font-family:FontAwesome;font-weight:400;font-style:normal;text-decoration:inherit;-webkit-font-smoothing:antialiased;*margin-right:.3em;content:"\f03a"}.nav:not(.nocollapse):not(.mobile) button.hamburger[aria-expanded=true]:before{content:"\f151"}.nav:not(.nocollapse):not(.mobile)>ul{display:none}.nav:not(.nocollapse):not(.mobile)>ul>li>a,.nav:not(.nocollapse):not(.mobile)>ul>li>button{border-top-width:1px}.nav:not(.nocollapse):not(.mobile)>ul[aria-expanded=true]{display:block}.nav:not(.nocollapse):not(.mobile) li{float:none;display:block}.nav:not(.nocollapse):not(.mobile) li ul{position:static}.nav:not(.nocollapse):not(.mobile) li li a,.nav:not(.nocollapse):not(.mobile) li li button{text-indent:10px}.nav:not(.nocollapse):not(.mobile) li li li a,.nav:not(.nocollapse):not(.mobile) li li li button{text-indent:20px}.nav:not(.nocollapse):not(.mobile) li[role=menu] ul{border:0}.nav:not(.nocollapse):not(.mobile) li[role=menu] ul>li:first-child>a,.nav:not(.nocollapse):not(.mobile) li[role=menu] ul>li:first-child>button{border-top-width:1px}.nav:not(.nocollapse):not(.mobile) li[role=menu]>a:after,.nav:not(.nocollapse):not(.mobile) li[role=menu]>button:after{float:none;content:"\f0d7"}.nav:not(.nocollapse):not(.mobile) li[role=menu][aria-pressed=true]>a:after,.nav:not(.nocollapse):not(.mobile) li[role=menu][aria-pressed=true]>button:after{content:"\f0d8"}.nav:not(.nocollapse):not(.mobile) a,.nav:not(.nocollapse):not(.mobile) button:not(.hamburger){display:block;width:100%;text-align:left}.nav:not(.nocollapse):not(.mobile).green li ul a,.nav:not(.nocollapse):not(.mobile).green li[role=menu] ul a,.nav:not(.nocollapse):not(.mobile).green a,.nav:not(.nocollapse):not(.mobile).green li ul button,.nav:not(.nocollapse):not(.mobile).green li[role=menu] ul button,.nav:not(.nocollapse):not(.mobile).green button{-o-border-radius:0;border-radius:0}.nav:not(.nocollapse):not(.mobile).blue li ul a,.nav:not(.nocollapse):not(.mobile).blue li[role=menu] ul a,.nav:not(.nocollapse):not(.mobile).blue a,.nav:not(.nocollapse):not(.mobile).blue li ul button,.nav:not(.nocollapse):not(.mobile).blue li[role=menu] ul button,.nav:not(.nocollapse):not(.mobile).blue button{-o-border-radius:0;border-radius:0}.nav:not(.nocollapse):not(.mobile).purple li ul a,.nav:not(.nocollapse):not(.mobile).purple li[role=menu] ul a,.nav:not(.nocollapse):not(.mobile).purple a,.nav:not(.nocollapse):not(.mobile).purple li ul button,.nav:not(.nocollapse):not(.mobile).purple li[role=menu] ul button,.nav:not(.nocollapse):not(.mobile).purple button{-o-border-radius:0;border-radius:0}.nav:not(.nocollapse):not(.mobile).yellow li ul a,.nav:not(.nocollapse):not(.mobile).yellow li[role=menu] ul a,.nav:not(.nocollapse):not(.mobile).yellow a,.nav:not(.nocollapse):not(.mobile).yellow li ul button,.nav:not(.nocollapse):not(.mobile).yellow li[role=menu] ul button,.nav:not(.nocollapse):not(.mobile).yellow button{-o-border-radius:0;border-radius:0}.nav:not(.nocollapse):not(.mobile).orange li ul a,.nav:not(.nocollapse):not(.mobile).orange li[role=menu] ul a,.nav:not(.nocollapse):not(.mobile).orange a,.nav:not(.nocollapse):not(.mobile).orange li ul button,.nav:not(.nocollapse):not(.mobile).orange li[role=menu] ul button,.nav:not(.nocollapse):not(.mobile).orange button{-o-border-radius:0;border-radius:0}.nav:not(.nocollapse):not(.mobile).red li ul a,.nav:not(.nocollapse):not(.mobile).red li[role=menu] ul a,.nav:not(.nocollapse):not(.mobile).red a,.nav:not(.nocollapse):not(.mobile).red li ul button,.nav:not(.nocollapse):not(.mobile).red li[role=menu] ul button,.nav:not(.nocollapse):not(.mobile).red button{-o-border-radius:0;border-radius:0}.nav:not(.nocollapse):not(.mobile).pink li ul a,.nav:not(.nocollapse):not(.mobile).pink li[role=menu] ul a,.nav:not(.nocollapse):not(.mobile).pink a,.nav:not(.nocollapse):not(.mobile).pink li ul button,.nav:not(.nocollapse):not(.mobile).pink li[role=menu] ul button,.nav:not(.nocollapse):not(.mobile).pink button{-o-border-radius:0;border-radius:0}.nav:not(.nocollapse):not(.mobile).turquoise li ul a,.nav:not(.nocollapse):not(.mobile).turquoise li[role=menu] ul a,.nav:not(.nocollapse):not(.mobile).turquoise a,.nav:not(.nocollapse):not(.mobile).turquoise li ul button,.nav:not(.nocollapse):not(.mobile).turquoise li[role=menu] ul button,.nav:not(.nocollapse):not(.mobile).turquoise button{-o-border-radius:0;border-radius:0}.nav:not(.nocollapse):not(.mobile).asphalt li ul a,.nav:not(.nocollapse):not(.mobile).asphalt li[role=menu] ul a,.nav:not(.nocollapse):not(.mobile).asphalt a,.nav:not(.nocollapse):not(.mobile).asphalt li ul button,.nav:not(.nocollapse):not(.mobile).asphalt li[role=menu] ul button,.nav:not(.nocollapse):not(.mobile).asphalt button{-o-border-radius:0;border-radius:0}.nav:not(.nocollapse):not(.mobile).charcoal li ul a,.nav:not(.nocollapse):not(.mobile).charcoal li[role=menu] ul a,.nav:not(.nocollapse):not(.mobile).charcoal a,.nav:not(.nocollapse):not(.mobile).charcoal li ul button,.nav:not(.nocollapse):not(.mobile).charcoal li[role=menu] ul button,.nav:not(.nocollapse):not(.mobile).charcoal button{-o-border-radius:0;border-radius:0}.nav:not(.nocollapse):not(.mobile).white li ul a,.nav:not(.nocollapse):not(.mobile).white li[role=menu] ul a,.nav:not(.nocollapse):not(.mobile).white a,.nav:not(.nocollapse):not(.mobile).white li ul button,.nav:not(.nocollapse):not(.mobile).white li[role=menu] ul button,.nav:not(.nocollapse):not(.mobile).white button{-o-border-radius:0;border-radius:0}.nav:not(.nocollapse):not(.mobile).black li ul a,.nav:not(.nocollapse):not(.mobile).black li[role=menu] ul a,.nav:not(.nocollapse):not(.mobile).black a,.nav:not(.nocollapse):not(.mobile).black li ul button,.nav:not(.nocollapse):not(.mobile).black li[role=menu] ul button,.nav:not(.nocollapse):not(.mobile).black button{-o-border-radius:0;border-radius:0}}@media (max-width:480px){.half:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.halves:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.third:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.thirds:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.fourth:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.fourths:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.fifth:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.fifths:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.sixth:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.sixths:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.seventh:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.sevenths:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.eighth:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.eighths:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.ninth:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.ninths:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.tenth:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.tenths:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.eleventh:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.elevenths:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.twelfth:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.twelfths:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile){width:100%!important;float:none;clear:both}.half.right-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.halves.right-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.third.right-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.thirds.right-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.third.right-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.thirds.right-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.fourth.right-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.fourths.right-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.fourth.right-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.fourths.right-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.fourth.right-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.fourths.right-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.fifth.right-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.fifths.right-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.fifth.right-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.fifths.right-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.fifth.right-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.fifths.right-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.fifth.right-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.fifths.right-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.sixth.right-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.sixths.right-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.sixth.right-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.sixths.right-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.sixth.right-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.sixths.right-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.sixth.right-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.sixths.right-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.sixth.right-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.sixths.right-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.seventh.right-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.sevenths.right-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.seventh.right-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.sevenths.right-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.seventh.right-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.sevenths.right-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.seventh.right-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.sevenths.right-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.seventh.right-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.sevenths.right-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.seventh.right-six:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.sevenths.right-six:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.eighth.right-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.eighths.right-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.eighth.right-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.eighths.right-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.eighth.right-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.eighths.right-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.eighth.right-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.eighths.right-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.eighth.right-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.eighths.right-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.eighth.right-six:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.eighths.right-six:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.eighth.right-seven:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.eighths.right-seven:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.ninth.right-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.ninths.right-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.ninth.right-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.ninths.right-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.ninth.right-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.ninths.right-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.ninth.right-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.ninths.right-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.ninth.right-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.ninths.right-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.ninth.right-six:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.ninths.right-six:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.ninth.right-seven:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.ninths.right-seven:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.ninth.right-eight:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.ninths.right-eight:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.tenth.right-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.tenths.right-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.tenth.right-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.tenths.right-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.tenth.right-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.tenths.right-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.tenth.right-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.tenths.right-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.tenth.right-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.tenths.right-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.tenth.right-six:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.tenths.right-six:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.tenth.right-seven:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.tenths.right-seven:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.tenth.right-eight:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.tenths.right-eight:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.tenth.right-nine:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.tenths.right-nine:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.eleventh.right-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.elevenths.right-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.eleventh.right-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.elevenths.right-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.eleventh.right-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.elevenths.right-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.eleventh.right-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.elevenths.right-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.eleventh.right-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.elevenths.right-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.eleventh.right-six:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.elevenths.right-six:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.eleventh.right-seven:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.elevenths.right-seven:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.eleventh.right-eight:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.elevenths.right-eight:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.eleventh.right-nine:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.elevenths.right-nine:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.eleventh.right-ten:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.elevenths.right-ten:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.twelfth.right-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.twelfths.right-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.twelfth.right-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.twelfths.right-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.twelfth.right-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.twelfths.right-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.twelfth.right-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.twelfths.right-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.twelfth.right-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.twelfths.right-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.twelfth.right-six:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.twelfths.right-six:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.twelfth.right-seven:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.twelfths.right-seven:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.twelfth.right-eight:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.twelfths.right-eight:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.twelfth.right-nine:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.twelfths.right-nine:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.twelfth.right-ten:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.twelfths.right-ten:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.twelfth.right-eleven:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.twelfths.right-eleven:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile){left:0}.half.left-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.halves.left-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.third.left-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.thirds.left-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.third.left-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.thirds.left-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.fourth.left-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.fourths.left-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.fourth.left-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.fourths.left-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.fourth.left-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.fourths.left-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.fifth.left-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.fifths.left-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.fifth.left-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.fifths.left-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.fifth.left-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.fifths.left-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.fifth.left-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.fifths.left-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.sixth.left-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.sixths.left-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.sixth.left-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.sixths.left-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.sixth.left-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.sixths.left-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.sixth.left-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.sixths.left-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.sixth.left-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.sixths.left-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.seventh.left-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.sevenths.left-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.seventh.left-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.sevenths.left-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.seventh.left-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.sevenths.left-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.seventh.left-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.sevenths.left-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.seventh.left-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.sevenths.left-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.seventh.left-six:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.sevenths.left-six:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.eighth.left-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.eighths.left-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.eighth.left-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.eighths.left-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.eighth.left-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.eighths.left-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.eighth.left-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.eighths.left-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.eighth.left-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.eighths.left-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.eighth.left-six:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.eighths.left-six:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.eighth.left-seven:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.eighths.left-seven:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.ninth.left-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.ninths.left-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.ninth.left-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.ninths.left-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.ninth.left-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.ninths.left-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.ninth.left-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.ninths.left-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.ninth.left-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.ninths.left-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.ninth.left-six:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.ninths.left-six:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.ninth.left-seven:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.ninths.left-seven:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.ninth.left-eight:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.ninths.left-eight:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.tenth.left-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.tenths.left-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.tenth.left-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.tenths.left-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.tenth.left-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.tenths.left-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.tenth.left-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.tenths.left-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.tenth.left-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.tenths.left-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.tenth.left-six:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.tenths.left-six:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.tenth.left-seven:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.tenths.left-seven:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.tenth.left-eight:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.tenths.left-eight:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.tenth.left-nine:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.tenths.left-nine:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.eleventh.left-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.elevenths.left-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.eleventh.left-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.elevenths.left-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.eleventh.left-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.elevenths.left-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.eleventh.left-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.elevenths.left-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.eleventh.left-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.elevenths.left-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.eleventh.left-six:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.elevenths.left-six:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.eleventh.left-seven:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.elevenths.left-seven:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.eleventh.left-eight:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.elevenths.left-eight:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.eleventh.left-nine:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.elevenths.left-nine:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.eleventh.left-ten:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.elevenths.left-ten:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.twelfth.left-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.twelfths.left-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.twelfth.left-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.twelfths.left-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.twelfth.left-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.twelfths.left-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.twelfth.left-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.twelfths.left-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.twelfth.left-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.twelfths.left-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.twelfth.left-six:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.twelfths.left-six:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.twelfth.left-seven:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.twelfths.left-seven:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.twelfth.left-eight:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.twelfths.left-eight:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.twelfth.left-nine:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.twelfths.left-nine:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.twelfth.left-ten:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.twelfths.left-ten:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.twelfth.left-eleven:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.twelfths.left-eleven:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile){right:0}.half.skip-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.halves.skip-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.third.skip-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.thirds.skip-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.third.skip-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.thirds.skip-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.fourth.skip-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.fourths.skip-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.fourth.skip-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.fourths.skip-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.fourth.skip-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.fourths.skip-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.fifth.skip-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.fifths.skip-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.fifth.skip-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.fifths.skip-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.fifth.skip-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.fifths.skip-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.fifth.skip-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.fifths.skip-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.sixth.skip-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.sixths.skip-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.sixth.skip-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.sixths.skip-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.sixth.skip-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.sixths.skip-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.sixth.skip-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.sixths.skip-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.sixth.skip-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.sixths.skip-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.seventh.skip-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.sevenths.skip-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.seventh.skip-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.sevenths.skip-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.seventh.skip-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.sevenths.skip-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.seventh.skip-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.sevenths.skip-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.seventh.skip-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.sevenths.skip-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.seventh.skip-six:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.sevenths.skip-six:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.eighth.skip-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.eighths.skip-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.eighth.skip-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.eighths.skip-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.eighth.skip-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.eighths.skip-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.eighth.skip-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.eighths.skip-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.eighth.skip-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.eighths.skip-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.eighth.skip-six:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.eighths.skip-six:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.eighth.skip-seven:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.eighths.skip-seven:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.ninth.skip-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.ninths.skip-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.ninth.skip-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.ninths.skip-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.ninth.skip-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.ninths.skip-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.ninth.skip-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.ninths.skip-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.ninth.skip-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.ninths.skip-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.ninth.skip-six:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.ninths.skip-six:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.ninth.skip-seven:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.ninths.skip-seven:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.ninth.skip-eight:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.ninths.skip-eight:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.tenth.skip-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.tenths.skip-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.tenth.skip-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.tenths.skip-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.tenth.skip-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.tenths.skip-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.tenth.skip-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.tenths.skip-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.tenth.skip-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.tenths.skip-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.tenth.skip-six:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.tenths.skip-six:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.tenth.skip-seven:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.tenths.skip-seven:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.tenth.skip-eight:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.tenths.skip-eight:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.tenth.skip-nine:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.tenths.skip-nine:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.eleventh.skip-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.elevenths.skip-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.eleventh.skip-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.elevenths.skip-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.eleventh.skip-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.elevenths.skip-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.eleventh.skip-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.elevenths.skip-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.eleventh.skip-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.elevenths.skip-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.eleventh.skip-six:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.elevenths.skip-six:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.eleventh.skip-seven:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.elevenths.skip-seven:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.eleventh.skip-eight:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.elevenths.skip-eight:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.eleventh.skip-nine:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.elevenths.skip-nine:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.eleventh.skip-ten:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.elevenths.skip-ten:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.twelfth.skip-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.twelfths.skip-one:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.twelfth.skip-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.twelfths.skip-two:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.twelfth.skip-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.twelfths.skip-three:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.twelfth.skip-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.twelfths.skip-four:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.twelfth.skip-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.twelfths.skip-five:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.twelfth.skip-six:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.twelfths.skip-six:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.twelfth.skip-seven:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.twelfths.skip-seven:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.twelfth.skip-eight:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.twelfths.skip-eight:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.twelfth.skip-nine:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.twelfths.skip-nine:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.twelfth.skip-ten:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.twelfths.skip-ten:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.twelfth.skip-eleven:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile),.twelfths.skip-eleven:not(.mobile):not(.one-up-mobile):not(.two-up-mobile):not(.three-up-mobile):not(.four-up-mobile):not(.five-up-mobile){margin-left:0}.desktop-only{display:none!important}.ipad-only{display:none!important}.small-tablet-only{display:none!important}.mobile-only{display:block!important}.hide-on-small-tablet{display:block!important}.hide-on-mobile{display:none!important}.no-border-mobile{border:0}.small-mobile{font-size:.8em}.medium-mobile{font-size:1em}.large-mobile{font-size:1.5em}.zero-mobile.padded,.zero-mobile.half-padded,.zero-mobile.double-padded,.zero-mobile.triple-padded{padding:0!important;margin:0!important}.zero-mobile.pad-top,.zero-mobile.half-pad-top,.zero-mobile.double-pad-top,.zero-mobile.triple-pad-top{padding:0!important;margin:0!important}.zero-mobile.pad-right,.zero-mobile.half-pad-right,.zero-mobile.double-pad-right,.zero-mobile.triple-pad-right{padding:0!important;margin:0!important}.zero-mobile.pad-bottom,.zero-mobile.half-pad-bottom,.zero-mobile.double-pad-bottom,.zero-mobile.triple-pad-bottom{padding:0!important;margin:0!important}.zero-mobile.pad-left,.zero-mobile.half-pad-left,.zero-mobile.double-pad-left,.zero-mobile.triple-pad-left{padding:0!important;margin:0!important}.zero-mobile.gapped,.zero-mobile.half-gapped,.zero-mobile.double-gapped,.zero-mobile.triple-gapped{padding:0!important;margin:0!important}.zero-mobile.gap-top,.zero-mobile.half-gap-top,.zero-mobile.double-gap-top,.zero-mobile.triple-gap-top{padding:0!important;margin:0!important}.zero-mobile.gap-right,.zero-mobile.half-gap-right,.zero-mobile.double-gap-right,.zero-mobile.triple-gap-right{padding:0!important;margin:0!important}.zero-mobile.gap-bottom,.zero-mobile.half-gap-bottom,.zero-mobile.double-gap-bottom,.zero-mobile.triple-gap-bottom{padding:0!important;margin:0!important}.zero-mobile.gap-left,.zero-mobile.half-gap-left,.zero-mobile.double-gap-left,.zero-mobile.triple-gap-left{padding:0!important;margin:0!important}.padded-mobile{padding:10px}.pad-top-mobile{padding-top:10px}.pad-right-mobile{padding-right:10px}.pad-bottom-mobile{padding-bottom:10px}.pad-left-mobile{padding-left:10px}.no-padding-mobile.padded,.no-padding-mobile.half-padded,.no-padding-mobile.double-padded,.no-padding-mobile.triple-padded,.no-padded-mobile.padded,.no-padded-mobile.half-padded,.no-padded-mobile.double-padded,.no-padded-mobile.triple-padded,.no-pad-mobile.padded,.no-pad-mobile.half-padded,.no-pad-mobile.double-padded,.no-pad-mobile.triple-padded{padding:0!important}.no-padding-mobile.pad-top,.no-padding-mobile.half-pad-top,.no-padding-mobile.double-pad-top,.no-padding-mobile.triple-pad-top,.no-padded-mobile.pad-top,.no-padded-mobile.half-pad-top,.no-padded-mobile.double-pad-top,.no-padded-mobile.triple-pad-top,.no-pad-mobile.pad-top,.no-pad-mobile.half-pad-top,.no-pad-mobile.double-pad-top,.no-pad-mobile.triple-pad-top{padding:0!important}.no-padding-mobile.pad-right,.no-padding-mobile.half-pad-right,.no-padding-mobile.double-pad-right,.no-padding-mobile.triple-pad-right,.no-padded-mobile.pad-right,.no-padded-mobile.half-pad-right,.no-padded-mobile.double-pad-right,.no-padded-mobile.triple-pad-right,.no-pad-mobile.pad-right,.no-pad-mobile.half-pad-right,.no-pad-mobile.double-pad-right,.no-pad-mobile.triple-pad-right{padding:0!important}.no-padding-mobile.pad-bottom,.no-padding-mobile.half-pad-bottom,.no-padding-mobile.double-pad-bottom,.no-padding-mobile.triple-pad-bottom,.no-padded-mobile.pad-bottom,.no-padded-mobile.half-pad-bottom,.no-padded-mobile.double-pad-bottom,.no-padded-mobile.triple-pad-bottom,.no-pad-mobile.pad-bottom,.no-pad-mobile.half-pad-bottom,.no-pad-mobile.double-pad-bottom,.no-pad-mobile.triple-pad-bottom{padding:0!important}.no-padding-mobile.pad-left,.no-padding-mobile.half-pad-left,.no-padding-mobile.double-pad-left,.no-padding-mobile.triple-pad-left,.no-padded-mobile.pad-left,.no-padded-mobile.half-pad-left,.no-padded-mobile.double-pad-left,.no-padded-mobile.triple-pad-left,.no-pad-mobile.pad-left,.no-pad-mobile.half-pad-left,.no-pad-mobile.double-pad-left,.no-pad-mobile.triple-pad-left{padding:0!important}.no-pad-top-mobile{padding-top:0}.no-pad-right-mobile{padding-right:0}.no-pad-bottom-mobile{padding-bottom:0}.no-pad-left-mobile{padding-left:0}.gapped-mobile{margin:10px}.gap-top-mobile{margin-top:10px}.gap-right-mobile{margin-right:10px}.gap-bottom-mobile{margin-bottom:10px}.gap-left-mobile{margin-left:10px}.no-margin-mobile.gapped,.no-margin-mobile.half-gapped,.no-margin-mobile.double-gapped,.no-margin-mobile.triple-gapped,.no-gapped-mobile.gapped,.no-gapped-mobile.half-gapped,.no-gapped-mobile.double-gapped,.no-gapped-mobile.triple-gapped,.no-gap-mobile.gapped,.no-gap-mobile.half-gapped,.no-gap-mobile.double-gapped,.no-gap-mobile.triple-gapped{margin:0!important}.no-margin-mobile.gap-top,.no-margin-mobile.half-gap-top,.no-margin-mobile.double-gap-top,.no-margin-mobile.triple-gap-top,.no-gapped-mobile.gap-top,.no-gapped-mobile.half-gap-top,.no-gapped-mobile.double-gap-top,.no-gapped-mobile.triple-gap-top,.no-gap-mobile.gap-top,.no-gap-mobile.half-gap-top,.no-gap-mobile.double-gap-top,.no-gap-mobile.triple-gap-top{margin:0!important}.no-margin-mobile.gap-right,.no-margin-mobile.half-gap-right,.no-margin-mobile.double-gap-right,.no-margin-mobile.triple-gap-right,.no-gapped-mobile.gap-right,.no-gapped-mobile.half-gap-right,.no-gapped-mobile.double-gap-right,.no-gapped-mobile.triple-gap-right,.no-gap-mobile.gap-right,.no-gap-mobile.half-gap-right,.no-gap-mobile.double-gap-right,.no-gap-mobile.triple-gap-right{margin:0!important}.no-margin-mobile.gap-bottom,.no-margin-mobile.half-gap-bottom,.no-margin-mobile.double-gap-bottom,.no-margin-mobile.triple-gap-bottom,.no-gapped-mobile.gap-bottom,.no-gapped-mobile.half-gap-bottom,.no-gapped-mobile.double-gap-bottom,.no-gapped-mobile.triple-gap-bottom,.no-gap-mobile.gap-bottom,.no-gap-mobile.half-gap-bottom,.no-gap-mobile.double-gap-bottom,.no-gap-mobile.triple-gap-bottom{margin:0!important}.no-margin-mobile.gap-left,.no-margin-mobile.half-gap-left,.no-margin-mobile.double-gap-left,.no-margin-mobile.triple-gap-left,.no-gapped-mobile.gap-left,.no-gapped-mobile.half-gap-left,.no-gapped-mobile.double-gap-left,.no-gapped-mobile.triple-gap-left,.no-gap-mobile.gap-left,.no-gap-mobile.half-gap-left,.no-gap-mobile.double-gap-left,.no-gap-mobile.triple-gap-left{margin:0!important}.no-gap-top-mobile{margin-top:0}.no-gap-right-mobile{margin-right:0}.no-gap-bottom-mobile{margin-bottom:0}.no-gap-left-mobile{margin-left:0}.offset-mobile{margin:-10px}.offset-top-mobile{margin-top:-10px}.offset-right-mobile{margin-right:-10px}.offset-bottom-mobile{margin-bottom:-10px}.offset-left-mobile{margin-left:-10px}.no-offset-top-mobile{margin-top:0}.no-offset-right-mobile{margin-right:0}.no-offset-bottom-mobile{margin-bottom:0}.no-offset-left-mobile{margin-left:0}.half-padded-mobile{padding:5px}.double-padded-mobile{padding:20px}.triple-padded-mobile{padding:30px}.half-pad-top-mobile{padding-top:5px}.double-pad-top-mobile{padding-top:20px}.triple-pad-top-mobile{padding-top:30px}.half-pad-right-mobile{padding-right:5px}.double-pad-right-mobile{padding-right:20px}.triple-pad-right-mobile{padding-right:30px}.half-pad-bottom-mobile{padding-bottom:5px}.double-pad-bottom-mobile{padding-bottom:20px}.triple-pad-bottom-mobile{padding-bottom:30px}.half-pad-left-mobile{padding-left:5px}.double-pad-left-mobile{padding-left:20px}.triple-pad-left-mobile{padding-left:30px}.half-gapped-mobile{margin:5px}.double-gapped-mobile{margin:20px}.triple-gapped-mobile{margin:30px}.half-gap-top-mobile{margin-top:5px}.double-gap-top-mobile{margin-top:20px}.triple-gap-top-mobile{margin-top:30px}.half-gap-right-mobile{margin-right:5px}.double-gap-right-mobile{margin-right:20px}.triple-gap-right-mobile{margin-right:30px}.half-gap-bottom-mobile{margin-bottom:5px}.double-gap-bottom-mobile{margin-bottom:20px}.triple-gap-bottom-mobile{margin-bottom:30px}.half-gap-left-mobile{margin-left:5px}.double-gap-left-mobile{margin-left:20px}.triple-gap-left-mobile{margin-left:30px}.half-offset-mobile{margin:-5px}.double-offset-mobile{margin:-20px}.triple-offset-mobile{margin:-30px}.half-offset-top-mobile{margin-top:-5px}.double-offset-top-mobile{margin-top:-20px}.triple-offset-top-mobile{margin-top:-30px}.half-offset-right-mobile{margin-right:-5px}.double-offset-right-mobile{margin-right:-20px}.triple-offset-right-mobile{margin-right:-30px}.half-offset-bottom-mobile{margin-bottom:-5px}.double-offset-bottom-mobile{margin-bottom:-20px}.triple-offset-bottom-mobile{margin-bottom:-30px}.half-offset-left-mobile{margin-left:-5px}.double-offset-left-mobile{margin-left:-20px}.triple-offset-left-mobile{margin-left:-30px}.fixed-mobile{position:fixed}.fixed-top-mobile{position:fixed;top:0}.fixed-right-mobile{position:fixed;right:0}.fixed-bottom-mobile{position:fixed;bottom:0}.fixed-left-mobile{position:fixed;left:0}.fixed-middle-mobile{position:fixed;top:50%}.fixed-center-mobile{position:fixed;left:50%}.absolute-mobile{position:absolute}.absolute-top-mobile{position:absolute;top:0}.absolute-right-mobile{position:absolute;right:0}.absolute-bottom-mobile{position:absolute;bottom:0}.absolute-left-mobile{position:absolute;left:0}.absolute-middle-mobile{position:absolute;top:50%}.absolute-center-mobile{position:absolute;left:50%}.static-mobile{position:static}.inline-mobile{display:inline}.block-mobile{display:block}.clear-mobile{clear:both}.clear-mobile:after{content:" ";display:block;height:0;font-size:0;clear:both;visibility:hidden}.center-mobile{display:block;margin:0 auto}.pull-right-mobile{float:right}.pull-left-mobile{float:left}.pull-none-mobile{float:none}.bordered-mobile{border:1px solid #ddd}.border-top-mobile{border-top:1px solid #ddd}.border-right-mobile{border-right:1px solid #ddd}.border-bottom-mobile{border-bottom:1px solid #ddd}.border-left-mobile{border-left:1px solid #ddd}.align-top-mobile{display:table-cell;vertical-align:top}.align-right-mobile{text-align:right}.align-bottom-mobile{display:table-cell;vertical-align:bottom}.align-left-mobile{text-align:left}.align-center-mobile{text-align:center}.justify-mobile{text-align:justify}.truncate-mobile{overflow:hidden;white-space:nowrap;text-overflow:ellipsis}.rotate-90-mobile{display:inline-block;width:auto;-webkit-transform:rotate(90deg);transform:rotate(90deg)}.rotate-90-ctr-mobile{display:inline-block;width:auto;-webkit-transform:rotate(-90deg);transform:rotate(-90deg)}.rotate-180-mobile{display:inline-block;width:auto;-webkit-transform:rotate(180deg);transform:rotate(180deg)}.rotate-180-ctr-mobile{display:inline-block;width:auto;-webkit-transform:rotate(-180deg);transform:rotate(-180deg)}.rotate-270-mobile{display:inline-block;width:auto;-webkit-transform:rotate(270deg);transform:rotate(270deg)}.rotate-270-ctr-mobile{display:inline-block;width:auto;-webkit-transform:rotate(-270deg);transform:rotate(-270deg)}.rotate-360-mobile{display:inline-block;width:auto;-webkit-transform:rotate(360deg);transform:rotate(360deg)}.rotate-360-mobile{display:inline-block;width:auto;-webkit-transform:rotate(-360deg);transform:rotate(-360deg)}.one-column-mobile,.one-columns-mobile{-webkit-column-count:1;-moz-column-count:1;column-count:1}.two-column-mobile,.two-columns-mobile{-webkit-column-count:2;-moz-column-count:2;column-count:2}.three-column-mobile,.three-columns-mobile{-webkit-column-count:3;-moz-column-count:3;column-count:3}.four-column-mobile,.four-columns-mobile{-webkit-column-count:4;-moz-column-count:4;column-count:4}.five-column-mobile,.five-columns-mobile{-webkit-column-count:5;-moz-column-count:5;column-count:5}.nav:not(.nocollapse):before{float:left;display:inline-block;content:attr(title);padding:.25em .5em;font-weight:700}.nav:not(.nocollapse) button.hamburger{display:inline-block}.nav:not(.nocollapse) button.hamburger:before{font-family:FontAwesome;font-weight:400;font-style:normal;text-decoration:inherit;-webkit-font-smoothing:antialiased;*margin-right:.3em;content:"\f03a"}.nav:not(.nocollapse) button.hamburger[aria-expanded=true]:before{content:"\f151"}.nav:not(.nocollapse)>ul{display:none}.nav:not(.nocollapse)>ul>li>a,.nav:not(.nocollapse)>ul>li>button{border-top-width:1px}.nav:not(.nocollapse)>ul[aria-expanded=true]{display:block}.nav:not(.nocollapse) li{float:none;display:block}.nav:not(.nocollapse) li ul{position:static}.nav:not(.nocollapse) li li a,.nav:not(.nocollapse) li li button{text-indent:10px}.nav:not(.nocollapse) li li li a,.nav:not(.nocollapse) li li li button{text-indent:20px}.nav:not(.nocollapse) li[role=menu] ul{border:0}.nav:not(.nocollapse) li[role=menu] ul>li:first-child>a,.nav:not(.nocollapse) li[role=menu] ul>li:first-child>button{border-top-width:1px}.nav:not(.nocollapse) li[role=menu]>a:after,.nav:not(.nocollapse) li[role=menu]>button:after{float:none;content:"\f0d7"}.nav:not(.nocollapse) li[role=menu][aria-pressed=true]>a:after,.nav:not(.nocollapse) li[role=menu][aria-pressed=true]>button:after{content:"\f0d8"}.nav:not(.nocollapse) a,.nav:not(.nocollapse) button:not(.hamburger){display:block;width:100%;text-align:left}.nav:not(.nocollapse).green li ul a,.nav:not(.nocollapse).green li[role=menu] ul a,.nav:not(.nocollapse).green a,.nav:not(.nocollapse).green li ul button,.nav:not(.nocollapse).green li[role=menu] ul button,.nav:not(.nocollapse).green button{-o-border-radius:0;border-radius:0}.nav:not(.nocollapse).blue li ul a,.nav:not(.nocollapse).blue li[role=menu] ul a,.nav:not(.nocollapse).blue a,.nav:not(.nocollapse).blue li ul button,.nav:not(.nocollapse).blue li[role=menu] ul button,.nav:not(.nocollapse).blue button{-o-border-radius:0;border-radius:0}.nav:not(.nocollapse).purple li ul a,.nav:not(.nocollapse).purple li[role=menu] ul a,.nav:not(.nocollapse).purple a,.nav:not(.nocollapse).purple li ul button,.nav:not(.nocollapse).purple li[role=menu] ul button,.nav:not(.nocollapse).purple button{-o-border-radius:0;border-radius:0}.nav:not(.nocollapse).yellow li ul a,.nav:not(.nocollapse).yellow li[role=menu] ul a,.nav:not(.nocollapse).yellow a,.nav:not(.nocollapse).yellow li ul button,.nav:not(.nocollapse).yellow li[role=menu] ul button,.nav:not(.nocollapse).yellow button{-o-border-radius:0;border-radius:0}.nav:not(.nocollapse).orange li ul a,.nav:not(.nocollapse).orange li[role=menu] ul a,.nav:not(.nocollapse).orange a,.nav:not(.nocollapse).orange li ul button,.nav:not(.nocollapse).orange li[role=menu] ul button,.nav:not(.nocollapse).orange button{-o-border-radius:0;border-radius:0}.nav:not(.nocollapse).red li ul a,.nav:not(.nocollapse).red li[role=menu] ul a,.nav:not(.nocollapse).red a,.nav:not(.nocollapse).red li ul button,.nav:not(.nocollapse).red li[role=menu] ul button,.nav:not(.nocollapse).red button{-o-border-radius:0;border-radius:0}.nav:not(.nocollapse).pink li ul a,.nav:not(.nocollapse).pink li[role=menu] ul a,.nav:not(.nocollapse).pink a,.nav:not(.nocollapse).pink li ul button,.nav:not(.nocollapse).pink li[role=menu] ul button,.nav:not(.nocollapse).pink button{-o-border-radius:0;border-radius:0}.nav:not(.nocollapse).turquoise li ul a,.nav:not(.nocollapse).turquoise li[role=menu] ul a,.nav:not(.nocollapse).turquoise a,.nav:not(.nocollapse).turquoise li ul button,.nav:not(.nocollapse).turquoise li[role=menu] ul button,.nav:not(.nocollapse).turquoise button{-o-border-radius:0;border-radius:0}.nav:not(.nocollapse).asphalt li ul a,.nav:not(.nocollapse).asphalt li[role=menu] ul a,.nav:not(.nocollapse).asphalt a,.nav:not(.nocollapse).asphalt li ul button,.nav:not(.nocollapse).asphalt li[role=menu] ul button,.nav:not(.nocollapse).asphalt button{-o-border-radius:0;border-radius:0}.nav:not(.nocollapse).charcoal li ul a,.nav:not(.nocollapse).charcoal li[role=menu] ul a,.nav:not(.nocollapse).charcoal a,.nav:not(.nocollapse).charcoal li ul button,.nav:not(.nocollapse).charcoal li[role=menu] ul button,.nav:not(.nocollapse).charcoal button{-o-border-radius:0;border-radius:0}.nav:not(.nocollapse).white li ul a,.nav:not(.nocollapse).white li[role=menu] ul a,.nav:not(.nocollapse).white a,.nav:not(.nocollapse).white li ul button,.nav:not(.nocollapse).white li[role=menu] ul button,.nav:not(.nocollapse).white button{-o-border-radius:0;border-radius:0}.nav:not(.nocollapse).black li ul a,.nav:not(.nocollapse).black li[role=menu] ul a,.nav:not(.nocollapse).black a,.nav:not(.nocollapse).black li ul button,.nav:not(.nocollapse).black li[role=menu] ul button,.nav:not(.nocollapse).black button{-o-border-radius:0;border-radius:0}}